<?php
/**
 * IdP Setting Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */

class IdpSettingsController extends AppController {
	var $name = 'IdpSettings';
	var $uses = array('Transaction', 'SpGroup', 'GroupAdminSpSort', 'Organization',
			'IdpAdministrator', 'Mygroup');
	var $helpers = array('Javascript');

	function beforeFilter() {
		parent::beforeFilter();
		$groupid = $this->_getGroupIdFromURL();
		$this->_checkAccessible($groupid);
		$this->_validateAdminGroup($groupid);
	}

	protected function _checkAccessible($groupid) {
		if (is_null($groupid) || !$this->Group->isIdPGroup($groupid)) {
			$this->cakeError('notFound', array());
			return;
		}
		parent::_checkAccessible($groupid);
	}

	/**
	 * サービス表示順を表示
	 */
	function view($groupid) {
		$entityid = $this->getIdentityProvider();
		$sortList = $this->GroupAdminSpSort->findSpIdListByIdp($groupid);
		$spcList = $this->SpGroup->findNameOrderName();
		if (empty($spcList)) {
			$this->set('spcList', array());
			return;
		}

		// 未設定時はSPコネクタの名前順で表示
		if (empty($sortList)) {
			$this->set('spcList', $spcList);
		}

		// GroupAdminSpSortの順番に$spcListを並び変える
		$tmpList = array();
		$notDefineNo = 10000;
		foreach ($spcList as $spcid => $name) {
			if (isset($sortList[ $spcid ])) {
				$orderNumber = $sortList[ $spcid ];
				$tmpList[$orderNumber] = array('spcid' => $spcid, 'name' => $name);
			} else {
				$tmpList[$notDefineNo++] = array('spcid' => $spcid, 'name' => $name);
			}
		}
		ksort($tmpList);
		$changedSpcList = array();
		foreach ($tmpList as $no => $val) {
			$changedSpcList[ $val['spcid'] ] = $val['name'];
		}
		$this->set('spcList', $changedSpcList);
		$this->set('groupid', $groupid);
	}

// InternalApiControllerに移動
//	/**
//	 * 表示順を保存します。
//	 */
//	function save() {
//		$this->validateAccess();
//		$sortedSpcidList = $this->params['form']['result'];
//
//		$this->Transaction->begin();
//		$entityid = $this->getIdentityProvider();
//		$result = $this->GroupAdminSpSort->updateAll($entityid, $sortedSpcidList, $this->getAccountID());
//		if (!$result) {
//			$this->Transaction->rollback();
//			$this->cakeError('systemError');
//			return;
//		}
//
//		$this->Transaction->commit();
//		$this->setFlash(__("message.idp.admin.sorterd", true));
//		$this->redirect('/mygroups/view');
//	}
//
//	function validateAccess() {
//		$access = $this->IdpAdministrator->isAdmin(
//			$this->getEPPN(), $this->getEPTID(), $this->getIdentityProvider());
//		if (!$access) {
//			$this->cakeError('cannotAccess', array('url' => $this->here));
//			return;
//		}
//	}
}
?>