<?php
/**
 * Manage Agencies Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Controller', 'Manages');

class ManageAgenciesController extends ManagesController {
	var $name = 'ManageAgencies';
	var $uses = array('Group','GroupAgency','Account','Mygroup','AccountEppn',
		'AddGroup', 'AgreementIsmemberof', 'Agreement',
		'ProvideAttribute', 'Transaction', 'AgreementEntrustment', 'Invite');
	var $components = array('IdpTranslator', 'AttributeOperator', 'GroupInvite',
				'Csrf', 'TakerAgreement');
	var $helpers = array('Javascript');

	function beforeFilter() {
		if (!Configure::read('agency')) {
			$this->cakeError('notFound', array());
			return;
		}

		$this->Csrf->validatePost = false;
		parent::beforeFilter();
	}

	/**
	 * 代理者入力の表示
	 * メールアドレスでメンバーの追加・削除を行う
	 */
	function agency() {
		$this->layout = 'iframe';
		switch ($this->getSubmitName()) {
			case 'add':
				if (empty($this->data)) {
					break;
				}
				$result = $this->invite($this->data['ManageAgency']['mail'], $this->data['ManageAgency']['forcibleAdmission']);
				if ($result) {
					$url = '';
					if (isset($this->data['ManageAgency']['forcibleAdmission'])
						&& $this->data['ManageAgency']['forcibleAdmission']) {
						$url = '?f=1';
					}
					$this->redirect('/manage_agencies/agency'.$url);
				}
				break;
			case 'delete':
				if (empty($this->data)) {
					break;
				}
				$this->delete();
				break;
			default:
				if (isset($this->params['url']['f']) && $this->params['url']['f']) {
					$this->data = array('ManageAgency' => array('forcibleAdmission' => true));
				}
				break;
		}

		$forcibleAdmission = $this->GroupInvite->canForce();
		$this->set('forcibleAdmission', $forcibleAdmission);
		$this->show("mail");
	}

	/**
	 * SPからグループKEYを指定し、
	 * そのグループのメンバー追加（招待）・削除を行う。
	 */
	public function group($groupKey) {
		$this->layout = 'iframe';

		// 管理者判定をする
		$group = $this->Group->findByGroupKey($groupKey);
		if (empty($group)) {
			$this->cakeError('notFound', array());
		}
		$groupId = $group['Group']['id'];
		$isAdmin = $this->Mygroup->isAdmin($this->getAccountID(), $groupId);
		if (!$isAdmin) {
			$this->cakeError('notFound', array());
		}

		switch ($this->getSubmitName()) {
			case 'add':
				if (empty($this->data)) {
					break;
				}
				$result = $this->invite($this->data['ManageAgency']['mail'],
					$this->data['ManageAgency']['forcibleAdmission'], $groupKey);
				if ($result) {
					$url = '';
					if (isset($this->data['ManageAgency']['forcibleAdmission']) && $this->data['ManageAgency']['forcibleAdmission']) {
						$url = '?f=1';
					}
					$this->redirect('/manage_agencies/group/'.$groupKey.$url);
				}
				break;
			case 'delete':
				if (empty($this->data)) {
					break;
				}
				$this->deleteGroup($groupId);
				break;
			default:
				if (isset($this->params['url']['f']) && $this->params['url']['f']) {
					$this->data = array('ManageAgency' => array('forcibleAdmission' => true));
				}
				break;
		}

		$forcibleAdmission = $this->GroupInvite->canForce();
		$this->set('forcibleAdmission', $forcibleAdmission);
		$this->set('groupKey', $groupKey);
		$this->set('selfAcccountId', $this->getAccountID());
		$this->showGroup($groupId);
		$this->render('agency_group');
	}

	private function showGroup($groupid) {
		$members = $this->getGroupMembers($groupid);
		$this->removeUnnecessaryLanguage($members);
		$this->IdpTranslator->setIdP($members, $this->getLanguage());
		$this->set('members', $members);
	}

	/**
	 * 京都大学用代理入力
	 */
	function agency_kyoto_u_ac_jp() {
		$this->layout = 'iframe';
		switch ($this->getSubmitName()) {
			case 'add':
				if (empty($this->data)) {
					break;
				}
				if ($this->joinByLocalId($this->data['ManageAgency']['spsid'])) {
					$this->Session->setFlash(__("グループへ追加しました。", true), 'message');
					$this->redirect('/manage_agencies/agency_kyoto_u_ac_jp');
				}
				break;
			case 'delete':
				if (empty($this->data)) {
					break;
				}
				$this->delete();
				break;
			default:
				break;
		}
		$this->show("local_id");
	}

	/**
	 * 京都大学用代理入力
	 */
	function agency_kyoto_u_ac_jp_multi() {
		// システム管理者のみがアクセス可能
		if (!$this->isSuperManager()) {
			$this->cakeError('cannotAccess', array('url' => $this->here));
			return;
		}
		$this->layout = 'iframe';
		switch ($this->getSubmitName()) {
			case 'add':
				if (empty($this->data)) {
					break;
				}
				if (empty($this->data['ManageAgency']['spsid'])) {
					$this->Session->setFlash(__("データが入力されていません。", true), 'message');
					$this->redirect('/manage_agencies/agency_kyoto_u_ac_jp_multi');
				}
				if ($this->_joinMultiByString($this->data['ManageAgency']['spsid'])) {
					$this->Session->setFlash(__("グループへの参加が完了しました。", true), 'message');
					$this->redirect('/manage_agencies/agency_kyoto_u_ac_jp_multi');
				}
				break;
			default:
				break;
		}

		$mygroups = $this->Mygroup->find('all', array(
				'fields' => array('id', 'account_id', 'groupid', 'admin',
					'(select name from accounts where id = account_id) as name',
					'(select local_id from account_eppns e where e.account_id = Mygroup.account_id) as localid'
					),
				'order' => array('groupid', 'admin DESC', 'id')));
		$list = array();
		foreach ($mygroups as $mygroup) {
			$groupid = $mygroup['Mygroup']['groupid'];
			// superグループは除外
			if ($groupid == 1) {
				continue;
			}
			if (empty($list[ $groupid ])) {
				$list[ $groupid ] = array();
			}
			array_push($list[ $groupid ], $mygroup[0]['name'].'('.$mygroup[0]['localid'].')');
		}
		$memberList = array();
		foreach ($list as $names) {
			if (count($names) <= 1) {
				continue;
			}
			array_push($memberList, $names);
		}
		$this->set('memberList', $memberList);
//debug($memberList);
//debug($mygroups);
	}

	private function _joinMultiByString($localIdString) {
		$localIdList = $this->splitLocalId($localIdString);
		if (!$this->validateLocalIds($localIdList)) {
			return false;
		}

		$this->_joinMulti($localIdList);
		return true;
	}

	private function _joinMulti($localIdList) {
		$accountEppns = $this->_findAccountEppnsByLocalIdList($localIdList);
		foreach ($localIdList as $localIds) {
			$adminLocalId = array_shift($localIds);
			$accountIds = array();
			foreach ($localIds as $localId) {
				$accountIds[] = $accountEppns[$localId]['account_id'];
			}

			$adminAccountEppn = $accountEppns[$adminLocalId];
			$groupKey = $adminAccountEppn['eppn'];
			$accountId = $adminAccountEppn['account_id'];
			$name = $adminAccountEppn['eppn'];
			$entityId = $adminAccountEppn['entityid'];
			$this->_createUnlessAgentGroup($groupKey, $accountId, $name, $entityId);

			foreach ($accountIds as $accountId) {
				$result = $this->_joinGroup($accountId, $groupKey);
			}
		}
	}

	private function _getAllLocalIds($localIdList) {
		$allLocalIds = array();
		foreach ($localIdList as $localIds) {
			$allLocalIds = array_merge($allLocalIds, $localIds);
		}
		return array_unique($allLocalIds);
	}

	private function _findAccountEppnsByLocalIdList($localIdList) {
		$allLocalIds = $this->_getAllLocalIds($localIdList);

		$accountEppns = $this->AccountEppn->find('all', array(
			'fields' => array('local_id', 'account_id', 'eppn', 'entityid'),
			'conditions' => array('local_id' => $allLocalIds)
		));
		if (!$accountEppns) {
			throw new Exception('account not found');
		}

		$results = array();
		foreach ($accountEppns as $accountEppn) {
			$results[$accountEppn['AccountEppn']['local_id']] = $accountEppn['AccountEppn'];
		}
		return $results;
	}

	private function splitLocalId($localIdString) {
		$localIdList = array();
		$lines = preg_split('/\r\n|\r|\n/', $localIdString);
		foreach ($lines as $i => $line) {
			$line = trim($line);
			if ($line === '') {
				continue;
			}
			$ids = preg_split('/\s+/', $line);
			$localIdList[$i+1] = $ids;
		}
		return $localIdList;
	}

	private function getNotExistLocalIds($localIds) {
		$accountEppns = $this->AccountEppn->find('all', array(
			'fields' => array('local_id'),
			'conditions' => array('local_id' => $localIds)
		));
		if ($accountEppns) {
			foreach ($accountEppns as $accountEppn) {
				$key = array_search($accountEppn['AccountEppn']['local_id'], $localIds);
				if ($key !== false) {
					unset($localIds[$key]);
				}
			}
		}
		return $localIds;
	}

	private function validateLocalIds($localIdList) {
		$errors = array();
		$admin_ids = array();
		foreach ($localIdList as $i => $ids) {
			if (count($ids) < 2) {
				$errors[$i] = 'IDが指定されていません。';
				continue;
			}
			if (isset($admin_ids[$ids[0]])) {
				$errors[$i] = $admin_ids[$ids[0]].'行目でIDが指定されています。';
				continue;
			}
			$not_exist_ids = $this->getNotExistLocalIds($ids);
			if ($not_exist_ids) {
				$errors[$i] = sprintf('データが見つかりませんでした。(%s)', implode(',', $not_exist_ids));
				continue;
			}
			$admin_ids[$ids[0]] = $i;
		}

		$has_error = count($errors) > 0;
		if ($has_error) {
			$error_messages = array();
			foreach ($errors as $i => $error) {
				$error_messages[] = sprintf('%d行目: %s', $i, $error);
			}
			$this->Session->setFlash(implode("<br />", $error_messages), 'message');
		}
		return !$has_error;
	}

	private function joinByLocalId($localId) {
		// 同一アカウントがあるかを検索
		$accountEppn = $this->AccountEppn->find('first',
			array('conditions' => array('local_id' => $localId)));
		if (empty($accountEppn)) {
			$this->Session->setFlash(__("入力したSPS-IDに該当する人は存在しません。", true), 'message');
			return;
		}
		$accountId = $accountEppn['AccountEppn']['account_id'];

		// 自分自身なら登録しない
		if ($accountId == $this->getAccountID()) {
			$this->Session->setFlash(__("自分自身は登録できません。", true), 'message');
			return;
		}

		$result = $this->_joinGroup($accountId, $this->getSelfGroupKey());
		if (!$result) {
			return;
		}
		$this->data['ManageAgency']['spsid'] = "";
	}

	/**
	 * @param unknown_type $groupKey 自グループではなくグループキーしていの画面で利用します
	 */
	private function invite($address, $forcibleAdmission, $groupKey = null) {
		$invalid_addresses = $this->GroupInvite->getInvalidAddresses(array($address));
		if ($invalid_addresses) {
			$this->Session->setFlash(__("メールアドレスが正しくありません。", true), 'message');
			return false;
		}
		$valid_addresses = $this->GroupInvite->getValidAddresses(array($address));
		if (empty($valid_addresses)) {
			$this->Session->setFlash(__("メールアドレスを入力してください。", true), 'message');
			return false;
		}
		$valid_address = implode("\n", $valid_addresses);

		$choiceforcibleAdmission = $this->GroupInvite->canForce() && $forcibleAdmission;

		if (!$choiceforcibleAdmission) {
			if (empty($groupKey)) {
				$result = $this->_inviteGroup($valid_address, $this->getSelfGroupKey());
			} else {
				$result = $this->_inviteGroup($valid_address, $groupKey);
			}
			if ($result) {
				$this->Session->setFlash(__("招待メールを送信しました。", true), 'message');
			}
		} else {
			$account = $this->Account->find('first',
				array('conditions' => array('mail' => $address, 'is_temp' => 0)));
			if (empty($account)) {
				$this->Session->setFlash(__("該当する人は存在しません。", true), 'message');
				return false;
			}
			$accountId = $account['Account']['id'];

			// 自分自身なら登録しない
			if ($accountId == $this->getAccountID()) {
				$this->Session->setFlash(__("自分自身は登録できません。", true), 'message');
				return false;
			}
			if (empty($groupKey)) {
				$result = $this->_joinGroup($accountId, $this->getSelfGroupKey());
			} else {
				$result = $this->_joinGroup($accountId, $groupKey);
			}
			if ($result) {
				$this->Session->setFlash(__("グループへ追加しました。", true), 'message');
			}
		}
		return $result;
	}

	private function _inviteGroup($address, $groupKey) {
		// グループ情報を取得
		$group = $this->GroupInvite->findGroupByGroupKey($groupKey);

		try {
			// グループに招待させる
			$result = $this->GroupInvite->execute(array(
				'accountId' => $this->getAccountID(),
				'group' => $group,
				'addresses' => array($address),
				'addressMl' => '',
				'memberIdList' => array(),
				'adminInput' => array(),
				'body' => null,
				'lang' => $this->getLanguage(),
				'accountLang' => $this->getLanguage(),
			));
		} catch (Exception $e) {
			$this->Session->setFlash(__("登録に失敗しました。", true), 'message');
			$this->log('ManageAgenciesController#_joinGroup() 登録に失敗しました。');
			return false;
		}
		return $result;
	}

	private function _joinGroup($accountId, $groupKey) {
		// グループ情報を取得
		$group = $this->Group->findByGroupKey($groupKey);

		// 既に登録されていたら登録しない
		$count = $this->Mygroup->find('count', array('conditions' => array(
			'groupid' => $group['Group']['id'], 'account_id' => $accountId)));
		if ($count != 0) {
			$this->Session->setFlash(__("既に登録されています。", true), 'message');
			return false;
		}

		// グループに参加させる
		try {
			$this->GroupInvite->joinGroupDB($group['Group']['id'], $accountId);
		} catch (Exception $e) {
			$this->Session->setFlash(__("登録に失敗しました。", true), 'message');
			$this->log('ManageAgenciesController#_joinGroup() 登録に失敗しました。');
			return false;
		}
		return true;
	}

	private function delete() {
		// グループ情報を取得
		$selfGroupKey = $this->getSelfGroupKey();
		$selfGroup = $this->Group->findByGroupKey($selfGroupKey);

		foreach ($this->data['checkList'] as $account) {
			if (empty($account)){
				continue;
			}
			$targetAccountId = $account[0];
			$groupid = $selfGroup['Group']['id'];
			// アカウントIDが自グループのメンバーかを確認
			$mygroup = $this->Mygroup->findByAccountIdAndGroupId($targetAccountId, $groupid);
			if (empty($mygroup)) {
				continue;
			}
			$this->Mygroup->delete($mygroup['Mygroup']['id']);
			// AgreementEntrustmen
			$this->AgreementEntrustment->deleteAll(array(
					'group_id' => $groupid, 'account_id' => $targetAccountId), false);
			// Invite
			$invite = $this->Invite->findByTempAccountIDGroupID($targetAccountId, $groupid);
			$this->Invite->updateInvalid($invite['Invite']['id']);
		}
	}

	private function deleteGroup($groupId) {
		foreach ($this->data['checkList'] as $account) {
			if (empty($account)){
				continue;
			}
			$targetAccountId = $account[0];
			// アカウントIDがグループのメンバーかを確認
			$mygroup = $this->Mygroup->findByAccountIdAndGroupId($targetAccountId, $groupId);
			if (empty($mygroup)) {
				continue;
			}
			$this->Mygroup->delete($mygroup['Mygroup']['id']);
			// AgreementEntrustmen
			$this->AgreementEntrustment->deleteAll(array(
					'group_id' => $groupId, 'account_id' => $targetAccountId), false);
			// Invite
			$invite = $this->Invite->findByTempAccountIDGroupID($targetAccountId, $groupId);
			$this->Invite->updateInvalid($invite['Invite']['id']);
		}
	}

	private function getSelfGroupKey() {
		return $this->getEPPN();
	}

	private function _createUnlessAgentGroup($groupKey, $accountId, $name, $entityId) {
		// 自分自身のグループがあるば取得
		$group = $this->Group->findByGroupKey($groupKey);
		if (!$group) {
			// 自分自身のグループを作成
			$this->Transaction->begin();
			try {
				$this->createSelfGroup($groupKey, $accountId, $name, $entityId);
				$this->Transaction->commit();
			} catch (Exception $e) {
				$this->Session->setFlash(__("グループの生成に失敗しました。", true), 'message');
				$this->log('ManageAgenciesController#show() グループの生成に失敗しました。  '.$e->getMessage());
				$this->Transaction->rollback();
				throw $e;
			}
		}
	}

	private function getSelfGroupMembers($accountId) {
		$members = array();
		$selfGroupKey = $this->getSelfGroupKey();

		// 自分自身のグループがあるか確認
		$selfGroup = $this->Group->findByGroupKey($selfGroupKey);
		if ($selfGroup && $selfGroup['Group']['active'] == 9) {
			$this->Group->delete($selfGroup['Group']['id']);
			$selfGroup = false;
		}
		if (!$selfGroup) {
			// 自分自身のグループを作成
			$this->Transaction->begin();
			try {
				$this->createSelfGroup($selfGroupKey, $accountId, $this->getEPPN(), $this->getEntityID());
				$this->Transaction->commit();
			} catch (Exception $e) {
				$this->Session->setFlash(__("グループの生成に失敗しました。", true), 'message');
				$this->log('ManageAgenciesController#show() グループの生成に失敗しました。  '.$e->getMessage());
				$this->Transaction->rollback();
			}
		} else {
			$members = $this->getGroupMembers($selfGroup['Group']['id'], $this->getAccountID());
		}
		return $members;
	}

	private function getGroupMembers($groupId, $accountId = null) {
		// 代理者(メンバー)一覧を取得(メールアドレス、名前)
		// メンバーを検索
		return $this->Account->find('all', array(
			'fields' => array('Account.id', 'Account.name', 'Account.name_en', 'Account.mail', 'Account.is_temp'),
			'joins'  => array(array(
				'type' => 'INNER', 'table' => 'mygroups', 'alias' => 'Mygroup',
				'conditions' => array(
					'Account.id = Mygroup.account_id',
					'Mygroup.groupid' => $groupId,
				)),
			),
			'conditions' => array('NOT' => array('account_id' => $accountId)),
			'order' => array('Mygroup.admin DESC', 'Account.is_temp', 'Account.name ASC')
		));
	}

	private function addLocalIdToMembers($members) {
		if (!$members) {
			return $members;
		}

		$idList = array();
		foreach ($members as $member) {
			array_push($idList, $member['Account']['id']);
		}
		$account_id_eppns = $this->getAccountIdEppns($idList);

		foreach ($members as $key => $member) {
			$localId = $eppn = '';

			$accountId = $member['Account']['id'];
			if (isset($account_id_eppns[$accountId])) {
				$localId = $account_id_eppns[$accountId]['local_id'];
				$eppn    = $account_id_eppns[$accountId]['eppn'];
			}
			$members[$key]['Account']['local_id'] = $localId;
			$members[$key]['Account']['eppn'] = $eppn;
		}
		return $members;
	}

	private function getAccountIdEppns($idList) {
		$eppns = $this->AccountEppn->find('all', array(
			'fields' => array('account_id', 'local_id', 'eppn'),
			'conditions' => array('account_id' => $idList)
		));
		$account_id_eppns = array();
		foreach ($eppns as $eppn) {
			$account_id_eppns[$eppn['AccountEppn']['account_id']] = $eppn['AccountEppn'];
		}
		return $account_id_eppns;
	}

	/**
	 * @param string $key mail or local_id
	 */
	private function show($key) {
		// 自分の情報を取得
		$self = $this->Account->find('all', array(
			'fields' => array('id', 'name', 'name_en', 'is_temp'),
			'conditions' => array('id' => $this->getAccountID()),
			'limit' => '1'
		));
		$this->IdpTranslator->setIdP($self, $this->getLanguage());

		$members = $this->getSelfGroupMembers($self[0]['Account']['id']);
		// local_idを付加する
		if ($key == 'local_id') {
			$members = $this->addLocalIdToMembers($members);
		}
		$this->removeUnnecessaryLanguage($members);
		$this->IdpTranslator->setIdP($members, $this->getLanguage());

		$this->set('self', $self);
		$this->set('members', $members);
	}

	private function createSelfGroup($selfGroupKey, $accountId, $name, $entityId) {
		// 自分自身のグループを作成
		$group = array(
			'name' => $name,
			'group_key' => $selfGroupKey,
			'introduction' => '代理グループ',
			'active' => '0',
			'public' => '0',
			'openmember' => '1',
			'inspectjoin' => '2',
			'inspectquit' => '0',
			'independence' => '1',
			'down_permission' => '0',
			'up_permission' => '0',
			'group_response' => '2',
			'sp' => '0',
		);
		$this->GroupAgency->create();
		$groupAgency = $this->GroupAgency->save(array('GroupAgency' => $group));
		if (!$groupAgency) {
			throw new Exception('createSelfGroup() GroupAgency save error.');
		}

		// 自身を自身のグループに参加
		$selfGroup = $this->Group->findByGroupKey($selfGroupKey);
		$mygroup = array(
			'account_id' => $accountId,
			'groupid' => $selfGroup['Group']['id'],
			'admin' => 1,
		);
		$this->Mygroup->create();
		if (!$this->Mygroup->save($mygroup)) {
			throw new Exception('createSelfGroup() Mygroup save error.');
		}

		// SPコネクタと接続
		$this->connectSPC($selfGroup['Group']['id'], $accountId);

		return $groupAgency;
	}

	// 自動接続のSPコネクタと接続する
	private function connectSPC($groupid, $accountId) {
		$spcidList = Configure::read('agency_autoconnect_spcidList');
		if (!isset($spcidList)) {
			$spcidList = array();
		}
		foreach ($spcidList as $spcid) {
			// SPコネクタ接続
			$AddGroup = array(
				'top_id' => $spcid,
				'bottom_id' => $groupid,
				'top' => 1,
				'bottom' => 0,
			);
			$this->AddGroup->create();
			$result = $this->AddGroup->save(array('AddGroup' => $AddGroup));
			if (!$result) {
				return false;
			}

//			// 同意 ismemberof
//			$ismemberof = array(
//				'account_id' => $accountId,
//				'spc_id' => $spcid,
//				'group_id' => $groupid,
//				'ismemberof' => 1,
//				'before_ismemberof' => 0,
//			);
//			$this->AgreementIsmemberof->create();
//			$result = $this->AgreementIsmemberof->save($ismemberof);
//			if (!$result) {
//				return false;
//			}
//
//			// 同意 ユーザ情報
//			$agreement = array(
//				'account_id' => $accountId,
//				'group_id' => $spcid,
//				'eptid' => true,
//				'name' => true,
//				'mail' => true,
//				'organization' => true,
//				'idp' => true,
//				'language' => true,
//				'introduction' => true,
//				'before_eptid' => true,
//				'before_name' => true,
//				'before_mail' => true,
//				'before_organization' => true,
//				'before_idp' => true,
//				'before_language' => true,
//				'before_introduction' => true,
//			);
//			$this->Agreement->create();
//			$result = $this->Agreement->save($agreement);
//			if (!$result) {
//				return false;
//			}
			// SPの利用属性に合わせて、自身を同意させる
			$attributeList = $this->TakerAgreement->getAttributeListForSave($spcid);
//debug($attributeList);
			// 利用属性を登録
			$result = $this->TakerAgreement->saveAttribute(
					$groupid, $attributeList, array($spcid), $accountId);
			if (!$result) {
				throw new Exception('saveAttribute error');
			}

		}
		return true;
	}
}
?>
