<?php
/**
 * Management Agreement Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Controller', 'Manages');

class ManageAgreementsController extends ManagesController {

	var $name = 'ManageAgreements';
	var $uses = array('Account', 'AgreementConnect', 'AddGroup', 'AutoGroup');
	var $components = array('TakerAgreement', 'AgreementOperator');
	var $helpers = array('Javascript');

	function beforeFilter() {
		parent::beforeFilter();
		$groupid = $this->_getGroupIdFromURL();
		$this->_checkAccessible($groupid);
		$this->_validateAdminGroup($groupid);
	}

	protected function _checkAccessible($groupid) {
		if (is_null($groupid)
			|| !($this->Group->isGroup($groupid) || $this->Group->isIdPGroup($groupid))) {
			$this->cakeError('notFound', array());
			return;
		}
		parent::_checkAccessible($groupid);
	}

	/**
	 * メンバーの同意属性情報を表示する
	 * @param $page ページID
	 * @param $groupid グループID
	 */
	function state($page, $groupid) {
		$isIdPgroup = $this->AutoGroup->isIdPGroup($groupid);
		if ($isIdPgroup) {
			// 機関グループではメンバーの同意属性情報を表示しない
			$accountList = array();
			$accountCount = 0;
		} else {
			// メンバーの同意情報を取得
			$accountList = $this->AgreementOperator->sortMembers($page, $groupid);
			// メンバー数を取得
			$accountCount = $this->Account->find('count', array(
				'conditions' => array('id in (select account_id from mygroups where groupid = ?)' => $groupid)));
		}

		$this->set('isIdPGroup', $isIdPgroup);

		$this->debugLog('state : accountList = ' . print_r($accountList, true));
		$this->set('accountList', $accountList);
		$this->set('groupid', $groupid);
		$this->data['ManageAgreement']['id'] = $groupid;

		$this->controllPage($page, $accountCount, Configure::read("page.memberlimit"));
		$this->set('page', $page);
		// タブ化対応
		$this->setFavariteFiveGroup($groupid);
	}

	/**
	 * 接続しているSPコネクタの利用属性情報の変更に対しての同意情報を表示する
	 */
	function update($groupid) {
		$agreementList = $this->AgreementOperator->updateAgreementList($groupid, $this->getLanguage());
		$this->set('agreementList', $agreementList);
		$this->set('groupid', $groupid);
		$this->data['ManageAgreement']['id'] = $groupid;
		// タブ化対応
		$this->setFavariteFiveGroup($groupid);
	}

	/**
	 * SPコネクタの利用属性変更に同意する
	 * @param $agreeid agreement_connects.id
	 * @param グループID
	 */
	function agree_ok($agreeid, $groupid) {
		$valid = $this->AgreementOperator->updateAgreeOK($agreeid, $groupid, $this->getAccountID());
		if ($valid)
			$this->Session->setFlash(__("manage_agreements.agreed", true), 'message');

		$count = $this->AgreementConnect->findNonAgreementCount($groupid);
		if ($count > 0)
			$this->redirect("/admin/manage_agreements/update/$groupid");
		else
			$this->redirect("/group_lists/detail/$groupid");
	}

	/**
	 * SPコネクタの利用属性変更に同意しない
	 * @param $agreeid agreement_connects.id
	 * @param グループID
	 */
	function agree_ng($agreeid, $groupid) {
		$valid = $this->AgreementOperator->updateAgreeNG($agreeid, $groupid);
		if ($valid)
			$this->Session->setFlash(__("manage_agreements.not.agreed", true), 'message');

		$count = $this->AgreementConnect->findNonAgreementCount($groupid);
		if ($count > 0)
			$this->redirect("/admin/manage_agreements/update/$groupid");
		else
			$this->redirect("/group_lists/detail/$groupid");
	}
}
?>
