<?php
/**
 * Management SP Connector Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
//App::import('Controller', 'Manages');
App::import('Controller', 'SpGroups');

class ManageSpGroupsController extends SpGroupsController {
	var $name = 'ManageSpGroups';
	var $uses = array('ManageSpGroup', 'SpGroup', 'Mygroup', 'ManageGroup', 'SpHost', 'AddGroup', 'AddGroupInspect', 'GroupSphost',
					'SpAdministrator', 'ProvideAttribute', 'Transaction', 'InspectAttribute', 'Tag',
					'ExtendedAttribute', 'GroupExtendedValue', 'AutoGroup', 'AgreementConnect',
					'AgreementIsmemberof', 'Agreement', 'IconImage', 'Invite', 'SpAccess');
	var $components = array('Url', 'SpcUpdator', 'AgreementOperator', 'Csrf', 'ParameterReplacer', 'Callback',
					'ExtendedAttributeUtils');
	var $helpers = array('Javascript', 'Base64Url');

	function beforeFilter() {
		// グループ詳細画面を表示してから、接続ボタンをクリックするのでcsrfを無効にする
		$this->Csrf->validatePost = false;
		parent::beforeFilter();
		$groupid = $this->_getGroupIdFromURL();
		$this->_validateAdminGroup($groupid);
	}

	/**
	 * SPコネクタを編集する。
	 * @param integer $myGroupid group ID
	 */
	function edit($myGroupid) {
		$accountid = $this->getAccountID();
		switch ($this->getSubmitName()) {
			case '':
				$this->data = $this->ManageSpGroup->find('first',
					array('conditions' => array('id' => $myGroupid)));

				// SP一覧を設定
				$this->set('hostList', $this->getSpHostList($myGroupid));

				// 選択済みSP一覧を取得
// 				$groupSphosts =$this->GroupSphost->find('all', array('conditions' => array('group_id' => $myGroupid)));
// 				$selectedSpList = array();
// 				for ($i = 0; $i < count($groupSphosts); $i++) {
// 					$selectedSpList[$i] = $groupSphosts[$i]['GroupSphost']['sp_id'];
// 				}
// 				foreach($groupSphosts as $sp) {
// 					$spid = $sp['GroupSphost']['sp_id'];
// 					$this->data['ManageSpGroup']['service_name'.$spid] = $sp['GroupSphost']['service_name'];
// 					$this->data['ManageSpGroup']['service_name_en'.$spid] = $sp['GroupSphost']['service_name_en'];
// 					$this->data['ManageSpGroup']['lead_url'.$spid] = $sp['GroupSphost']['lead_url'];
// 					$this->data['ManageSpGroup']['lead_non_map_url'.$spid] = $sp['GroupSphost']['lead_non_map_url'];
// 					$this->data['ManageSpGroup']['callback_url'.$spid] = $sp['GroupSphost']['callback_url'];
// 					$this->data['ManageSpGroup']['logout_option'.$spid] = $sp['GroupSphost']['logout_option'];
// 					$this->data['ManageSpGroup']['relogin_url'.$spid] = $sp['GroupSphost']['relogin_url'];
// 					$this->data['ManageSpGroup']['service_link_both'.$spid] = $sp['GroupSphost']['service_link_both'];
// 				}
// 				$this->set('selectedSp', $selectedSpList);
				$this->setSelectedSp($myGroupid);

				// 提供属性を設定
				$attribute = $this->ProvideAttribute->find('first', array('conditions' => array('group_id' => $myGroupid)));
				if (!empty($attribute)) {
					$this->data['ManageSpGroup']['attr_ismemberof'] = $attribute['ProvideAttribute']['ismemberof'];
					$this->data['ManageSpGroup']['attr_eptid'] = $attribute['ProvideAttribute']['eptid'];
					$this->data['ManageSpGroup']['attr_name'] = $attribute['ProvideAttribute']['name'];
					$this->data['ManageSpGroup']['attr_mail'] = $attribute['ProvideAttribute']['mail'];
					$this->data['ManageSpGroup']['attr_eppn'] = $attribute['ProvideAttribute']['eppn'];
					$this->data['ManageSpGroup']['attr_organization'] = $attribute['ProvideAttribute']['organization'];
					$this->data['ManageSpGroup']['attr_idp'] = $attribute['ProvideAttribute']['idp'];
					$this->data['ManageSpGroup']['attr_language'] = $attribute['ProvideAttribute']['language'];
					$this->data['ManageSpGroup']['attr_introduction'] = $attribute['ProvideAttribute']['introduction'];
				}

				$tagNames = array_values($this->Tag->getPublicTagNames(null, $myGroupid));
				$this->set('tagNames', $tagNames);
				$candidateTagNames = array_values($this->Tag->getPublicTagNames($accountid));
				$this->set('candidateTagNames', $candidateTagNames);

				// 拡張属性を設定
				$attributes = $this->ExtendedAttribute->find('all', array('conditions' => array('spc_id' => $myGroupid)));
				$extendedAttributes = array();
				$extendedAttrNames = array();
				$this->data['ManageSpGroup']['attr_extended'] = 0;
				if (!empty($attributes)) {
//					$this->data['ManageSpGroup']['attr_extended'][0] = 1;
					$this->data['ManageSpGroup']['attr_extended'] = 1;
					$attrCount = 0;
					foreach ($attributes as $attr) {
						$attr_name = 'extended_attr' . ($attrCount + 1);
						$extendedAttrNames[$attrCount] = $attr_name;
						$extendedAttributes[$attr_name . '_id'] = $attr['ExtendedAttribute']['id'];
						$extendedAttributes[$attr_name . '_kind'] = $attr['ExtendedAttribute']['kind'];
						$extendedAttributes[$attr_name . '_name'] = $attr['ExtendedAttribute']['name'];
						$extendedAttributes[$attr_name . '_friendly_name'] = $attr['ExtendedAttribute']['friendly_name'];
						$extendedAttributes[$attr_name . '_value'] = $attr['ExtendedAttribute']['value'];
						$attrCount++;
					}
//					$this->data['extended_attr_count'] = $attrCount;
				}
				$this->set('extendedAttrNames', $extendedAttrNames);
				$this->set('extendedAttributes', $extendedAttributes);
				$this->set('errorPoint', -1);   // エラー無いので、-1

				break;
			case 'save':
				if (empty($this->data)) {
					return;
				}
				$this->Transaction->begin();

				// 更新前の情報を取得
				$beforeAttribute = $this->ProvideAttribute->find('first',
					array('conditions' => array('group_id' => $myGroupid)));
				$beforeTargetSp = $this->GroupSphost->find('list', array(
					'fields' => array('sp_id', 'sp_id'),
					'conditions' => array('group_id' => $myGroupid)));
				$beforeExAttribute = $this->ExtendedAttribute->find('all',
					array('fields' => array('id', 'name', 'friendly_name'),
						'conditions' => array('spc_id' => $myGroupid, 'kind' => 1),
						'order' => array('id' => 'asc')));

				$result = $this->save($myGroupid);


				// バリデーションエラー
				if ($result === 'validate_error') {
					$this->Transaction->rollback();
					$spGroup = $this->ManageSpGroup->find('first', array('fields' => array('name'),
						'conditions' => array('id' => $myGroupid)));
					//$this->data['ManageSpGroup']['name'] = $spGroup['ManageSpGroup']['name'];

					$hostList = $this->getSpHostList($myGroupid);
					foreach ($hostList as $hostId => $host) {
						if (isset($this->data['ManageSpGroup']['service_name'.$hostId])) {
							$hostList[$hostId]['service_name'] = $this->data['ManageSpGroup']['service_name'.$hostId];
						}
						if (isset($this->data['ManageSpGroup']['lead_url'.$hostId])) {
							$hostList[$hostId]['service_name'] = $this->data['ManageSpGroup']['service_name'.$hostId];
						}
					}

					// SP一覧を設定
					//$this->set('hostList', $this->getSpHostList());
					$this->set('hostList', $this->getSpHostList($myGroupid));
//					$this->setSelectedSp($myGroupid);
					$this->ExtendedAttributeUtils->extendedShow('ManageSpGroup');
					$this->Session->setFlash(__("message.input.error", true), 'message');
					return;
				} else if (!$result) {
					$this->Transaction->rollback();
					$this->Session->setFlash(__("common.error.occurred", true), 'message');
					$this->redirect('/mygroups/view');
					return;
				}

				// 更新後の情報を取得
				$afterAttribute = $this->ProvideAttribute->find('first',
					array('conditions' => array('group_id' => $myGroupid)));
				$afterTargetSp = $this->GroupSphost->find('list', array(
					'fields' => array('sp_id', 'sp_id'),
					'conditions' => array('group_id' => $myGroupid)));
				$afterExAttribute = $this->ExtendedAttribute->find('all',
					array('fields' => array('id', 'name', 'friendly_name'),
						'conditions' => array('spc_id' => $myGroupid, 'kind' => 1),
						'order' => array('id' => 'asc')));

				// 下位グループの管理者へメールを送信する
				$result = $this->SpcUpdator->sendMailGroup($myGroupid, $beforeAttribute, $beforeTargetSp, $afterAttribute, $afterTargetSp);
				if (!$result) {
					$this->Transaction->rollback();
					$this->Session->setFlash(__("common.error.occurred", true), 'message');
					$this->redirect('/mygroups/view/');
					return;
				}

				$result = $this->Tag->updatePublicTags($accountid, $myGroupid, $this->data['tagNames']);
				if (!$result) {
					$this->Transaction->rollback();
					$this->Session->setFlash(__("common.error.occurred", true), 'message');
					$this->redirect('/mygroups/view');
					return;
				}

				// グループの管理者へ拡張属性追加案内メールを送信する
				$result = $this->_sendMailGroupExtended($myGroupid, $beforeExAttribute, $afterExAttribute, $afterTargetSp);
				if (!$result) {
					$this->Transaction->rollback();
					$this->Session->setFlash(__("common.error.occurred", true), 'message');
					$this->redirect('/mygroups/view/');
					return;
				}

				$this->Transaction->commit();

				$this->Session->setFlash(__("message.sp_groups.save", true), 'message');
				$this->redirect('/sp_groups/detail/'.$myGroupid);
				break;
			case 'cancel':
				$this->redirect('/sp_groups/detail/'.$myGroupid);
				return;
			default:
				break;
		}
	}

	private function setSelectedSp($myGroupid) {
		// 選択済みSP一覧を取得
		$groupSphosts = $this->GroupSphost->find('all', array('conditions' => array('group_id' => $myGroupid)));
		$selectedSpList = array();
		for ($i = 0; $i < count($groupSphosts); $i++) {
			$selectedSpList[$i] = $groupSphosts[$i]['GroupSphost']['sp_id'];
		}
		foreach ($groupSphosts as $sp) {
			$spid = $sp['GroupSphost']['sp_id'];
			$this->data['ManageSpGroup']['service_name'.$spid] = $sp['GroupSphost']['service_name'];
			$this->data['ManageSpGroup']['service_name_en'.$spid] = $sp['GroupSphost']['service_name_en'];
			$this->data['ManageSpGroup']['lead_url'.$spid] = $sp['GroupSphost']['lead_url'];
			$this->data['ManageSpGroup']['lead_non_map_url'.$spid] = $sp['GroupSphost']['lead_non_map_url'];
			$this->data['ManageSpGroup']['callback_url'.$spid] = $sp['GroupSphost']['callback_url'];
			$this->data['ManageSpGroup']['logout_option'.$spid] = $sp['GroupSphost']['logout_option'];
			$this->data['ManageSpGroup']['relogin_url'.$spid] = $sp['GroupSphost']['relogin_url'];
			$this->data['ManageSpGroup']['service_link_both'.$spid] = $sp['GroupSphost']['service_link_both'];
		}

		$this->set('selectedSp', $selectedSpList);
	}

	private function save($myGroupid) {
		// 対象SPは対象としてよいSPなのかを確認(不正利用対策)
		$isSetTargetSP = $this->canSetTargetSp($myGroupid);
		if (!$isSetTargetSP) {
			$this->cakeError('illegalAccess');
			return;
		}

		// グループに登録
		$this->data['ManageSpGroup']['group_key'] = $this->ManageSpGroup->field(
			'group_key', array('id' => $myGroupid));
		$this->data['ManageSpGroup']['sp'] = 1;
		if (!$this->ManageSpGroup->save($this->data)) {
			return 'validate_error';
		}


		/*
		 * autoフィールドは、変更しないため現在値を取得して、再登録時に
		 * 指定する。
		 */
		$auto = $this->GroupSphost->field('auto', array('group_id' => $myGroupid), 'auto DESC');
		// グループSPホストを一度削除する。
		$this->GroupSphost->deleteAll(array('group_id' => $myGroupid), false);
		// グループSPホストに登録
		if (!empty($this->data['ManageSpGroup']['sp_host'])) {
			$isError = false;
			foreach ($this->data['ManageSpGroup']['sp_host'] as $spid) {
				$this->GroupSphost->create();
				$group_sp = array(
					'GroupSphost' => array(
						'group_id' => $myGroupid,
						'sp_id' => $spid,
						'service_name' => $this->data['ManageSpGroup']['service_name'.$spid],
						'lead_url' => $this->data['ManageSpGroup']['lead_url'.$spid],
						'lead_non_map_url' => $this->data['ManageSpGroup']['lead_non_map_url'.$spid],
						'callback_url' => $this->data['ManageSpGroup']['callback_url'.$spid],
						'logout_option' => $this->data['ManageSpGroup']['logout_option'.$spid],
						'relogin_url' => $this->data['ManageSpGroup']['relogin_url'.$spid],
						'auto' => $auto,
					),
				);
				if (isset($this->data['ManageSpGroup']['service_link_both'.$spid][0])) {
					$group_sp['GroupSphost']['service_link_both'] = $this->data['ManageSpGroup']['service_link_both'.$spid][0];
				}
				$result = $this->GroupSphost->save($group_sp);
				if (!$result) {
					$isError = true;
					foreach ($this->GroupSphost->validationErrors as $k => $validationError) {
						$this->ManageSpGroup->validationErrors[$k.$spid] = $validationError;
					}
				}
			}
			if ($isError) {
				return 'validate_error';
			}
		}

		// 提供属性を登録
		$attribute = $this->ProvideAttribute->find('first', array(
			'fields' => array('id'),
			'conditions' => array('group_id' => $myGroupid)));
		$attribute['ProvideAttribute']['group_id'] = $myGroupid;
		$attribute['ProvideAttribute']['ismemberof'] = 0;
		$attribute['ProvideAttribute']['eptid'] = 0;
		$attribute['ProvideAttribute']['name'] = 0;
		$attribute['ProvideAttribute']['mail'] = 0;
		$attribute['ProvideAttribute']['eppn'] = 0;
		$attribute['ProvideAttribute']['organization'] = 0;
		$attribute['ProvideAttribute']['idp'] = 0;
		$attribute['ProvideAttribute']['introduction'] = 0;
		$attribute['ProvideAttribute']['language'] = 0;
		if (!empty($this->data['ManageSpGroup']['attr_ismemberof']))
			$attribute['ProvideAttribute']['ismemberof'] = $this->data['ManageSpGroup']['attr_ismemberof'][0];
		if (!empty($this->data['ManageSpGroup']['attr_eptid']))
			$attribute['ProvideAttribute']['eptid'] = $this->data['ManageSpGroup']['attr_eptid'][0];
		if (!empty($this->data['ManageSpGroup']['attr_name']))
			$attribute['ProvideAttribute']['name'] = $this->data['ManageSpGroup']['attr_name'][0];
		if (!empty($this->data['ManageSpGroup']['attr_mail']))
			$attribute['ProvideAttribute']['mail'] = $this->data['ManageSpGroup']['attr_mail'][0];
		if (!empty($this->data['ManageSpGroup']['attr_eppn']))
			$attribute['ProvideAttribute']['eppn'] = $this->data['ManageSpGroup']['attr_eppn'][0];
		if (!empty($this->data['ManageSpGroup']['attr_organization']))
			$attribute['ProvideAttribute']['organization'] = $this->data['ManageSpGroup']['attr_organization'][0];
		if (!empty($this->data['ManageSpGroup']['attr_idp']))
			$attribute['ProvideAttribute']['idp'] = $this->data['ManageSpGroup']['attr_idp'][0];
		if (!empty($this->data['ManageSpGroup']['attr_introduction']))
			$attribute['ProvideAttribute']['introduction'] = $this->data['ManageSpGroup']['attr_introduction'][0];
		if (!empty($this->data['ManageSpGroup']['attr_language']))
			$attribute['ProvideAttribute']['language'] = $this->data['ManageSpGroup']['attr_language'][0];

		// 提供属性に変更があるかを確認
		$beforeAttribute = $this->ProvideAttribute->find('first', array(
			'conditions' => array('group_id' => $myGroupid)));
		$diff = $this->diffArray($beforeAttribute['ProvideAttribute'], $attribute['ProvideAttribute']);
		if ($diff) {
			// 違いがあれば更新する
			$result = $this->ProvideAttribute->save($attribute);
			if (!$result) {
				return false;
			}
		}

		// 拡張属性を保存する
		$result = $this->ExtendedAttributeUtils->saveExtended($myGroupid, 'ManageSpGroup');
		if (!$result) {
			return 'validate_error';
		}

		return true;
	}

	private function diffArray($beforeAttribute, $attribute) {
		if ($beforeAttribute['ismemberof'] != $attribute['ismemberof']) {
			return true;
		}
		if ($beforeAttribute['eptid'] != $attribute['eptid']) {
			return true;
		}
		if ($beforeAttribute['name'] != $attribute['name']) {
			return true;
		}
		if ($beforeAttribute['mail'] != $attribute['mail']) {
			return true;
		}
		if ($beforeAttribute['eppn'] != $attribute['eppn']) {
			return true;
		}
		if ($beforeAttribute['organization'] != $attribute['organization']) {
			return true;
		}
		if ($beforeAttribute['idp'] != $attribute['idp']) {
			return true;
		}
		if ($beforeAttribute['introduction'] != $attribute['introduction']) {
			return true;
		}
		if ($beforeAttribute['language'] != $attribute['language']) {
			return true;
		}
		return false;
	}

	/**
	 * SPコネクタを削除する。
	 * @param $groupid
	 */
	function delete($groupid) {
		switch ($this->getSubmitName()) {
			case '':
				$this->data['ManageSpGroup']['id'] = $groupid;
				break;
			case 'delete':
				$this->Transaction->begin();
				// SPコネクタデータ削除
				if (!$this->deleteSPData($groupid)) {
					$this->Transaction->rollback();
					$this->log('ManageSpGroupsController::delete : deleteSPData() Error');
					$this->cakeError('systemError');
				}
				$this->Transaction->commit();

				$this->Session->setFlash(__("message.sp_groups.del", true), 'message');
				$this->redirect('/mygroups/view');
				break;
			case 'cancel':
				$this->redirect('/sp_groups/detail/'.$groupid);
				return;
			default:
				break;
		}
	}
 
	/**
	 * データを削除する。
	 * @param $groupid group ID
	 */
	private function deleteSPData($groupid) {
		// SPコネクタ情報は論理削除
		$result = $this->ManageSpGroup->softDelete($groupid);
		if (!$result) {
			return false;
		}

		// Myグループ情報は物理削除
		$this->Mygroup->deleteAll(array('groupid' => $groupid), false);
		if (!$result) {
			return false;
		}

		// グループのコネクションは物理削除
		$this->AddGroup->deleteAll(array('OR' => array(
			'top_id' => $groupid, 'bottom_id' => $groupid)), false);
		if (!$result) {
			return false;
		}

		// SPコネクタとSPの紐付けを削除
		$this->GroupSphost->deleteAll(array('group_id' => $groupid), false);
		if (!$result) {
			return false;
		}

		// Tagは物理削除
		$this->Tag->deleteAll(array('group_id' => $groupid), false);
		if (!$result) {
			return false;
		}

		// 拡張属性は物理削除
		$this->ExtendedAttribute->deleteAll(array('spc_id' => $groupid), true);   // GroupExtendedValue も削除
		if (!$result) {
			return false;
		}

		// グループ間/SPコネクタ-グループ間接続審査も物理削除
		$result = $this->AddGroupInspect->deleteAll(array('top_id' => $groupid), false);
		if (!$result) {
			return false;
		}

		// 同意要請のグループ伝播も物理削除
		$result = $this->AgreementConnect->deleteAll(array('spc_id' => $groupid), false);
		if (!$result) {
			return false;
		}

		// グループIDに関する属性送信同意も物理削除
		$result = $this->AgreementIsmemberof->deleteAll(array('spc_id' => $groupid), false);
		if (!$result) {
			return false;
		}

		// グループID以外の属性送信同意も物理削除
		$result = $this->Agreement->deleteAll(array('group_id' => $groupid), false);
		if (!$result) {
			return false;
		}

		// アイコン画像バイナリも物理削除
		$result = $this->IconImage->deleteAll(array('group_id' => $groupid), false);
		if (!$result) {
			return false;
		}

		// 属性送信変更時の通知伝播も物理削除
		$result = $this->InspectAttribute->deleteAll(array('to_group_id' => $groupid), false);
		if (!$result) {
			return false;
		}

		// グループのメンバー招待情報も物理削除
		$result = $this->Invite->deleteAll(array('groupid' => $groupid), false);
		if (!$result) {
			return false;
		}

		// SPコネクタに設定された送信される属性も物理削除
		$result = $this->ProvideAttribute->deleteAll(array('group_id' => $groupid), false);
		if (!$result) {
			return false;
		}

		// SPコネクタ並べ替え用アクセス情報も物理削除
		$result = $this->SpAccess->deleteAll(array('spc_id' => $groupid), false);
		if (!$result) {
			return false;
		}

		return true;
	}

	/**
	 * 「下にグループを追加」を表示する。
	 * @param $myGroupid group ID
	 */
	function add_down($myGroupid) {
		$this->add_down_list(1, $myGroupid);
	}

	/**
	 * 下位グループ接続を表示する。
	 * @param $page
	 * @param $myGroupid group ID
	 */
	function add_down_list($page, $myGroupid) {
		$this->data['ManageSpGroup']['id'] = $myGroupid;

		// 自グループ情報を設定
		$SpGroup = $this->SpGroup->find('first', array(
			'fields' => array('name', 'id'),
			'conditions' => array('id' => $myGroupid)));
		//$this->set('ManageSpGroup', $SpGroup);
		$this->set("message", "");

		// このグループの下になることが可能なグループを検索
		// sp = 1, up_permission = 1
		// 自分のグループじゃない
		// 既にこのグループの下になっていたら表示しない
		$groupCount = $this->ManageGroup->find("count", array(
			'conditions' => array(
				'active' => 0,
				'sp' => 0,
				'public' => 1,
				'NOT' => array('id' => array($myGroupid)),
				'id NOT IN (select bottom_id from add_groups where top_id = ?)' => $myGroupid
		)));

		$pgroups = $this->ManageGroup->find("all", array(
			'fields' => array('id', 'name', 'name_en', 'introduction', 'introduction_en', 'down_permission'),
			'conditions' => array(
				'active' => 0,
				'sp' => 0,
				'public' => 1,
				'NOT' => array('id' => array($myGroupid)),
				'id NOT IN (select bottom_id from add_groups where top_id = ?)' => $myGroupid),
			'limit' => Configure::read("page.grouplimit"),
			'order' => array('name'),
			'page' => $page,
		));

		// ページ制御
		$this->controllPage($page, $groupCount, Configure::read("page.grouplimit"));
		$this->set('page', $page);

		$this->setAddGroupView($pgroups, 'PublicGroupOptions', 'sp');
		$this->render('add_down');
	}

	/**
	 * 上下グループを表示する。
	 * @param $groupid
	 */
	function updown($groupid) {
		$group['ManageSpGroup']['id'] = $groupid;
		$this->set("SpGroup", $group);

		// 下位グループを検索
		$bottomUniGroups = $this->Group->find('all', array(
			'fields' => array('id', 'name', 'name_en', 'introduction', 'introduction_en'),
			'conditions' => array(
				'active' => array(0, 1),
				'id IN (select bottom_id from add_groups where bottom = 0 and top_id = ?)' => $groupid),
			'order' => array('name'),
		));
		// 英語対応
		$count = count($bottomUniGroups);
		for ($i = 0; $i < $count; $i++) {
			$gname = $this->getGroupName($bottomUniGroups[$i]);
			$bottomUniGroups[$i]['Group']['name'] = $gname;
		}
		$this->set('bottomUniGroups', $bottomUniGroups);

		// グラフ設定
		$nodes = "";
		$edges = "";

		// ２段目に配置
		$nodes = $this->setNodes($nodes, "this", $groupid, '', 0, 150, 1, true);

		// 下グループ
		$count = count($bottomUniGroups);
		for ($i = 0; $i < $count; $i++) {
			$x = $i * 150;
			$nodes = $this->setNodes(
				$nodes, $bottomUniGroups[$i]['Group']['name'], $bottomUniGroups[$i]['Group']['id'],
				$bottomUniGroups[$i]['Group']['introduction'], $x, 250, 0);
			$edges = $this->setEdges($edges, $bottomUniGroups[$i]['Group']['name'], "this", true);
		}
		$this->set('nodes', $nodes);
		$this->set('edges', $edges);
	}

	/**
	 * 該当グループを自SPコネクタの下に追加する。
	 * 追加になるグループIDなら追加
	 * 追加するのに申請する必要があるのあら、申請
	 * @param $myGroupid 下になるグループID
	 */
	function add_down_save($myGroupid) {
		if ($this->getSubmitName() == "cancel") {
			$this->redirect('/admin/manage_sp_groups/updown/'.$myGroupid);
			return;
		}

		$selectedGroupId = $this->data['ManageSpGroup']['publicGroup'];
		if (empty($selectedGroupId)) {
			$this->redirect('/admin/manage_sp_groups/updown/'.$myGroupid);
			return;
		}

		// そもそも追加できるグループなのかをチェック
		if (!$this->validate_add_save_down($myGroupid, $selectedGroupId, true)) {
			$this->Session->setFlash(__("message.manage.cannot.connect.group", true), 'message');
			$this->redirect('/admin/manage_sp_groups/updown/'.$myGroupid);
			return;
		}

		$count = $this->AddGroup->find('count', array(
			'conditions' => array('top_id' => $myGroupid, 'bottom_id' => $selectedGroupId)));
		if ($count > 0) {
			$this->Session->setFlash(__("message.common.already.registered", true), 'message');
			$this->redirect('/admin/manage_sp_groups/add_down/'.$myGroupid);
			return;
		}

		// 既に申請したいたら、何もしない
		$count = $this->AddGroupInspect->find('count', array('conditions' => array(
			'top_id' => $myGroupid,
			'bottom_id' => $selectedGroupId,
			'join' => 0,
			'kind' => 1,  // 自分が上になりたい
			'status' => 0
		)));
		if ($count > 0) {
			$selectedGroup = $this->Group->find('first', array('fields' => array('id', 'name'), 'conditions' => array('id' => $selectedGroupId)));
			$msgstr = $this->createMessage("message.manage.already.connect.application", $selectedGroup['Group']['name']);
			$this->Session->setFlash($msgstr, 'message');

			$this->redirect('/admin/manage_sp_groups/add_down/'.$myGroupid);
			return;
		}

		// 選択したグループ情報を取得(該当グループは、上にこのグループを入れる際に申請するかのup_permissionを取得)
		$selectedGroupInfo = $this->Group->find("first", array('fields' => array('id', 'name'),
			'conditions' => array('id' => $selectedGroupId)));

		// グループを追加
		$this->downSave($myGroupid, $selectedGroupId);

		$this->Session->setFlash(
			__("message.sp_groups.request.group", true).
			' ('.h($selectedGroupInfo['Group']['name']).')', 'message');

		// 追加先の管理者へメールをする
		$this->sedMailForAddGroup($selectedGroupId, $myGroupid, 'down', 2, false, true);

		$this->redirect('/admin/manage_sp_groups/group_inspect_state/'.$myGroupid);
	}

	/**
	 * 該当グループを自SPコネクタの下に追加する。
	 * SPコネクタからグループの追加は必ず申請が必要となる。
	 * @param integer $myGroupid 自グループID
	 * @param integer $selectedGroupId 選択したグループID(SPコネクタ or グループ)
	 */
	private function downSave($myGroupid, $selectedGroupId) {
		$groupInspect = array(
			'account_id' => $this->getAccountID(),
			'top_id' => $myGroupid,
			'bottom_id' => $selectedGroupId,
			'to_id' => $selectedGroupId,   // 申請先グループID
			'from_id' => $myGroupid, // 申請元グループID
			'join' => 0,
			'kind' => 1, // 上になりたい申請
			'status' => 0,
			'top' => 1,    // 上になるのは自分なのでSPコネクタ
			'bottom' => 0, // 下になるのはSPコネクタ or グループ
		);
		$this->AddGroupInspect->create();
		$this->AddGroupInspect->save(array('AddGroupInspect' => $groupInspect));
	}

	/**
	 * グループ追加審査を表示する。
	 * @param unknown_type $groupid
	 */
	function inspect($groupid) {
		$this->data['ManageSpGroup']['id'] = $groupid;

		// 下になりたいグループからの申請
		$downGroups = $this->findInspectGroup($groupid, 0);
		$this->set('downGroups', $downGroups);

		if (empty($downGroups)) {
			$historys = $this->findInspectGroup2days($groupid, array(1, 2));
			$this->set('historys', $historys);
		}
		$this->render('inspect');
	}

	/**
	 * グループ追加審査を表示する。
	 * @param unknown_type $groupid
	 */
	function inspect_history($groupid) {
		$this->data['ManageSpGroup']['id'] = $groupid;

		$downGroups = $this->findInspectGroup($groupid, array(1, 2));
		$this->set('downGroups', $downGroups);
	}


	/**
	 *
	 * @param integer $groupid
	 * @param integer $status
	 */
	private function findInspectGroup($groupid, $status) {
		// 下になりたいグループからの申請
		$downGroups = $this->ManageSpGroup->find('all', array(
			'fields' => array('ManageSpGroup.id', 'ManageSpGroup.name', 'ManageSpGroup.introduction',
				'AddGroupInspect.id', 'AddGroupInspect.kind', 'AddGroupInspect.status',
				'AddGroupInspect.created', 'AddGroupInspect.modified'),
			'joins' => array(
				array(
					'type' => 'INNER', 'table' => 'add_group_inspects', 'alias' => 'AddGroupInspect',
					'conditions' => array(
						'ManageSpGroup.id = AddGroupInspect.from_id', // 申請元のグループ情報をとりたい
						'AddGroupInspect.top_id' => $groupid,
						'AddGroupInspect.to_id' => $groupid,
						'kind' => 0,
						'status' => $status
					)
				)
			),
			'order' => array('AddGroupInspect.modified DESC'),
		));
		return $downGroups;
	}

	/**
	 *
	 * @param integer $groupid
	 * @param integer $status
	 */
	private function findInspectGroup2days($groupid, $status) {
		// 下になりたいグループからの申請
		$date = date('Y-m-d H:i:s', strtotime('-48 hour ' . date("Y-m-d H:i:s")));
		$downGroups = $this->SpGroup->find('all', array(
			'fields' => array('SpGroup.id', 'SpGroup.name', 'SpGroup.introduction',
				'AddGroupInspect.id', 'AddGroupInspect.kind', 'AddGroupInspect.status',
				'AddGroupInspect.created', 'AddGroupInspect.modified'),
			'joins' => array(
				array(
					'type' => 'INNER', 'table' => 'add_group_inspects', 'alias' => 'AddGroupInspect',
					'conditions' => array(
						'SpGroup.id = AddGroupInspect.from_id', // 申請元のグループ情報をとりたい
						'AddGroupInspect.top_id' => $groupid,
						'AddGroupInspect.to_id' => $groupid,
						'kind' => 0,
						'status' => $status,
						'AddGroupInspect.modified > \''.$date.'\'',
					)
				)
			),
			'limit' => Configure::read("page.historylimit"),
			'order' => array('AddGroupInspect.modified DESC'),
		));
		return $downGroups;
	}

	/**
	 * グループ追加を許可する。
	 * @param unknown_type $updown
	 * @param integer $inspectId
	 * @param integer $spcid SPコネクタID(自身)
	 */
	function inspect_allow($updown, $inspectId, $spcid) {
		// 申請情報を取得
		$inspect = $this->AddGroupInspect->find('first', array(
			'conditions' => array('id' => $inspectId, 'status' => 0)));
		if (empty($inspect)) {
			$this->Session->setFlash(__("message.manage_inspects.no.information", true), 'message');
			$this->redirect('/admin/manage_sp_groups/inspect/'.$spcid);
		}

		$this->Transaction->begin();
		// グループを追加
		$addGroup = array(
			'AddGroup' => array(
				'top_id' => $inspect['AddGroupInspect']['top_id'],
				'bottom_id' => $inspect['AddGroupInspect']['bottom_id'],
				'top' => $inspect['AddGroupInspect']['top'],
				'bottom' => $inspect['AddGroupInspect']['bottom'],
			),
		);
		$result = $this->AddGroup->save($addGroup);
		if (!$result) {
			$this->Transaction->rollback();
			$this->cakeError('systemError');
			return;
		}

		// 処理済に更新
		$chgInspect = array(
			'AddGroupInspect' => array(
				'id' => $inspect['AddGroupInspect']['id'],
				'status' => 1,
			),
		);
		$result = $this->AddGroupInspect->save($chgInspect);
		if (!$result) {
			$this->Transaction->rollback();
			$this->cakeError('systemError');
			return;
		}

		// SPへの属性提供が管理者によって同意ずみに設定されていたら、接続先のメンバーを同意状態にする
		$inspectAttribute = $this->InspectAttribute->findAttributeByInspectId($inspectId);
		if (!empty($inspectAttribute)) {
			// 処理済に更新
			$entity = array();
			$entity['id'] = $inspectAttribute['id'];
			$entity['status'] = 1;
			$result = $this->InspectAttribute->save($entity);
			if (!$result) {
				$this->Transaction->rollback();
				$this->cakeError('systemError');
				return;
			}

			// 接続先のメンバーを同意済みにする(接続先の下位グループのメンバーは同意済みにしてはいけない)
			$fromGroupId = $inspect['AddGroupInspect']['bottom_id'];
			$result = $this->TakerAgreement->takeAgreementForManager($spcid, $fromGroupId, $inspectAttribute, 0,
				$inspect['AddGroupInspect']['account_id']);
			if (!$result) {
				$this->Transaction->rollback();
				$this->cakeError('systemError');
				return;
			}

			// 下位グループに同意データを登録、下位グループ管理者がOKすると、下位グループメンバーに連絡される
			$agreeid = $this->AgreementOperator->registerBottomAdminByConnectSPC($fromGroupId, $spcid);
			if (!$agreeid) {
				$this->Transaction->rollback();
				$this->cakeError('systemError');
				return;
			}
		}
		$this->Transaction->commit();

		// グループメンバーに同意する必要があることをメールする
		$attributePath = $this->InspectAttribute->getSelectAttributePath($inspectAttribute);
		$this->Url->get("/map/batch_send_mails/agreement_mail/$fromGroupId/0/$spcid$attributePath");
debug("/map/batch_send_mails/agreement_mail/$fromGroupId/0/$spcid$attributePath");
debug($inspectAttribute);

		// 下位グループの管理者へ連絡
		$this->AgreementOperator->sendMailBottomAdmin(
			$fromGroupId, $spcid, $agreeid, 'add_group_conntact_bottom_group_template');

		// 申請元のグループ管理者へ結果をメールする
		$this->sendMailAllowDeny($spcid, $inspect['AddGroupInspect']['from_id'], true, false, true);
		$this->Session->setFlash(__("message.manage_inspects.approve.group.add", true), 'message');

		// calbackUrlを取得
		$groupid = $inspect['AddGroupInspect']['bottom_id'];
		$group = $this->Group->findGroup($groupid);
		$groupKey = $group['Group']['group_key'];
		$serviceInfoList = $this->GroupSphost->findList($spcid);
		foreach ($serviceInfoList as $serviceInfo) {
			$spid = $serviceInfo['GroupSphost']['sp_id'];
			$callbackUrl = $serviceInfo['GroupSphost']['callback_url'];
			if (empty($callbackUrl)) {
				continue;
			}
			$callbackUrl = $this->ParameterReplacer->replaceGroupKey($groupKey, $callbackUrl);
//			$callbackUrl = $this->ParameterReplacer->replaceEntityId($callbackUrl);
			// グループ管理者のIDを取得
			$mygroups = $this->Mygroup->find('first', array(
				'conditions' => array('groupid' => $groupid, 'admin' => array(1, 2)),
				'order' => array('id')));
			$account = $this->Account->find('first', array(
				'conditions' => array('id' => $mygroups['Mygroup']['account_id'])));
			$callbackUrl = $this->ParameterReplacer->replaceAdminEmail($account['Account']['mail'], $callbackUrl);
			$this->log('$callback_url = ' . $callbackUrl, "debug");
			// サービスのAPI呼び出し
			$json = $this->Callback->call($spid, $callbackUrl);
			if (!isset($json) || ($json['result'] != 'OK' && $json['result'] != 'ok')) {
				$this->Session->setFlash(__("message.failed.service.api", true), 'message');
			}
		}

		// 未処理のデータを検索
		$count = $this->AddGroupInspect->find('count', array(
			'conditions' => array(
				'to_id' => $spcid,
				'top' => 1,
				'status' => 0
			)));
		if ($count > 0) {
			$this->redirect('/admin/manage_sp_groups/inspect/'.$spcid);
		} else {
			$this->redirect('/admin/manage_sp_groups/inspect_history/'.$spcid);
		}
	}

	/**
	 * グループ追加を拒否する。
	 * @param unknown_type $updown
	 * @param integer $inspectId
	 * @param integer $groupid
	 */
	function inspect_deny($updown, $inspectId, $groupid) {
		// 申請情報を取得
		$inspect = $this->AddGroupInspect->find('first', array(
			'fields' => array('id', 'top_id', 'bottom_id', 'top', 'bottom', 'status', 'from_id'),
			'conditions' => array('id' => $inspectId, 'status' => 0)));
		if (empty($inspect)) {
			$this->Session->setFlash(__("message.manage_inspects.no.information", true), 'message');
			$this->redirect('/admin/manage_sp_groups/inspect/'.$groupid);
		}

		// 処理済に
		$inspect['AddGroupInspect']['status'] = 2;
		$this->AddGroupInspect->save($inspect);

		// 申請元のグループ管理者へ結果をメールする
		$this->sendMailAllowDeny($groupid, $inspect['AddGroupInspect']['from_id'], false, false, true);

		$this->Session->setFlash(__("message.manage_inspects.reject.group.add", true), 'message');

		// 未処理のデータを検索
		$count = $this->AddGroupInspect->find('count', array(
			'conditions' => array(
				'to_id' => $groupid,
				'top' => 1,
				'status' => 0
			)));
		if ($count > 0) {
			$this->redirect('/admin/manage_sp_groups/inspect/'.$groupid);
		} else {
			$this->redirect('/admin/manage_sp_groups/inspect_history/'.$groupid);
		}
	}

	/**
	 * グループ追加の申請状況を表示する。
	 * @param $groupid
	 */
	function group_inspect_state($groupid) {
		$this->data['ManageInspect']['id'] = $groupid;

		// グループ追加審査情報を検索
		$groups = $this->SpGroup->find('all', array(
			'fields' => array('SpGroup.id', 'SpGroup.name', 'SpGroup.introduction',
				'AddGroupInspect.id', 'AddGroupInspect.kind', 'AddGroupInspect.status', 'AddGroupInspect.created'),
			'joins' => array(
				array(
					'type' => 'INNER', 'table' => 'add_group_inspects', 'alias' => 'AddGroupInspect',
					'conditions' => array(
						'SpGroup.id = AddGroupInspect.to_id', // 申請先のグループ情報をとりたい
						'AddGroupInspect.from_id' => $groupid,
					)
				)
			),
			'order' => array('AddGroupInspect.created DESC'),
			'limit' => Configure::read("page.statelimit"),
		));
		$this->set('groups', $groups);
	}

	/**
	 * グループ追加の申請を取り消す。
	 * @param integer $inspectid
	 * @param integer $groupid
	 */
	function group_inspect_cancel($inspectid, $groupid) {
		$groupInspect = $this->AddGroupInspect->find('first', array(
			'fields' => array('id', 'status'),
			'conditions' => array('id' => $inspectid)));
		if (empty($groupInspect)) {
			$this->Session->setFlash(__("message.manage_inspects.no.information", true), 'message');
			$this->redirect('/mygroups/view');
		}


		$inspect = array(
			'AddGroupInspect' => array(
				'id' => $inspectid,
				'status' => 3,
			),
		);
		$this->AddGroupInspect->save($inspect);

		$this->Session->setFlash(__("message.manage_inspects.cancel.application", true), 'message');

		// 申請中のデータがあるかを確認
		$count = $this->AddGroupInspect->find('count', array(
			'conditions' => array('from_id' => $groupid, 'status' => 0)));
		if ($count > 0) {
			$this->redirect('/admin/manage_sp_groups/group_inspect_state/'.$groupid);
		} else {
			$this->redirect('/sp_groups/detail/'.$groupid);
		}
	}

	/**
	 * グループ間の接続を削除する。
	 * @param $updown
	 * @param $pairGroupid
	 * @param $myGroupid
	 */
	function delete_connection($updown, $pairGroupid, $myGroupid) {
		if ($updown === 'up') {
			$top_id = $pairGroupid;
			$bottom_id = $myGroupid;
		} else if ($updown === 'down') {
			$top_id = $myGroupid;
			$bottom_id = $pairGroupid;
		}

		$this->Transaction->begin();
		$result = $this->GroupExtendedValue->deleteAll(array(
				'spc_id' => $top_id, 'group_id' => $bottom_id), false);
		if (!$result) {
			$this->Transaction->rollback();
			$this->cakeError('systemError');
			return;
		}
		$result = $this->AddGroup->deleteAll(array(
				'top_id' => $top_id, 'bottom_id' => $bottom_id), false);
		if (!$result) {
			$this->Transaction->rollback();
			$this->cakeError('systemError');
			return;
		}
		$this->Transaction->commit();

		$this->redirect('/admin/manage_sp_groups/updown/'.$myGroupid);
	}

	/**
	 * グループ入力拡張属性追加時にグループ管理者にメールを出す
	 * @param $spcid    SP コネクタ ID
	 * @param $beforeExAttribute    セーブ前の拡張属性
	 * @param $beforeExAttribute    セーブ後の拡張属性
	 * @param $afterTargetSp    対象SP
	 */
	private function _sendMailGroupExtended($spcid, $beforeExAttribute, $afterExAttribute, $afterTargetSp) {
		if (empty($afterExAttribute)) { // グループで入力する属性無し
			$this->log('_sendMailGroupExtended : empty $afterExAttribute', "debug");
			return true;
		}
		$spc = $this->Group->find('first', array('conditions' => array('id' => $spcid)));
		$Mail = array();
		$Mail['spcNameJp'] = $this->Group->getGroupNameJp($spc);
		$Mail['spcNameEn'] = $this->Group->getGroupNameEn($spc);

		// 追加された拡張属性を設定
		$Mail['attributeList'] = array();
		$changeAttributeList = array();
		$list = $this->ExtendedAttribute->find('list', array('fields' => array('id', 'id'),
			'conditions' => array('spc_id' => $spcid, 'kind' => 1)));
		foreach ($beforeExAttribute as $bAttr) {
			$id = $bAttr['ExtendedAttribute']['id'];
			if (!in_array($id, $list)) {    // 同じ id のデータがない
				continue;
			}
			$list[$id] = 0;
		}
		foreach ($list as $aID) {
			if ($aID != 0) {    // 追加された属性
				foreach ($afterExAttribute as $aAttr) {
					if ($aID != $aAttr['ExtendedAttribute']['id']) {
						continue;
					}
					array_push($Mail['attributeList'], $aAttr['ExtendedAttribute']['friendly_name']);
				}
			}
		}
		if (empty($Mail['attributeList'])) { // 追加されたグループで入力する属性無し
			$this->log('_sendMailGroupExtended : empty attributeList', "debug");
			return true;
		}

		// 対象SPを設定
		// サービス名を検索
		$spHostList = $this->SpHost->find('all', array('conditions' => array('id' => $afterTargetSp)));
		$this->log('_sendMailGroupExtended : $spHostList = ', "debug");
		$this->log($spHostList, "debug");
		$Mail['serviceNameListJp'] = array();
		$Mail['serviceNameListEn'] = array();
		foreach ($spHostList as $host) {
			array_push($Mail['serviceNameListJp'], $host['SpHost']['service_name']);
			array_push($Mail['serviceNameListEn'], $host['SpHost']['service_name_en']);
			$this->log('_sendMailGroupExtended : service_name = ' . $host['SpHost']['service_name'] , "debug");
		}

		// SPコネクタに接続している上位グループIDを検索
		$groupIdList = $this->AddGroup->find('list', array(
			'fields' => array('bottom_id', 'bottom_id'),
			'conditions' => array('top_id' => $spcid, 'top' => 1)));
		$groupIdList = $this->AutoGroup->getGroupIdListWithOutIdPGroup($groupIdList);
		if (empty($groupIdList)) {
			$this->log('_sendMailGroupExtended : empty $groupIdList', "debug");
			return true;
		}

		// グループを検索
		$groupList = $this->Group->find('all', array(
			'fields' => array('id', 'name', 'name_en'),
			'conditions' => array('id' => $groupIdList)));

		// グループ管理者へメールする
		foreach ($groupIdList as $groupid) {
			$Mail['url'] = Router::url('/admin/manage_groups/extended/', true) . $spcid . '/' . $groupid;
			foreach ($groupList as $group) {
				if ($groupid === $group['Group']['id']) {
					if (!empty($group['Group']['name']))
						$Mail['groupNameJp'] = $group['Group']['name'];
					else
						$Mail['groupNameJp'] = $group['Group']['name_en'];

					if (!empty($group['Group']['name_en']))
						$Mail['groupNameEn'] = $group['Group']['name_en'];
					else
						$Mail['groupNameEn'] = $group['Group']['name'];
					break;
				}
			}

			$this->log('_sendMailGroupExtended : $Mail = ', "debug");
			$this->log($Mail, "debug");
			$this->sendMailAllAdmin($groupid, 'enter.extended.attribute', 'request_enter_extended_attribute', $Mail);
		}
		return true;
	}
}
?>
