<?php
/**
 * グループへ入会・退会申請があることを管理者にメールする。
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 *
 * 実行例）
 * http://localhost/map/batch_inspects/batch?mapurl=http%3A%2F%2Flocalhost%2Fmap%2F
 * https://map.gakunin.nii.ac.jp/map/batch_inspects/batch?mapurl=https%3a%2f%2fmap%2egakunin%2enii%2eac%2ejp%2fmap%2f
 * https://test-map.gakunin.nii.ac.jp/map/batch_inspects/batch?mapurl=https%3a%2f%2ftest%2dmap%2egakunin%2enii%2eac%2ejp%2fmap%2f
 */
class BatchInspectsController extends AppController {
	var $name = 'BatchInspects';
	var $uses = array('ManageInspect', 'Mygroup', 'Account', 'Group');

	function beforeFilter() {
		// サーバ時間からのアクセスのみを許可する
		if ($_SERVER['REMOTE_ADDR'] != $_SERVER['SERVER_ADDR']) {
			$this->cakeError('cannotAccess', array());
			return;
		}
		$this->autoRender = false;
	}


	function batch() {
		if ($this->params['url']['url'] != 'batch_inspects/batch') {
			return;
		}

		$url = $this->params['url']['mapurl'];
		$this->execute($url, 0);
		$this->execute($url, 1);
	}

	private function execute($url, $kind) {
		// 対象の入会データを取得
		$inspectList = $this->ManageInspect->find('all', array(
			'fields' => array('groupid', 'id'),
			'conditions' => array(
				'status' => '0', 'kind' => $kind, 'sent' => 0 )
		));

		// ２重配送を防止するため、ここで取得データを送信済みにする
		$idList = array();
		$groupIdList = array();
		foreach($inspectList as $ipspect) {
			$id = $ipspect['ManageInspect']['id'];
			$groupid = $ipspect['ManageInspect']['groupid'];
			array_push($idList, $id);
			array_push($groupIdList, $groupid);
		}
		$idList = array_unique($idList);
		$groupIdList = array_unique($groupIdList);
		$this->ManageInspect->updateAll(array('sent' => 1), array('id' => $idList));


		foreach ($groupIdList as $groupid) {
			// SPコネクタはメール送信しない
			$group = $this->Group->find('first', array('fields' => array('id', 'sp', 'name', 'name_en'),
				'conditions' => array('id' => $groupid)));
			if ($group['Group']['sp'] != 0) {
				continue;
			}

			// グループの管理者を取得
			$myGroupList = $this->Mygroup->find('all', array(
				'fields' => array('account_id', 'groupid', 'admin'),
				'conditions' => array('groupid' => $groupid, 'admin >=' => 1)
			));

			$accountIdList = array();
			foreach($myGroupList as $mygroup) {
				array_push($accountIdList, $mygroup['Mygroup']['account_id']);
			}
			if (count($accountIdList) == 0)
				continue;

			// 管理者のメールアドレスを取得
			$accountList = $this->Account->find('all', array('fields' => array('mail', 'language'),
				'conditions' => array('id' => $accountIdList)));

			// グループ名を設定
			$gnameJp = $group['Group']['name'];
			$gnameEn = $group['Group']['name_en'];
			if (empty($group['Group']['name'])) {
				$gnameJp = $group['Group']['name_en'];
			}
			if (empty($group['Group']['name_en'])) {
				$gnameEn = $group['Group']['name'];
			}
			$Mail['gnameJp'] = $gnameJp;
			$Mail['gnameEn'] = $gnameEn;

			// メール送信
			if ($kind == 0) {
				$title = "join.group";
				$Mail['url'] = $url."admin/manage_inspects/join/".$groupid;
				$template = 'inspect_join_group_template';
			} else {
				$title = "quit.group";
				$Mail['url'] = $url."admin/manage_inspects/quit/".$groupid;
				$template = 'inspect_quit_group_template';
			}

			$this->sendMailAllAdmin($groupid, $title, $template, $Mail);
		}
	}
}
?>
