<?php
/**
 * AutoGroupCreator Componet
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class AutoGroupCreatorComponent extends Object
{
	var $Group;
	var $AutoGroup;
	var $Transaction;
	var $IdpAdministrator;
	var $AccountEppn;
	var $Mygroup;
	var $AutoGroupKeySequence;
	var $AccountAttribute;

	function initialize(&$controller) {
		$this->controller =& $controller;
		$this->Group = ClassRegistry::init('Group');
		$this->AutoGroup = ClassRegistry::init('AutoGroup');
		$this->Transaction = ClassRegistry::init('Transaction');
		$this->IdpAdministrator = ClassRegistry::init('IdpAdministrator');
		$this->AccountEppn = ClassRegistry::init('AccountEppn');
		$this->Mygroup = ClassRegistry::init('Mygroup');
		$this->AutoGroupKeySequence = ClassRegistry::init('AutoGroupKeySequence');
		$this->AccountAttribute = ClassRegistry::init('AccountAttribute');
	}

	/**
	 * o属性、ou属性からグループを自動的に作成する。
	 * @param $o
	 * @param $jao
	 * @param $ou
	 * @param $jaou
	 * @param $entityid
	 */
	function service($accountid, $o, $jao, $ou, $jaou, $entityid) {

		/* Nothing */
	}

	/**
	 * 自動グループ入会の同意が必要かどうかを確認します。
	 * 同意が必要な場合は、そのグループIDを返します。
	 * @return string グループID
	 */
	function servNeedAgree($accountid, $o, $jao, $ou, $jaou, $entityid, $eppn, $eptid) {
		$o = $this->toCaseIgnoreMatch($o);
		$jao = $this->toCaseIgnoreMatch($jao);
		$ou = $this->toCaseIgnoreMatch($ou);
		$jaou = $this->toCaseIgnoreMatch($jaou);

		// O/OUグループ管理者を管理者＋メンバー権限に変更する。
		$this->updateAdmin($accountid, $entityid, $eppn, $eptid);

		$oGroupId = $this->isNeedJoinGroup($accountid, 'o', $o, $entityid);
		$ouGroupId = $this->isNeedJoinGroup($accountid, 'ou', $ou, $entityid);
		if (!$oGroupId && !$ouGroupId) {
			return false;
		}

		$id = '/';
		if ($oGroupId)
			$id = $id.$oGroupId.'/';
		if ($ouGroupId)
			$id = $id.$ouGroupId.'/';

		return $id;
	}

	/**
	 * O/OUグループ管理者のアクセス時、メンバー権限になっていたら管理者＋メンバー権限に変更する。
	 */
	private function updateAdmin($accountid, $entityid, $eppn, $eptid) {
		$count = 0;
		if (!empty($eppn)) {
			$count = $this->IdpAdministrator->find('count', array(
					'conditions' => array('entityid' => $entityid, 'eppn' => $eppn)));
		}
		if ($count <= 0 && !empty($eptid)) {
			$count = $this->IdpAdministrator->find('count', array(
					'conditions' => array('entityid' => $entityid, 'eptid' => $eptid)));
		}
		if ($count <= 0) {
			return;
		}

		// 自動グループを検索
		$autoGroupIdList = $this->AutoGroup->find('list', array(
			'fields' => array('group_id'),
			'conditions' => array('entityid' => $entityid)));
		if (empty($autoGroupIdList)) {
			return;
		}

		// 存在する自動グループのIDを検索
		$groupIdList = $this->Group->find('list', array('fields' => array('id', 'id'),
			'conditions' => array('id' => $autoGroupIdList, 'active' => 0)));
		if (empty($groupIdList)) {
			return;
		}

		$mygroupIdList = $this->Mygroup->find('list', array(
				'fields' => array('id'),
				'conditions' => array(
					'account_id' => $accountid, 'groupid' => $groupIdList, 'admin' => 0)));
		foreach ($mygroupIdList as $id) {
			$mygroup = array(
				'id' => $id,
				'admin' => 2,
			);
			$this->Mygroup->save($mygroup);
		}
	}

	/**
	 * 自動的にグループに入会させます。
	 */
	function servAutoJoinGroup($accountid, $o, $jao, $ou, $jaou, $entityid) {
		$o = $this->toCaseIgnoreMatch($o);
		$jao = $this->toCaseIgnoreMatch($jao);
		$ou = $this->toCaseIgnoreMatch($ou);
		$jaou = $this->toCaseIgnoreMatch($jaou);

		// o属性、ou属性の取得、保存
		$beforeAttribute = $this->AccountAttribute->find('first', array('conditions' => array(
			'account_id' => $accountid, 'entityid' => $entityid)));

		$afterAttibute = array();
		if (!empty($beforeAttribute)) {
			$afterAttibute['AccountAttribute']['id'] = $beforeAttribute['AccountAttribute']['id'];
		}
		$afterAttibute['AccountAttribute']['account_id'] = $accountid;
		$afterAttibute['AccountAttribute']['entityid'] = $entityid;
		if (!empty($o))
			$afterAttibute['AccountAttribute']['o'] = $o;
		if (!empty($ou))
			$afterAttibute['AccountAttribute']['ou'] = $ou;

		$this->AccountAttribute->create();
		$result = $this->AccountAttribute->save($afterAttibute);
		if (!$result) {
			return;
		}

		// o属性グループの入会・退会
		if (empty($beforeAttribute) && !empty($o)) {
			// o属性のグループに入会する
			$this->joinGroup($accountid, 'o', $o, $entityid);
		} else if ($beforeAttribute['AccountAttribute']['o'] === $o) {
			// AccountAttributeにデータがあっても既に入会しているかは分からない(重複グループ名があるときなど)
			$this->joinGroup($accountid, 'o', $o, $entityid);
		} else if ($beforeAttribute['AccountAttribute']['o'] != $o) {
			// 前回のo属性に関連するグループから退会させる
			$this->quitGroup($accountid, 'o', $entityid, $beforeAttribute);
			// 今回のo属性のグループに入会させる
			$this->joinGroup($accountid, 'o', $o, $entityid);
		}

		// ou属性グループの入会・退会
		if (empty($beforeAttribute) && !empty($ou)) {
			// ou属性のグループに入会する
			$this->joinGroup($accountid, 'ou', $ou, $entityid);
		} else if ($beforeAttribute['AccountAttribute']['ou'] === $ou) {
			// AccountAttributeにデータがあっても既に入会しているかは分からない(重複グループ名があるときなど)
			$this->joinGroup($accountid, 'ou', $ou, $entityid);
		} else if ($beforeAttribute['AccountAttribute']['ou'] != $ou) {
			// 前回のo属性に関連するグループから退会させる
			$this->quitGroup($accountid, 'ou', $entityid, $beforeAttribute);
			// 今回のo属性のグループに入会させる
			$this->joinGroup($accountid, 'ou', $ou, $entityid);
		}
	}

	/**
	 * caseIgnoreMatch(LDAP)の定義に当てはまる値を返します。
	 * 英大小文字の区別なし(MySQLのSELECTでは区別しないので何もしない)
	 * 先頭、末尾の空白を無視
	 * 連続する空白は、単一の空白とみなす
	 */
	private function toCaseIgnoreMatch($value) {
		return $this->AutoGroup->toCaseIgnoreMatch($value);
	}

	/**
	 * 該当グループに入会する
	 */
	private function joinGroup($accountid, $attributeName, $attributeValue, $entityid) {
		$autoGroup = $this->AutoGroup->find('first', array(
			'conditions' => array('entityid' => $entityid, $attributeName => $attributeValue)));
		if (empty($autoGroup)) {
			return;
		}

		$count = $this->Mygroup->find('count', array('conditions' => array(
			'account_id' => $accountid,
			'groupid' => $autoGroup['AutoGroup']['group_id'])));
		if ($count >= 1) {
			// 既に入会している
			return;
		}

		$mygroup = array(
			'Mygroup' => array(
				'account_id' => $accountid,
				'groupid' => $autoGroup['AutoGroup']['group_id'],
				'admin' => 0,
			),
		);
		$this->Mygroup->create();
		$this->Mygroup->save($mygroup);
	}

	/**
	 * グループに入会する必要があるかを確認します。
	 * 入会する必要がある場合は、そのグループIDを返します。
	 */
	private function isNeedJoinGroup($accountid, $attributeName, $attributeValue, $entityid) {
		$autoGroup = $this->AutoGroup->find('first', array(
			'conditions' => array('entityid' => $entityid, $attributeName => $attributeValue)));
		if (empty($autoGroup)) {
			return false;
		}

		// 存在する自動グループのIDを検索
		$group = $this->Group->find('first', array(
			'fields' => array('id'),
			'conditions' => array('id' => $autoGroup['AutoGroup']['group_id'], 'active' => 0)));
		if (empty($group)) {
			return false;
		}

		$count = $this->Mygroup->find('count', array('conditions' => array(
			'account_id' => $accountid,
			'groupid' => $group['Group']['id'])));
		if ($count >= 1) {
			return false;
		}

		return $group['Group']['id'];
	}

	/**
	 * 該当グループから退会する
	 */
	private function quitGroup($accountid, $attributeName, $entityid, $beforeAttribute) {
		$beforeAutoGroup = $this->AutoGroup->find('first', array('conditions' => array(
			'entityid' => $entityid,
			$attributeName => $beforeAttribute['AccountAttribute'][$attributeName])));
		if (empty($beforeAutoGroup)) {
			return;
		}
		$mygroup = $this->Mygroup->find('first', array('conditions' => array(
			'account_id' => $accountid, 'groupid' => $beforeAutoGroup['AutoGroup']['group_id'])));
		if (empty($mygroup)) {
			// 入会していない
			return;
		}
		// 退会
		$this->Mygroup->delete($mygroup['Mygroup']['id']);
	}

	private function transactionCreateGroup($name_en, $name, $entityid, $attributeName, $groupNameEn, $groupNameJa) {
		$this->Transaction->begin();

		if ($attributeName === 'o' || $attributeName === 'ou') {
			if (!$this->createGroup($name_en, $name, $entityid, $attributeName, $groupNameEn, $groupNameJa)) {
				$this->Transaction->rollback();
			}
		}
// 		// idpグループの作成
// 		else if ($attributeName === 'idp') {
// 			$result = $this->createIdPGroup($name_en, $name, $entityid, $attributeName, $groupNameEn, $groupNameJa);
// 			if (!$result) {
// 				$this->Transaction->rollback();
// 			}
// 		}

		$this->Transaction->commit();
	}


// 	/**
// 	 * IdPグループの作成
// 	 * @param unknown $name_en
// 	 * @param unknown $name
// 	 * @param unknown $entityid
// 	 * @param unknown $attributeName
// 	 * @param unknown $groupNameEn
// 	 * @param unknown $groupNameJa
// 	 * @return boolean
// 	 */
// 	private function createIdPGroup($name_en, $name, $entityid, $attributeName, $groupNameEn, $groupNameJa) {
// 		// IdP管理者の検索
// 		$idpAdminEppnList = $this->IdpAdministrator->find('list', array(
// 				'fields' => array('eppn', 'eppn'),
// 				'conditions' => array('entityid' => $entityid)));
// 		$accountIdList = array();
// 		if (empty($idpAdminEppnList)) {
// 			$this->log('AutoGroupCreatorComponent IdP Administrator does not exist.');
// 		} else {
// 			$accountIdList = $this->AccountEppn->find('list', array(
// 					'fields' => array('account_id', 'account_id'),
// 					'conditions' => array('eppn' => $idpAdminEppnList)));
// 		}
// 		if (empty($accountIdList)) {
// 			// SPコネクタの管理者がいないので、システム管理者がSPコネクタの管理者となる
// 			$accountIdList = $this->Mygroup->getSystemAdminAccountIdList();
// 			if (empty($accountIdList)) {
// 				$this->log('AutoGroupCreatorComponent System Administrator does not exist.', "error");
// 				return false;
// 			}
// 		}

// 		// group_keyを生成
// 		$group_key = '';
// 		for ($i = 0; $i < 20; $i++) {
// 			$newid = $this->AutoGroupKeySequence->findNextId();
// 			if (strlen($newid) == 1) {
// 				$newid = '00'.$newid;
// 			} elseif (strlen($newid) == 2) {
// 				$newid = '0'.$newid;
// 			}
// 			$group_key = $attributeName.'-'.$newid;
// 			$count = $this->Group->find('count', array('conditions' => array(
// 					'group_key' => $group_key, 'name' => $group_key, 'name_en' => $group_key)));
// 			if ($count == 0) {
// 				break;
// 			}
// 			if ($i == 10) {
// 				$this->AutoGroupKeySequence->upId();
// 			}
// 		}

// 		// Group
// 		$groupName = $groupNameJa;
// 		if (!empty($groupName) && !empty($groupNameEn) && $groupName != $groupNameEn) {
// 			$groupName = $groupName + '/' + $groupNameEn;
// 		}
// 		$group = array(
// 				'Group' => array(
// 						'group_key' => $group_key,
// 						'name' => $groupName,
// 						'active' => 0,
// 						'public' => 1,
// 						'openmember' => 1,
// 						'inspectjoin' => 1,
// 						'inspectquit' => 1,
// 						'independence' => 1,
// 						'down_permission' => 0,
// 						'up_permission' => 0,
// 						'group_response' => 0,
// 						'sp' => 0,
// 				),
// 		);
// 		$group['Group']['introduction'] = __("message.auto.group.idp.introduction.ja", true)
// 		.chr(13).chr(10).__("message.auto.group.idp.introduction.eng", true);

// 		$this->Group->create();
// 		$result = $this->Group->save($group);
// 		if (!$result) {
// 			$error = $this->controller->validateErrors($this->Group);
// 			$this->log('AutoGroupCreatorComponent Group Error', "error");
// 			$this->log($error, "error");
// 			$this->log($group, "error");
// 			return false;
// 		}
// 		$groupId = $this->Group->getLastInsertID();

// 		// Mygroup
// 		foreach ($accountIdList as $accountid) {
// 			$mygroup = array(
// 					'Mygroup' => array(
// 							'account_id' => $accountid,
// 							'groupid' => $groupId,
// 							'admin' => 1, // 自動グループは管理者のみの権限とする
// 					),
// 			);
// 			$this->Mygroup->create();
// 			$result = $this->Mygroup->save($mygroup);
// 			if (!$result) {
// 				$error = $this->controller->validateErrors($this->Mygroup);
// 				$this->log('AutoGroupCreatorComponent Mygroup Error', "error");
// 				$this->log($error, "error");
// 				return false;
// 			}
// 		}

// 		// AutoGroup
// 		$autoGrup = array(
// 				'AutoGroup' => array(
// 						'idp' => 'true',
// 						'entityid' => $entityid,
// 						'group_id' => $groupId,
// 						'status' => 'active',
// 				),
// 		);
// 		$this->AutoGroup->create();
// 		$result = $this->AutoGroup->save($autoGrup);
// 		if (!$result) {
// 			$error = $this->controller->validateErrors($this->AutoGroup);
// 			$this->log('AutoGroupCreatorComponent AutoGroup Error', "error");
// 			$this->log($error, "error");
// 			return false;
// 		}

// 		return true;
// 	}

	/**
	 *
	 * @param string $name_en
	 * @param string $name
	 * @param string $attributeName o or ou
	 * @param string $entityid
	 * @param string $groupNameEn
	 * @param string $groupNameJa
	 */
	private function createGroup($name_en, $name, $entityid, $attributeName, $groupNameEn, $groupNameJa) {
		// IdP管理者の検索
		$idpAdmin = $this->IdpAdministrator->find('first', array('conditions' => array(
			'entityid' => $entityid)));
		if (empty($idpAdmin)) {
			$idpAdmin = $this->IdpAdministrator->find('first', array('conditions' => array(
				'entityid' => $entityid)));
		}
		if (empty($idpAdmin)) {
			$this->log('AutoGroupCreatorComponent Administrator does not exist.', "error");
			return false;
		}
		// group_keyを生成
		$group_key = '';
		for ($i = 0; $i < 20; $i++) {
			$newid = $this->AutoGroupKeySequence->findNextId();
			if (strlen($newid) == 1) {
				$newid = '00'.$newid;
			} elseif (strlen($newid) == 2) {
				$newid = '0'.$newid;
			}
			$group_key = $attributeName.'-'.$newid;
			$count = $this->Group->find('count', array('conditions' => array(
				'group_key' => $group_key, 'name' => $group_key, 'name_en' => $group_key)));
			if ($count == 0) {
				break;
			}
			if ($i == 10) {
				$this->AutoGroupKeySequence->upId();
			}
		}

		// Group
		$group = array(
			'Group' => array(
				'group_key' => $group_key,
				'name' => $groupNameJa,
				'name_en' => $groupNameEn,
				'active' => 0,
				'public' => 1,
				'openmember' => 1,
				'inspectjoin' => 1,
				'inspectquit' => 0,
				'independence' => 0, // oとouで上下関係を作るので独立させない
				'down_permission' => 0,
				'up_permission' => 0,
				'group_response' => 0,
				'sp' => 0,
			),
		);
		if ($attributeName === 'o') {
			$group['Group']['introduction'] = __("message.auto.group.o.introduction.ja", true)
				.chr(13).chr(10).__("message.auto.group.o.introduction.eng", true);
		}
		else if ($attributeName === 'ou') {
			$group['Group']['introduction'] = __("message.auto.group.ou.introduction.ja", true)
				.chr(13).chr(10).__("message.auto.group.ou.introduction.eng", true);
		}

		$this->Group->create();
		$result = $this->Group->save($group);
		if (!$result) {
			$error = $this->controller->validateErrors($this->Group);
			$this->log('AutoGroupCreatorComponent Group Error', "error");
			$this->log($error, "error");
			$this->log($group, "error");
			return false;
		}

		$groupId = $this->Group->getLastInsertID();
		$eppn = $idpAdmin['IdpAdministrator']['eppn'];
		$eptid = $idpAdmin['IdpAdministrator']['eptid'];
		$accountid = '';
		if (!empty($eppn)) {
			$accountEppn = $this->AccountEppn->find('first', array('conditions' => array('eppn' => $eppn)));
			if (!empty($accountEppn)) {
				$accountid = $accountEppn['AccountEppn']['account_id'];
			}
		}
		if (empty($accountid) && !empty($eptid)) {
			$accountEppn = $this->AccountEppn->find('first', array('conditions' => array('eptid' => $eptid)));
			if (!empty($accountEppn)) {
				$accountid = $accountEppn['AccountEppn']['account_id'];
			}
		}
		if (empty($accountid)) {
			$this->log('AutoGroupCreatorComponent Administrator does not exist.', "error");
			return false;
		}

		// Mygroup
		$mygroup = array(
			'Mygroup' => array(
				'account_id' => $accountid,
				'groupid' => $groupId,
				'admin' => 1, // 自動グループは管理者のみの権限とする
			),
		);
		$this->Mygroup->create();
		$result = $this->Mygroup->save($mygroup);
		if (!$result) {
			$error = $this->controller->validateErrors($this->Mygroup);
			$this->log('AutoGroupCreatorComponent Mygroup Error', "error");
			$this->log($error, "error");
			return false;
		}

		// AutoGroup
		if ($attributeName === 'o' || $attributeName === 'ou') {
			$autoGrup = array(
				'AutoGroup' => array(
					$attributeName => $name_en, // ['AutoGroup']['o'] or ['AutoGroup']['ou']
					'entityid' => $entityid,
					'group_id' => $groupId,
					'status' => 'active',
				),
			);
		}

		$this->AutoGroup->create();
		$result = $this->AutoGroup->save($autoGrup);
		if (!$result) {
			$error = $this->controller->validateErrors($this->AutoGroup);
			$this->log('AutoGroupCreatorComponent AutoGroup Error', "error");
			$this->log($error, "error");
			return false;
		}

		return true;
	}

	private function validateAttribute($accountid, $value, $entityid) {
		if (empty($accountid)) {
			return false;
		}
		if (strlen($value) == 0) {
			return false;
		}

		// entityIDに対応する管理者が登録されていなかったらグループは作成しない
		$count = $this->IdpAdministrator->find('count', array(
			'conditions' => array('entityid' => $entityid)));
		if ($count <= 0 ) {
			return false;
		}

		return true;
	}

	private function isExistAutoGroupO($o) {
		$count = $this->AutoGroup->find('count', array(
			'conditions' => array('o' => $o, 'status' => 'active')));
		if ($count >= 1) {
			return true;
		}
		return false;
	}

	private function isExistAutoGroupOU($ou) {
		$count = $this->AutoGroup->find('count', array(
			'conditions' => array('ou' => $ou, 'status' => 'active')));
		if ($count >= 1) {
			return true;
		}
		return false;
	}


// 	private function _isExistAutoGroupIdP($entityid) {
// 		$count = $this->AutoGroup->find('count', array(
// 			'conditions' => array('entityid' => $entityid, 'idp' => 'true', 'status' => 'active')));
// 		if ($count >= 1) {
// 			return true;
// 		}
// 		return false;
// 	}
}
?>
