<?php
/**
 * Extend the EmailComponent
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Component', 'Email');

class EmailJpComponent extends EmailComponent {
	function _render($content) {
		$msg = parent::_render($content);
		mb_convert_variables($this->charset, Configure::read('App.encoding'), $msg);
		return $msg;
	}

	function _smtp() {
		App::import('Core', array('CakeSocket'));

		$defaults = array(
			'host' => 'localhost',
			'port' => 25,
			'protocol' => 'smtp',
			'timeout' => 30
		);
		$this->smtpOptions = array_merge($defaults, $this->smtpOptions);
		$this->__smtpConnection =& new CakeSocket($this->smtpOptions);

		if (!$this->__smtpConnection->connect()) {
			$this->smtpError = $this->__smtpConnection->lastError();
			return false;
		} elseif (!$this->_smtpSend(null, '220')) {
			return false;
		}

		$httpHost = env('HTTP_HOST');

		if (isset($this->smtpOptions['client'])) {
			$host = $this->smtpOptions['client'];
		} elseif (!empty($httpHost)) {
			$host = $httpHost;
		} else {
			$host = 'localhost';
		}

		if (!$this->_smtpSend("EHLO {$host}", '250') && !$this->_smtpSend("HELO {$host}", '250')) {
			return false;
		}

		if (isset($this->smtpOptions['username']) && isset($this->smtpOptions['password'])) {
			$authRequired = $this->_smtpSend('AUTH LOGIN', '334|503');
			if ($authRequired == '334') {
				if (!$this->_smtpSend(base64_encode($this->smtpOptions['username']), '334')) {
					return false;
				}
				if (!$this->_smtpSend(base64_encode($this->smtpOptions['password']), '235')) {
					return false;
				}
			} elseif ($authRequired != '503') {
				return false;
			}
		}
		if (!$this->_smtpSend('MAIL FROM: ' . $this->_formatAddress($this->from, true))) {
			return false;
		}

		if (!is_array($this->to)) {
			$tos = array($this->to);
		} else {
			$tos = $this->to;
		}

		$rcptok = false;
		foreach ($tos as $to) {
			if (empty($to)) {
				continue;
			}
			if (!$this->_smtpSend('RCPT TO: ' . $this->_formatAddress($to, true))) {
				if (strpos($this->__header[0], $to.', ')) {
					$this->__header[0] = str_replace($to.', ', '', $this->__header[0]);
				} else if (strpos($this->__header[0], $to)) {
					$this->__header[0] = str_replace($to, '', $this->__header[0]);
				}
			} else {
				$rcptok = true;
			}
		}
		if (!$rcptok) {
			return false;
		}

		foreach ($this->cc as $cc) {
			if (!$this->_smtpSend('RCPT TO: ' . $this->_formatAddress($cc, true))) {
				return false;
			}
		}
		foreach ($this->bcc as $bcc) {
			if (!$this->_smtpSend('RCPT TO: ' . $this->_formatAddress($bcc, true))) {
				return false;
			}
		}

		if (!$this->_smtpSend('DATA', '354')) {
			return false;
		}

		$header = implode("\r\n", $this->__header);
		$message = implode("\r\n", $this->__message);
		if (!$this->_smtpSend($header . "\r\n\r\n" . $message . "\r\n\r\n\r\n.")) {
			return false;
		}
		$this->_smtpSend('QUIT', false);

		$this->__smtpConnection->disconnect();
		return true;
	}
}
?>