<?php
/**
 * IdPTranslator Componet
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class IdpTranslatorComponent extends Object
{
	var $AccountEppn;
	var $Organization;

	function startup() {
		$this->AccountEppn = ClassRegistry::init('AccountEppn');
		$this->Organization = ClassRegistry::init('Organization');
	}

	/**
	 * 組織名を設定する。
	 * @param $members ユーザ情報一覧(Account)
	 * @param $language 使用言語
	 * @return 組織名が設定されたユーザ情報一覧
	 */
	function setIdP(&$members, $language) {
		// entityidを追加する
		$accountIdList = array();
		foreach ($members as $member) {
			array_push($accountIdList, $member['Account']['id']);
		}
		$accountEppnList = $this->AccountEppn->find('all', array(
			'fields' => array('account_id', 'entityid'),
			'conditions' => array('account_id' => $accountIdList)));

		$organizations = $this->Organization->find('all', array('fields' => array('id', 'entityid', 'display_name', 'display_name_en')));
		for($i = 0; $i < count($members); $i++) {
			$members[$i]['Account']['idp'] = '';
			foreach ($accountEppnList as $eppn) {
				if ($eppn['AccountEppn']['account_id'] != $members[$i]['Account']['id']) {
					continue;
				}
				foreach($organizations as $organization) {
					if ($eppn['AccountEppn']['entityid'] != $organization['Organization']['entityid']) {
						continue;
					}
					if ($language === 'ja') {
						$display = $organization['Organization']['display_name'];
					} else {
						$display= $organization['Organization']['display_name_en'];
					}
					$match = preg_match('/^'.$display.'|[ ]'.$display.'/', $members[$i]['Account']['idp']);
					if ($match) {
						continue;
					}

					if (!empty($members[$i]['Account']['idp'])) {
						$display = ', '.$display;
					}

					$members[$i]['Account']['idp'] = $members[$i]['Account']['idp'].$display;
				}
			}
		}
	}

	/**
	 * アカウントEPPN情報にIdP名を付加する。
	 * @param $accoutEppnList
	 * @param $language
	 */
	function appendIdp(&$accoutEppnList, $language) {
		$organizations = $this->Organization->find('all', array('fields' => array('id', 'entityid', 'display_name', 'display_name_en')));
		for ($i = 0; $i < count($accoutEppnList); $i++) {
			foreach($organizations as $organization) {

				if ($accoutEppnList[$i]['AccountEppn']['entityid'] === $organization['Organization']['entityid']) {
					if ($language === 'ja') {
						$display = $organization['Organization']['display_name'];
					} else {
						$display= $organization['Organization']['display_name_en'];
					}
					$accoutEppnList[$i]['AccountEppn']['idp'] = $display;
					break;
				}
			}
		}
	}

	/**
	 * アカウントEPPN情報にIdP名（日英）を付加する。
	 * @param $accoutEppnList
	 * @param $language
	 */
	function appendIdpName(&$accoutEppnList) {
		$organizations = $this->Organization->find('all', array('fields' => array('id', 'entityid', 'display_name', 'display_name_en')));
		for ($i = 0; $i < count($accoutEppnList); $i++) {
			foreach($organizations as $organization) {

				if ($accoutEppnList[$i]['AccountEppn']['entityid'] === $organization['Organization']['entityid']) {
					$accoutEppnList[$i]['AccountEppn']['idpJa'] = $organization['Organization']['display_name'];
					$accoutEppnList[$i]['AccountEppn']['idpEn'] = $organization['Organization']['display_name_en'];
					break;
				}
			}
		}
	}
}
?>
