<?php
/**
 * NoticeUtils Componet
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2021, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class NoticeUtilsComponent extends Object
{
	var $Group;
	var $Mygroup;
	var $AccountEppn;
	var $GroupExtendedValue;
	var $AddGroup;
	var $AddGroupsSpcLog;
	var $Account;
	var $ManageInspect;
	var $Agreement;
	var $AgreementIsmemberof;
	var $AgreementEntrustment;
	var $SentNoticeMailLog;
	var $ManageGroup;
	var $AccountIntegration;
	var $SpAccess;
	var $SpcSort;
	var $Tag;
	var $UserSpSort;
	var $AddGroupInspect;
	var $AgreementConnect;
	var $IconImage;
	var $InspectAttribute;
	var $Inspect;
	var $Invite;


	function initialize(&$controller) {
		$this->controller =& $controller;
		$this->Group = ClassRegistry::init('Group');
		$this->Mygroup = ClassRegistry::init('Mygroup');
		$this->AccountEppn = ClassRegistry::init('AccountEppn');
		$this->GroupExtendedValue = ClassRegistry::init('GroupExtendedValue');
		$this->AddGroup = ClassRegistry::init('AddGroup');
		$this->AddGroupsSpcLog = ClassRegistry::init('AddGroupsSpcLog');
		$this->Account = ClassRegistry::init('Account');
		$this->ManageInspect = ClassRegistry::init('ManageInspect');
		$this->Agreement = ClassRegistry::init('Agreement');
		$this->AgreementIsmemberof = ClassRegistry::init('AgreementIsmemberof');
		$this->AgreementEntrustment = ClassRegistry::init('AgreementEntrustment');
		$this->SentNoticeMailLog = ClassRegistry::init('SentNoticeMailLog');
		$this->ManageGroup = ClassRegistry::init('ManageGroup');
		$this->AccountIntegration = ClassRegistry::init('AccountIntegration');
		$this->SpAccess = ClassRegistry::init('SpAccess');
		$this->SpcSort = ClassRegistry::init('SpcSort');
		$this->Tag = ClassRegistry::init('Tag');
		$this->UserSpSort = ClassRegistry::init('UserSpSort');
		$this->AddGroupInspect = ClassRegistry::init('AddGroupInspect');
		$this->AgreementConnect = ClassRegistry::init('AgreementConnect');
		$this->IconImage = ClassRegistry::init('IconImage');
		$this->InspectAttribute = ClassRegistry::init('InspectAttribute');
		$this->Inspect = ClassRegistry::init('Inspect');
		$this->Invite = ClassRegistry::init('Invite');
	}

	/**
	 * 単独管理者グループのリストを返す
	 * @param integer $accountId
	 * @param array() $deleteIdList 削除対象になっているaccount_eppn_idのリスト
	 * @param array() &$soloAdminList 単独管理者グループのid,参加人数,SPや機関グループが含まれるかのリストを返す
	 */
	public function getSoloAdminGroups($accountId, $deleteIdList, &$soloAdminList = array()) {
		$soloAdminGroups = array();
		// グループ管理者チェック（管理者をしているグループ全部のメンバーリスト取得）
		$mygroups = $this->Mygroup->find('all', array(
			'fields' => array('groupid', 'account_id', 'admin'),
			'conditions' => array(
				"groupid in (select groupid from mygroups where admin in (1,2) and account_id = ?)" => array($accountId),
				'account_id in (select id from accounts where is_temp = 0)',
				'groupid in (select id from groups where sp != 2)'
			),
			'order' => array('groupid')
		));
		if (empty($mygroups)) {	// 管理者になっているグループ無し
			return $soloAdminGroups;
		}
		$groupidList = array();
		$accountIdList = array();
		foreach ($mygroups as $mygroup) {
			$groupidList[] = $mygroup['Mygroup']['groupid'];
			$accountIdList[] = $mygroup['Mygroup']['account_id'];
		}
		$accountIdList = array_unique($accountIdList);
		$eppnList = $this->AccountEppn->find('all', array(
			'fields' => array('id', 'account_id'),
			'conditions' => array("account_id" => $accountIdList),
			'order' => array('id')
		));
		$beforeGroupid = -1;
		foreach ($mygroups as $mygroup) {
			$groupid = $mygroup['Mygroup']['groupid'];
			if ($beforeGroupid == $groupid) {
				continue;
			}
			$beforeGroupid = $groupid;
			$adminCount = array_count_values($groupidList)[$groupid];
			$soloAdmin = array();
			$soloAdmin['id'] = $groupid;
			$soloAdmin['count'] = $adminCount;
			$soloAdmin['special'] = false;
			if ($adminCount == 1) {	// 他にメンバーがいない
				if ($this->Group->isIdPGroup($groupid) || $this->Group->isSpConnector($groupid)) {
					$soloAdminGroups[] = $groupid;
					$soloAdmin['special'] = true;
					$soloAdminList[] = $soloAdmin;
				}
				continue;
			}
			$memberKeys = array_keys($groupidList, $groupid);
			$otherAdmins = array();
			foreach ($memberKeys as $mKey) {
				if ($mygroups[$mKey]['Mygroup']['account_id'] == $accountId) {
					continue;
				}
				if ($mygroups[$mKey]['Mygroup']['admin'] == 1 || $mygroups[$mKey]['Mygroup']['admin'] == 2) {
					$otherAdmins[] = $mygroups[$mKey]['Mygroup']['account_id'];	// 他に管理者がいる
				}
			}
			if (empty($otherAdmins)) {	// 他に管理者がいない
				$soloAdminGroups[] = $groupid;
				$soloAdminList[] = $soloAdmin;
				continue;
			}
			// 複数管理者が居る場合、他の管理者が全員削除対象になっていないか確認
			foreach ($otherAdmins as $oAdmin) {
				$sameAccountKeys = $this->getSameAccountKeys($oAdmin, $eppnList);
				foreach ($sameAccountKeys as $saKey) {
					$idKey = array_search($eppnList[$saKey]['AccountEppn']['id'], $deleteIdList);
					if ($idKey !== false) {	// 削除対象
						$result = array_diff($otherAdmins, array($eppnList[$saKey]['AccountEppn']['account_id']));
						$otherAdmins = array_values($result);
					}
				}
			}
			if (empty($otherAdmins)) {	// 全管理者削除対象
				$soloAdminGroups[] = $groupid;
				$soloAdminList[] = $soloAdmin;
			}
		}
		return $soloAdminGroups;
	}

	/**
	 * 同じアカウントのキーを返します
	 */
	public function getSameAccountKeys($id, $linkedAccounts) {
		$sameAccountKeys = array();
		foreach ($linkedAccounts as $key => $lAccount) {
			if ($lAccount['AccountEppn']['account_id'] == $id) {
				$sameAccountKeys[] = $key;
			}
		}
		return $sameAccountKeys;
	}

	/**
	 * アカウントと関連グループを削除(Transactionは呼ぶ側で対応)
	 * @param integer $accountId
	 * @param bool $manual
	 */
	public function deleteAccountGroup($accountId, $manual = false) {
		// 削除対象アカウント以外に管理者のいないグループ削除
		$mygroupIds = $this->Mygroup->findMyGroupIdList($accountId, array(), false);
		foreach($mygroupIds as $id) {
			$count = $this->Mygroup->find('count', array(
				'conditions' => array('groupid' => $id,
					'account_id in (select id from accounts where is_temp = 0)'
				)
			));
			if ($count > 1) {	// 他にメンバーが居る
				if (!$manual) {	// 手動削除ではない
					continue;
				}
				$count = $this->Mygroup->find('count', array(
					'conditions' => array(
						'groupid' => $id,
						'admin' => array(1, 2),
					)
				));
				if ($count > 1) {	// 他に管理者が居る
					continue;
				}
			}
			// グループ削除
			if (!$this->deleteGroup($id)) {
				$this->log("NoticeUtilsComponent ERROR delete Group(id = $id) delete Error");
				return false;
			}
		}
		if (!$this->deleteAccount($accountId)) {
			$this->log("NoticeUtilsComponent ERROR delete Account(id = $accountId) delete Error");
			return false;
		}
		return true;
	}

	/**
	 * グループを削除(Transactionは呼ぶ側で対応)
	 * @param integer $groupId
	 */
	public function deleteGroup($groupId) {
		// グループ情報は論理削除
		$result = $this->ManageGroup->softDelete($groupId);
		if (!$result) {
			return false;
		}

		// Myグループ情報は物理削除
		$result = $this->Mygroup->deleteAll(array('groupid' => $groupId), false);
		if (!$result) {
			return false;
		}

		$result = $this->GroupExtendedValue->deleteAll(array('group_id' => $groupId), false);
		if (!$result) {
			return false;
		}

		// グループのコネクションは物理削除
		$result = $this->AddGroup->deleteAll(array('OR' => array(
			'top_id' => $groupId, 'bottom_id' => $groupId)), false);
		if (!$result) {
			return false;
		}

		// サービスの接続・解除ログも物理削除
		$result = $this->AddGroupsSpcLog->deleteAll(array('group_id' => $groupId), false);
		if (!$result) {
			return false;
		}

		// 属性送信同意の委任も物理削除
		$result = $this->AgreementEntrustment->deleteAll(array('group_id' => $groupId), false);
		if (!$result) {
			return false;
		}

		// 仮アカウントも物理削除
		$result = $this->Account->deleteAll(array('temp_groupid' => $groupId), false);
		if (!$result) {
			return false;
		}

		// グループ間/SPコネクタ-グループ間接続審査も物理削除
		$result = $this->AddGroupInspect->deleteAll(array('OR' => array(
			'top_id' => $groupId, 'bottom_id' => $groupId)), false);
		if (!$result) {
			return false;
		}

		// 同意要請のグループ伝播も物理削除
		$result = $this->AgreementConnect->deleteAll(array('OR' => array(
			'top_group_id' => $groupId, 'bottom_group_id' => $groupId)), false);
		if (!$result) {
			return false;
		}

		// グループIDに関する属性送信同意も物理削除
		$result = $this->AgreementIsmemberof->deleteAll(array('group_id' => $groupId), false);
		if (!$result) {
			return false;
		}

		// アイコン画像バイナリも物理削除
		$result = $this->IconImage->deleteAll(array('group_id' => $groupId), false);
		if (!$result) {
			return false;
		}

		// 属性送信変更時の通知伝播も物理削除
		$result = $this->InspectAttribute->deleteAll(array('OR' => array(
			'from_group_id' => $groupId, 'to_group_id' => $groupId)), false);
		if (!$result) {
			return false;
		}

		// 入会申請も物理削除
		$result = $this->Inspect->deleteAll(array('groupid' => $groupId), false);
		if (!$result) {
			return false;
		}

		// グループのメンバー招待情報も物理削除
		$result = $this->Invite->deleteAll(array('groupid' => $groupId), false);
		if (!$result) {
			return false;
		}

		// グループ内サービスソート方法も物理削除
		$result = $this->SpcSort->deleteAll(array('groupid' => $groupId), false);
		if (!$result) {
			return false;
		}

		// グループ内でサービスを並べ替えも物理削除
		$result = $this->UserSpSort->deleteAll(array('groupid' => $groupId), false);
		if (!$result) {
			return false;
		}

		return true;
	}

	/**
	 * アカウントを削除(Transactionは呼ぶ側で対応)
	 * @param integer $accountId
	 */
	public function deleteAccount($accountId) {
		$result = $this->Account->delete($accountId);
		if (!$result) {
			return false;
		}

		$result = $this->Mygroup->deleteAll(array('account_id' => $accountId), false);
		if (!$result) {
			return false;
		}

		$idList = $this->AccountEppn->find('list', array('fields' => array('id'),
			'conditions' => array('account_id' => $accountId)));
		$result = $this->AccountEppn->deleteAll(array('id' => $idList), false);
		if (!$result) {
			return false;
		}

		$result = $this->ManageInspect->deleteAll(array('account_id' => $accountId), false);
		if (!$result) {
			return false;
		}

		$result = $this->Agreement->deleteAll(array('account_id' => $accountId), false);
		if (!$result) {
			return false;
		}

		$result = $this->AgreementIsmemberof->deleteAll(array('account_id' => $accountId), false);
		if (!$result) {
			return false;
		}

		$result = $this->AgreementEntrustment->deleteAll(array('account_id' => $accountId), false);
		if (!$result) {
			return false;
		}

		$result = $this->SentNoticeMailLog->deleteAll(array('account_eppn_id' => $idList), false);
		if (!$result) {
			return false;
		}

		$result = $this->AccountIntegration->deleteAll(array('account_id' => $accountId), false);
		if (!$result) {
			return false;
		}

		$result = $this->SpAccess->deleteAll(array('account_id' => $accountId), false);
		if (!$result) {
			return false;
		}

		$result = $this->SpcSort->deleteAll(array('account_id' => $accountId), false);
		if (!$result) {
			return false;
		}

		$result = $this->Tag->deleteAll(array('public' => 0, 'account_id' => $accountId), false);
		if (!$result) {
			return false;
		}

		$result = $this->UserSpSort->deleteAll(array('account_id' => $accountId), false);
		if (!$result) {
			return false;
		}

		return true;
	}
}
?>
