<?php
/**
 * SPC Sortor Component
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class SpcSortorComponent extends Object {

	var $SpAccess;
	var $SpcSort;
	var $GroupAdminSpSort;
	var $UserSpSort;
	var $Mygroup;


	function initialize(&$controller) {
		$this->controller =& $controller;
		$this->SpAccess = ClassRegistry::init('SpAccess');
		$this->SpcSort = ClassRegistry::init('SpcSort');
		$this->GroupAdminSpSort = ClassRegistry::init('GroupAdminSpSort');
		$this->UserSpSort =  ClassRegistry::init('UserSpSort');
		$this->Mygroup = ClassRegistry::init('Mygroup');
	}

	function countUp($userid, $spcid) {
	}

	/**
	 * サービスを並び替えます。
	 * MYグループ用
	 * @param entityid IdPのエンティティID
	 */
	function updateSortForMyGroup($relatedUrlList, $userid, $entityid) {
		foreach($relatedUrlList as $groupid => $spcs) {
			foreach ($spcs as $i => $spc) {
				$url = $spc[1];
				if (empty($url)) {
					continue;
				}
//				$spcid = $spc[2];
//				$url = "../redirect/sp/".$spcid."/".$groupid."/".base64_encode($url);
//				$relatedUrlList[$groupid][$i][1] = $url;
			}
		}

		$sortTypeList = $this->SpcSort->getSortTypeList($userid);
		$resultList = array();
		foreach ($relatedUrlList as $groupid => $relatedurl) {
			$sortedList = $this->sort($groupid, $userid, $relatedurl, $sortTypeList);
			$resultList[$groupid] = $sortedList;
		}

		return $resultList;
	}

	/**
	 * サービスを並び替えます
	 * グループトップ用
	 */
	function updateSortForDetail($groupid, $serviceLinkList, $userid) {
//		foreach ($serviceLinkList as $i => $spc) {
//			$url = $spc[1];
//			if (empty($url)) {
//				continue;
//			}
//			$spcid = $spc[2];
//			$url = "../../redirect/sp/".$spcid."/".base64_encode($url);
//			$serviceLinkList[$i][1] = $url;
//		}

		$resultList = $this->sort($groupid, $userid, $serviceLinkList);
		return $resultList;
	}

	/**
	 * 並べ替える
	 */
	private function sort($groupid, $userid, $serviceLinkList, $sortTypeList = array()) {
		if (empty($sortTypeList)) {
			$sortType = $this->SpcSort->getSortType($userid, $groupid);
		} else {
			if (isset($sortTypeList[$groupid])) {
				$sortType = $sortTypeList[$groupid];
			} else {
				$sortType = 'name';
			}
		}

		if ($sortType === 'access') {
			// アクセス頻度
			return $this->_sortAccessCount($userid, $serviceLinkList);
		} else if ($sortType === 'last') {
			// 最終アクセス順
			return $this->_sortLastAccess($userid, $serviceLinkList);
		} else if ($sortType === 'leave') {
			// おまかせ順(デフォルト) IdP管理者の設定順に表示する
			return $this->_sortLeaveing($groupid, $serviceLinkList);
		} else if ($sortType === 'manual') {
			// 手動順
			return $this->_sortManual($userid, $serviceLinkList, $groupid);
		} else {
			// 名前順
			// 何もしなくても既に名前順になっている
			return $serviceLinkList;
		}
	}

	/**
	 * おまかせ(IdP管理者の設定順)に並び変える
	 */
	private function _sortLeaveing($groupid, $serviceLinkList) {
		$sortList = $this->GroupAdminSpSort->findSpIdListByIdp($groupid);
		if (empty($sortList)) {
			// 設定されてなければ名前順
			return $serviceLinkList;
		}
		return $this->exchange($sortList, $serviceLinkList);

	}

	/**
	 * アクセス頻度順に並び変える
	 */
	private function _sortAccessCount($userid, $serviceLinkList) {
		$spcIdList = $this->SpAccess->findOrderAccessCount($userid);
		return $this->exchange($spcIdList, $serviceLinkList);
	}

	/**
	 * 最終アクセス順に並び変える
	 */
	private function _sortLastAccess($userid, $serviceLinkList) {
		$spcIdList = $this->SpAccess->findOrderLastAccess($userid);
		return $this->exchange($spcIdList, $serviceLinkList);

	}

	/**
	 * 手動順に並び変える
	 */
	private function _sortManual($userid, $serviceLinkList, $groupid) {
		//$spcIdList = $this->UserSpSort->getAllSpIdList($userid);
		$spcIdList = $this->UserSpSort->findSideSpIdList($userid, $groupid);
		return $this->exchange($spcIdList, $serviceLinkList);
	}

	private function exchange($spcIdList, $serviceLinkList) {
		if (empty($spcIdList)) {
			return $serviceLinkList;
		}

		// １度もアクセスしていないSPを付加する
		foreach ($serviceLinkList as $spcid => $serviceLink) {
			$spcIdList[$spcid] = $spcid;
		}

		$resultList = array();
		foreach ($spcIdList as $scpid) {
			if (!isset($serviceLinkList[$scpid])) {
				continue;
			}
			$resultList[$scpid] = $serviceLinkList[$scpid];
		}
		return $resultList;
	}

	/**
	 * SPコネクタのソート順を該当ユーザのレコード全てに保存します。
	 */
	public function updateTypeAll($userid, $type) {
		// mygroupsにはあるが、spc_sortsにないグループのレコードを追加
		$list = $this->Mygroup->findSpcSortNoRecord($userid);
		foreach ($list as $groupid) {
			$this->SpcSort->saveType($userid, $groupid, $type);
		}

		$sortType = "'".$type."'";
		$now = "'".date("Y/m/d h:i:s")."'";
		$this->SpcSort->updateAll(array('sort_type' => $sortType, 'modified' => $now),
				array('account_id' => $userid));
	}

	/**
	 * 該当ユーザのグループ全てに対して、表示順を全て更新します。
	 * @param $userid ユーザID
	 */
	public function updateSideAllGroup($userid, $spcidList) {
		// mygroupsにはあるが、user_sp_sortsにないグループのレコードを追加
		$list = $this->Mygroup->findUserSpSortNoRecord($userid);
		foreach ($list as $groupid) {
			$this->UserSpSort->saveSpcIdList($userid, $groupid, $spcidList);
		}

		$now = "'".date("Y/m/d h:i:s")."'";
		$this->UserSpSort->updateAll(array('spc_id_list' => "'".$spcidList."'", 'modified' => $now),
				array('account_id' => $userid));
	}
}
?>