<?php
/**
 * Internal API Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class InternalApiController extends AppController {
	var $name = 'InternalApi';
	var $components = array('RequestHandler', 'SpcSortor');
	var $helpers = array('Javascript');
	var $uses = array('Account', 'Mygroup', 'Group', 'SpcSort', 'GroupAdminSpSort',
			'UserSpSort', 'AutoGroup', 'SpAccess');

	function beforeFilter() {
//		Configure::write('debug', 0);
		$this->autoRender = false;
		$this->RequestHandler->setContent('json');
		$this->RequestHandler->respondAs('application/json; charset=UTF-8');
		$this->layout = "api";
	}

	/**
	 * CGトップ画面にてサービスをクリックした際に受け取る
	 */
	function accessSp() {
		$this->set('json', '');
		$spcid = $this->params['form']['spcid'];
		$this->SpAccess->updateAccess($this->getAccountID(), $spcid);
		$this->log("accessSp() spcid=".$spcid, "debug");
	}

	/**
	 * グループの管理者によって、サービス表示順が変更された際に受け取る
	 */
	function service_sort_admin() {
		$this->set('json', '');

		$groupid = $this->params['form']['groupid'];
		if (empty($groupid)) {
			$this->log("service_sort_admin() groupid is nothing. ", "error");
			return;
		}

		// グループ管理者のみアクセス可能
//		$admin = $this->Mygroup->isAdmin($this->getAccountID(), $groupid);
		$admin = $this->isGroupAdmin($groupid);
		if (!$admin) {
			$this->log("service_sort_admin() not admin groupid=".$groupid." accountid=".$this->getAccountID(), "error");
			return;
		}

		$this->log("service_sort_admin", "debug");
		$sortedSpcidList = $this->params['form']['result'];
		$this->log("sortedSpcidList=".$sortedSpcidList, "debug");

//		$entityid = $this->getIdentityProvider();
		$result = $this->GroupAdminSpSort->updateAll($groupid, $sortedSpcidList, $this->getAccountID());
		$this->log("result=".$result, "debug");

	}

	/**
	 * ユーザによって、
	 * トップ画面からサービス表示順の設定が変更された際に受け取る
	 */
	function change_sort() {
		$this->set('json', '');
		$sortType = $this->params['data']['spc_sort_type'];
		if (empty($sortType)) {
			return;
		}
		$idpGroupId = $this->getAutoGroupId();
		if (empty($idpGroupId)) {
			return;
		}

		//$this->SpcSort->updateType($this->getAccountID(), $idpGroupId, $sortType);
		// CGポータル画面で設定した表示順はすべてのグループに反映する
		//$this->SpcSort->updateTypeAll($this->getAccountID(), $sortType);
		$this->SpcSortor->updateTypeAll($this->getAccountID(), $sortType);
	}

	/**
	 * ユーザによって、
	 * トップ画面のサービス表示順が変更された際に受け取る(縦３列パターン)
	 */
	function top_service_sort() {
		$this->set('json', '');

		$this->log("top_service_sort", "debug");
		$leftSpcidList = $this->params['form']['resultLeft'];
		$rightSpcidList = $this->params['form']['resultRight'];
		$centerSpcidList = $this->params['form']['resultCenter'];

		$idpGroupId = $this->getAutoGroupId();
		if (empty($idpGroupId)) {
			return;
		}
		$result = $this->UserSpSort->updateVerticalAll(
			$this->getAccountID(), $idpGroupId, $leftSpcidList, $rightSpcidList, $centerSpcidList);
		$this->SpcSort->updateType($this->getAccountID(), $idpGroupId, "manual");

		$this->log("result=".$result, "debug");
		$this->log("leftSpcidList=".$leftSpcidList, "debug");
		$this->log("rightSpcidList=".$rightSpcidList, "debug");
	}

	/**
	 * ユーザによって、
	 * トップ画面のサービス表示順が変更された際に受け取る(横１列パターン)
	 */
	function top_service_side_sort() {
		$this->set('json', '');
	
		$this->log("top_service_side_sort", "debug");
		$spcidList = $this->params['form']['result'];
		$idpGroupId = $this->getAutoGroupId();
		if (empty($idpGroupId)) {
			return;
		}

// 		$result = $this->UserSpSort->updateSideAll(
// 				$this->getAccountID(), $idpGroupId, $spcidList);
//		$this->SpcSort->updateType($this->getAccountID(), $idpGroupId, "manual");

// ★　TODO ↓ $this->SpcSortor->updateTypeAllのようにmygroupにありuserSpSortにないレコードは追加するようにする

		$result = $this->SpcSortor->updateSideAllGroup(
				$this->getAccountID(), $spcidList);
		$this->SpcSortor->updateTypeAll($this->getAccountID(), "manual");

		$this->log("result=".$result, "debug");
		$this->log("spcidList=".$spcidList, "debug");
	}

	/**
	 * ユーザによって、
	 * グループ画面のサービス表示順が変更された際に受け取る(横１列パターン)
	 */
	function group_top_service_side_sort() {
		$this->set('json', '');
		$this->log("group_top_service_side_sort", "debug");
		$this->by_user_service_side_sort();
	}

	private function by_user_service_side_sort() {
		$spcidList = $this->params['form']['result'];
		$groupid = $this->params['form']['groupid'];
		$this->log("groupid=".$groupid, "debug");
		if (!$groupid) {
			return;
		}
		
		$isRole = $this->Mygroup->isMember($this->getAccountID(), $groupid);
		if (!$isRole) {
			$this->log("参加していないグループでサービス表示順を操作しました。", "debug");
			$this->log("account_id=".$this->getAccountID().", groupid=".$groupid , "debug");
			return;
		}
		
		// 画面からresultの他にグループIDも渡す,GETでよいと思う
		// → group_top_service_side_sort($groupid)
		$result = $this->UserSpSort->updateSideAll($this->getAccountID(), $groupid, $spcidList);
		$this->log("result=".$result, "debug");
		$this->SpcSort->updateType($this->getAccountID(), $groupid, "manual");
		$this->log("spcidList=".$spcidList, "debug");
	}

	/**
	 * ユーザによって、
	 * グループ画面のサービス表示順が変更された際に受け取る(横１列パターン)
	 */
	function mygroup_service_side_sort() {
		$this->set('json', '');
		$this->log("mygroup_service_side_sort", "debug");
		$this->by_user_service_side_sort();
	}

	function group_sort() {
		$this->log("group_sort", "debug");

		$time = time();
		$accountId = $this->getAccountID();
		foreach ($this->params['form']['group'] as $groupid) {
			$this->Mygroup->updateFavoriteOn($accountId, $groupid, $time--);
		}
		$json = json_encode("");
		$this->set("json", $json);
		return;
	}

	/**
	 * グループメンバーを取得します。
	 * @param integer $searchGroupid group id for search
	 * @param integer $myGroupid group id of its own
	 */
	function members($searchGroupid, $myGroupid, $language) {
		// validate admin group
		$count = $this->Mygroup->find('count', array('conditions' => array(
			'groupid' => $searchGroupid,
			'account_id' => $this->getAccountID(),
			'admin' => array(1, 2))));
		if ($count == 0) {
			$json = json_encode("");
			$this->set("json", $json);
			return;
		}
		$myGroupMemberIds = $this->Mygroup->find('list', array('fields' => array('account_id'),
			'conditions' => array('groupid' => $myGroupid)));
		$accoutIdList = $this->Mygroup->find('list', array('fields' => array('account_id'),
			'conditions' => array(
				'groupid' => $searchGroupid,
				'NOT' => array('account_id' => $myGroupMemberIds))));

		$accountList = $this->Account->find('all', array(
			'fields' => array('id', 'name', 'name_en', 'university', 'university_en'),
			'conditions' => array('id' => $accoutIdList)));
		for ($i = 0; $i < count($accountList); $i++) {
			if ($language === 'ja') {
				if (!empty($accountList[$i]['Account']['name']))
					$accountList[$i]['Account']['name'] = h($accountList[$i]['Account']['name']);
				else
					$accountList[$i]['Account']['name'] = h($accountList[$i]['Account']['name_en']);
				if (!empty($accountList[$i]['Account']['university']))
					$accountList[$i]['Account']['university'] = h($accountList[$i]['Account']['university']);
				else
					$accountList[$i]['Account']['university'] = h($accountList[$i]['Account']['university_en']);
			} else {
				if (!empty($accountList[$i]['Account']['name_en']))
					$accountList[$i]['Account']['name'] = h($accountList[$i]['Account']['name_en']);
				else
					$accountList[$i]['Account']['name'] = h($accountList[$i]['Account']['name']);
				if (!empty($accountList[$i]['Account']['university_en']))
					$accountList[$i]['Account']['university'] = h($accountList[$i]['Account']['university_en']);
				else
					$accountList[$i]['Account']['university'] = h($accountList[$i]['Account']['university']);
			}
		}

		$json = json_encode($accountList);
		$this->set("json", $json);
	}

	private function getAutoGroupId() {
		$autoGroup = $this->AutoGroup->findIdPGroup($this->getIdentityProvider());
		if (empty($autoGroup)) {
			return false;
		}

		$idpGroupId = $autoGroup['AutoGroup']['group_id'];
		return $idpGroupId;
	}
}
?>
