<?php
/**
 * AutoGroupAgreement Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Controller', 'Manages');

class AutoGroupAgreementsController extends ManagesController {
	var $name = 'AutoGroupAgreements';
	var $uses = array('Group', 'Mygroup', 'AutoGroup', 'AddGroup', 'Transaction');
	var $components = array('AutoGroupCreator', 'TakerAgreement', 'JoinOperator', 'AgreementOperator');
	var $helpers = array('Javascript', 'Common');

	/**
	 * 自動グループ入会の同意を表示する。
	 * @param $groupidFirst
	 * @param $groupidSecond
	 */
	function index($firstGroupId = null, $secondGroupId = null) {
		$this->Session->delete('AutoGroupMessage');

		if (empty($firstGroupId) && empty($secondGroupId))
			$this->redirect('/mygroups/view');

		// validation
		$result = $this->AutoGroup->isAutoGroup($firstGroupId, $this->getAccountID(),
			$this->getEntityID(), $this->getOrganizationNameEn(), $this->getOrganizationalUnitNameEn());
		if (!$result)
			$this->cakeError('illegalAccess', array());

		if (!empty($secondGroupId)) {
			$result = $this->AutoGroup->isAutoGroup($secondGroupId, $this->getAccountID(),
				$this->getEntityID(), $this->getOrganizationNameEn(), $this->getOrganizationalUnitNameEn());
			if (!$result)
				$this->cakeError('illegalAccess', array());
		}

		$idList = array();
		array_push($idList, $firstGroupId);
		if (!empty($secondGroupId))
			array_push($idList, $secondGroupId);

		$groupList = $this->Group->find('all', array('conditions' => array('id' => $idList)));
		if (empty($groupList)) {
			$this->redirect('/mygroups/view');
		}

		$mygroupList = $this->Mygroup->find('all', array(
			'fields' => array('count(groupid) as count', 'groupid'),
			'conditions' => array('groupid' => $idList), 'group' => array('groupid')));
		$this->removeUnnecessaryLanguage($groupList);

		$countList = array();
		foreach($mygroupList as $mygroup) {
			$count = $mygroup[0]['count'];
			$groupid = $mygroup['Mygroup']['groupid'];
			$countList[$groupid] = $count;
		}
		$this->set('groupList', $groupList);
		$this->set('countList', $countList);
		$this->data['AutoGroupAgreement']['first_groupid'] = $firstGroupId;
		$this->data['AutoGroupAgreement']['second_groupid'] = $secondGroupId;

		// 利用属性の表示
		$spcIdList = array();
		foreach ($idList as $id) {
			$idList = $this->AddGroup->getConnectedSPCIdList($id, $onlyActive=false);
			$spcIdList = array_merge($spcIdList, $idList);
		}
		$spcIdList = array_unique($spcIdList);
		$provideAttributeList = $this->TakerAgreement->serveSpcAgreementInfo($spcIdList, $this->getLanguage());
		$this->set('provideAttributeList', $provideAttributeList);

		// 同意している属性を設定
		if (!empty($provideAttributeList)) {
			$agreedList = $this->AgreementOperator->getAgreedList($firstGroupId, $this->getAccountID());
			if (!empty($secondGroupId)) {
				$agreedListSecond = $this->AgreementOperator->getAgreedList($secondGroupId, $this->getAccountID());
				$agreedList = $agreedList + $agreedListSecond;
			}
			$this->set('agreedList', $agreedList);
		}
		$this->render('index');
	}

	/**
	 * 自動グループへ入会する。
	 */
	function join() {
		if ($this->getSubmitName() != 'join') {
			$this->redirect('/mygroups/view');
		}

		$groupIdFirst = 0;
		$groupIdSecond = 0;
		// validate
		if (!empty($this->data['AutoGroupAgreement']['first_groupid'])) {
			$groupIdFirst = $this->data['AutoGroupAgreement']['first_groupid'];
		}
		if (!empty($this->data['AutoGroupAgreement']['second_groupid'])) {
			$groupIdSecond = $this->data['AutoGroupAgreement']['second_groupid'];
		}

		if (!$groupIdFirst && !$groupIdSecond) {
			$this->cakeError('illegalAccess', array());
			return;
		}
		if (!$this->validateJoin($groupIdFirst) || !$this->validateJoin($groupIdSecond)) {
			$this->cakeError('illegalAccess', array());
			return;
		}

		// 同意チェック確認
		if (isset($this->data['AutoGroupAgreement']['agreement'])
			&& empty($this->data['AutoGroupAgreement']['agreement'][0])) {
			$this->data['AutoGroupAgreement']['error_agreement'] = true;
			$this->Session->setFlash(__("message.agreement.sending.attribute", true), 'message');
			if ($groupIdSecond)
				$this->index($groupIdFirst, $groupIdSecond);
			else
				$this->index($groupIdFirst);
			return;
		}

		// execute join
		$this->tracsactionJoin($groupIdFirst, $groupIdSecond);
		$this->Session->setFlash(__("group_lists.message.join.finish", true), 'message');
		$this->redirect('/mygroups/view');
	}

	private function validateJoin($groupid) {
		if (empty($groupid))
			return true;

		$result = $this->AutoGroup->isAutoGroup($groupid, $this->getAccountID(),
			$this->getEntityID(), $this->getOrganizationNameEn(), $this->getOrganizationalUnitNameEn());
		if (!$result) {
			return false;
		}

		$count = $this->Mygroup->find('count', array('conditions' => array(
				'groupid' => $groupid, 'account_id' => $this->getAccountID())));
		if ($count > 0) {
			return false;
		}

		return true;
	}

	private function tracsactionJoin($groupIdFirst, $groupIdSecond) {
		$this->Transaction->begin();
		$resultFirst = true;
		$resultSecond = true;
		if (!empty($groupIdFirst)) {
			$resultFirst = $this->save($groupIdFirst);
		}
		if (!empty($groupIdSecond)) {
			$resultSecond = $this->save($groupIdSecond);
		}

		if (!$resultFirst || !$resultSecond) {
			$this->Transaction->rollback();
			$this->cakeError('systemError');
		}
		$this->Transaction->commit();
	}

	private function save($groupid) {
		$mygroup = array(
			'Mygroup' => array(
				'account_id' => $this->getAccountID(),
				'groupid' => $groupid,
				'admin' => 0,
			),
		);
		$this->Mygroup->create();
		$result = $this->Mygroup->save($mygroup);
		if (!$result) {
			return false;
		}

		// 利用属性を登録
		// 自グループID＋上位グループIDを検索
		$spcIdList = $this->AddGroup->getConnectedSPCIdList($groupid, $onlyActive=false);
		$groupIdList = $this->AddGroup->getTopIdMyIdList($groupid, $onlyActive=false);
		$attributeList = $this->TakerAgreement->getAttributeListForSave($spcIdList);
		foreach ($groupIdList as $id) {
			// 利用属性を登録
			$result = $this->TakerAgreement->saveAttribute(
					$id, $attributeList, $spcIdList, $this->getAccountID());
			if (!$result) {
				return false;
			}
		}

		// 委任登録
		$entrustment = false;
		if (!empty($this->data['AutoGroupAgreement']['entrustment'][0])
			&& $this->data['AutoGroupAgreement']['entrustment'][0] == true) {
			$entrustment = true;
		}
		$result = $this->JoinOperator->entrust($groupid, $this->getAccountID(), $entrustment);
		if (!$result) {
			return false;
		}
		return true;
	}
}
?>
