<?php
/**
 * Connector Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Controller', 'Top');

class ConnectorController extends TopController {
	var $name = 'Connector';
	var $layout = "";

	var $client_cert_key = "";

	function beforeFilter() {
		$this->setClientCertKey();
		$this->validationCertification();
		parent::beforeFilter();
	}

	function beforeRender() {
		if ($this->params["action"] === 'services') {
			Configure::write('debug', 0);
			$this->RequestHandler->setContent('json');
			$this->RequestHandler->respondAs('application/json; charset=UTF-8');
		}
	}

	function services($param) {

		$autoGroup = $this->getIdPGroup();

		if ($this->validationScope($param, $autoGroup['scope']) === false) {
			http_response_code (403);
			$this->cakeError('jsonCannotAccess');
			return;
		}

		$idpEntityid = $autoGroup['entityid'];

		$this->setAccountIdForServices($idpEntityid, $param);

		if ($this->_side($idpEntityid) === false) {
			http_response_code (404);
			$this->cakeError('jsonNotFound');
			return;
		}
		$sideResultList = $this->viewVars['sideResultList'];

		$resultList = array();

		foreach ($sideResultList as $spcid => $spc) {
			$result = array();

			$result['name'] = $spc['serviceName'];

			if (!empty($spc['serviceUrl'])
			    && ((($spc['sp_type'] == 3
				  && ($spc['group_count'] == 0
				      || ($spc['connectIdpGroup']
					  && $spc['group_count'] == 1)))
				 || (!$spc['ismemberof']
				     || ($spc['ismemberof']
					 && $spc['group_count'] == 1))))) {
				$result['service_url'] = $spc['serviceUrl'];
			}

			$result['logo'] = Router::url(array(
				'controller' => 'images',
				'action' => 'sp_group', 64),
				true).'/'.$spcid;

			if (!empty($spc['introduction'])) {
				$result['introduction'] = $spc['introduction'];
			}

			if (!empty($spc['public_tag'])) {
				$result['public_tags'] = $spc['public_tag'];
			}
			if (!empty($spc['tag'])) {
				$result['private_tags'] = $spc['tag'];
			}

			if ($spc['ismemberof'] && isset($spc['group_count']) && $spc['group_count'] != 0) {
				$result['groups'] = array();
				foreach($spc as $groupid => $service) {
					if (!is_numeric($groupid)) {
						continue;
					}
					$result['groups'][] = array(
						'name' => $service['group_name'],
						'url' => $service['url']);
				}
			}

			$resultList[] = $result;
		}

		$this->set('result', $resultList);

		$this->render('/connector/services');
	}

	private function setClientCertKey() {
		$redirect = '';
		foreach (array_keys(array_reverse($_SERVER)) as $key) {
			$replaced = preg_replace("/SSL_CLIENT_CERT$/", "", $key);
			if ($replaced !== $key) {
				$redirect = $replaced;
				break;
			}
		}
		$this->client_cert_key = $redirect."SSL_CLIENT_CERT";
	}

	private function validationScope($eppn, $scope) {
		if ($eppn === '@me') {
			return true;
		}
		if (!isset($scope)) {
			return false;
		}
		if (preg_match('/@'.$scope.'$/', $eppn) === 1) {
			return true;
		}
		return false;
	}
	
	private function validationCertification() {
		if (empty($_SERVER[$this->client_cert_key]) || empty($_SERVER["SSL_CLIENT_VERIFY"])) {
			debug('SSL_CLIENT not set');
			if (!empty($_SERVER[$this->client_cert_key])) {
				debug($this->client_cert_key.':'.$_SERVER[$this->client_cert_key]);
			}
			if (!empty($_SERVER["SSL_CLIENT_VERIFY"])) {
				debug('SSL_CLIENT_VERIFY:'.$_SERVER["SSL_CLIENT_VERIFY"]);
			}
			http_response_code (403);
			$this->cakeError('jsonCannotAccess');
			return;
		}
		$cert = $this->getClientCert();
		$count = $this->AutoGroup->find('count', array('conditions' => array(
			'idp' => 'true',
			'OR' => array('certificate' => $cert, 'certificate2' => $cert))));
		if ($count <= 0) {
			debug('not found certificate');
			debug($this->client_cert_key.'='.$_SERVER[$this->client_cert_key]);
			debug('$cert='.$cert);
			http_response_code (403);
			$this->cakeError('jsonCannotAccess');
			return;
		}
	}

	private function getIdPGroup() {
		$cert = $this->getClientCert();
		$autoGroup = $this->AutoGroup->find('first', array('fields' => array('id', 'entityid', 'scope'),
			'conditions' => array(
				'idp' => 'true',
				'OR' => array('certificate' => $cert, 'certificate2' => $cert))));
		if (empty($autoGroup)) {
			http_response_code (403);
			$this->cakeError('jsonCannotAccess');
		}
		return $autoGroup['AutoGroup'];
	}

	private function getClientCert() {
		$cert =  $_SERVER[$this->client_cert_key];
		$cert = preg_replace("/-----BEGIN CERTIFICATE-----/","",$cert);
		$cert = preg_replace("/-----END CERTIFICATE-----/", '', $cert);
		$cert = preg_replace("/\n/", '', $cert);
		$cert = preg_replace("/\r/", '', $cert);
		$cert = preg_replace("/ /", '', $cert);
		return $cert;
	}
}
?>
