<?php
/**
 * Group List Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Controller', 'Manages');

class GroupListsController extends ManagesController {
	var $name = 'GroupLists';
	var $uses = array('Group', 'SpGroup', 'Account','Mygroup','Inspect', 'GroupList',
		'SpGroupList', 'AddGroup', 'GroupSphost', 'SpHost', 'Organization', 'AccountEppn',
		'ProvideAttribute', 'Transaction', 'Agreement', 'AgreementEntrustment', 'Invite',
		'AutoGroup', 'SpcSort', 'UserSpSort', 'SentNoticeMailLog', 'ManageInspect');
	var $components = array('IdpTranslator', 'ParameterReplacer', 'TakerAgreement', 'JoinOperator',
			'AgreementOperator', 'AgencyUtils', 'GroupAdministrator', 'Csrf', 'SpcSortor');
	var $helpers = array('Attribute');

	function beforeFilter() {
		$this->Csrf->validatePost = false;
		parent::beforeFilter();
		// 継承クラスでは下記の処理をスキップする
		if ($this->params["controller"] === "group_lists") {
			$groupid = $this->_getGroupIdFromURL();
			$this->_checkAccessible($groupid);
		}
	}

	protected function _checkAccessible($groupid) {
		// 継承クラスでは下記の処理をスキップする
		if ($this->params["controller"] === "group_lists") {
			if ($this->params['action'] === 'search') {
				return;
			}
			if (is_null($groupid)) {
				$this->cakeError('notFound', array());
				return;
			}
			if ($this->params['action'] === 'join_confirm') {
				if (!($this->Group->isGroup($groupid) || $this->Group->isIdPGroup($groupid)
					|| $this->Group->isSpConnector($groupid))) {
					$this->cakeError('notFound', array());
					return;
				}
			} else {
				if (!($this->Group->isGroup($groupid) || $this->Group->isIdPGroup($groupid))) {
					$this->cakeError('notFound', array());
					return;
				}
			}
		}
		parent::_checkAccessible($groupid);
	}

	/**
	 *
	 */
	function search() {
		$word = isset($this->params['url']['word']) ? trim($this->params['url']['word']) : '';
		$this->set('word', $word);
		$page = isset($this->params['url']['page']) ? (int)$this->params['url']['page'] : 1;
		if ($page <= 0) { $page = 1; }

		$joins = array(array(
			'type'=>'LEFT OUTER', 'table'=>'mygroups', 'alias'=>'Mygroup',
			'conditions'=>array('Group.id = Mygroup.groupid')));
		$conditions = array('Group.public' => 1, 'active' => 0);

		if(!empty($word)) {
			$conditions['or'] = array(
				'Group.name like'=>'%'.$word.'%',
				'Group.introduction like'=>'%'.$word.'%',
			);
		}

		// グループ検索
		$conditions['sp'] = 0;
		$groupCount = $this->Group->find('count', array('conditions' => $conditions));
		$groups = $this->Group->find('all', array(
			'conditions' => $conditions,
			'joins' => $joins,
			'fields' => array('Group.id', 'Group.name', 'Group.name_en',
					'Group.introduction', 'Group.introduction_en', 'Group.icon', 'Group.sp', 'count(Mygroup.id) as count_member'),
			'group' => array('Group.id'),
			'order' => array('name'),
			'limit' => Configure::read("page.grouplimit"),
			'page' => $page,
		)
		);

		foreach ($groups as $key => $group) {
			$intro = $group['Group']['introduction'];
			if (mb_strlen($intro) > 50) {
				$intro = mb_substr($intro, 0, 50)."...";;
			}
			$groups[$key]['Group']['introduction'] = $intro;
		}
		// 多言語対応
		$this->removeUnnecessaryLanguage($groups);
		$this->set('groups', $groups);

		$this->set('group_administrators', $this->_getGroupAdministrators($groups, $this->getLanguage()));

		// Page control
		$this->controllPage($page, $groupCount, Configure::read("page.grouplimit"));
		$this->set('page', $page);

		$this->render('list');
	}

	/**
	 *
	 * @param integer $id group ID
	 */
	function detail($id) {
		$group = $this->Group->find('first', array(
			'conditions' => array('Group.id' => $id),
			'joins' => array(array(
				'type' => 'LEFT OUTER', 'table' => 'mygroups', 'alias' => 'Mygroup',
				'conditions' => array('Group.id = Mygroup.groupid'))),
			'fields' => array('Group.id', 'Group.name', 'Group.name_en', 'Group.group_key',
				'Group.introduction', 'Group.introduction_en', 'Group.icon',
				'Group.active', 'Group.inspectjoin', 'Group.openmember', 'Group.public', 'Group.created',
				'count(Mygroup.id) as count_member'),
			'group' => array('Group.id', 'Group.name', 'Group.introduction')
		));
		// 英語は表示しないので空にする
		$group['Group']['name_en'] = $group['Group']['introduction_en'] = '';
		$this->set('group', $group);

		$isAdmin = $this->isGroupAdmin($id);
		$this->set('groupAdmin', false);
		$this->set('spconnnect', false);
		if (empty($this->viewVars['group_key'])) {
			$this->set('group_key', '');
		}

		// 利用属性を設定
		$groupIdList = $this->AddGroup->find('list', array('fields' => array('top_id'),
				'conditions' => array('bottom_id' => $id, 'top' => 0)));
		array_push($groupIdList, $id);
		$spcIdList = $this->AddGroup->find('list', array('fields' => array('top_id'),
				'conditions' => array('bottom_id' => $groupIdList, 'top' => 1)));
		$providerAttribute = $this->ProvideAttribute->find('all',
				array('conditions' => array('group_id' => $spcIdList)));

		// このグループがSPコネクタと接続していないのなら、送信される属性にグループIDは入らない
		$myConnectSpcIdList = $this->AddGroup->find('list', array('fields' => array('top_id'),
				'conditions' => array('bottom_id' => $id, 'top' => 1)));
		if (empty($myConnectSpcIdList)) {
			foreach (array_keys($providerAttribute) as $key) {
				$providerAttribute[$key]['ProvideAttribute']['ismemberof'] = 0;
			}
		}
		$this->set('providerAttribute', $providerAttribute);

		// 接続しているSPのサービスのリンクを取得
		$iconList = $this->SpGroup->findIconList();
		if ($this->AutoGroup->isIdPGroup($id)) {
			$serviceLinkList = array();
		} else {
			$serviceLinkList = $this->GroupSphost->getServiceLink($id, $this->getLanguage(), true, $iconList);
			$privateServiceIdList = $this->Group->find('list', array(
				'fields' => array('id'),
				'conditions' => array(
					'id' => array_keys($serviceLinkList),
					'sp' => 2)));
			foreach ($privateServiceIdList as $private_service_id) {
				if (isset($serviceLinkList[$private_service_id])) {
					unset($serviceLinkList[$private_service_id]);
				}
			}
			$serviceLinkList = $this->SpcSortor->updateSortForDetail(
				$id, $serviceLinkList, $this->getAccountID());
		}
		$this->set('serviceLinkList', $serviceLinkList);
		// isMemberOfに同意していないグループを取得
		//  メンバーの同意情報を取得
		$useSpcIdList = array();
		foreach ($myConnectSpcIdList as $val) {
			$useSpcIdList[$val] = $val;
		}
		// ユーザの同意情報を取得
		$userAgreeList = $this->TakerAgreement->servData($this->getAccountID(), $this->getLanguage(), $useSpcIdList);
//debug($userAgreeList);
		// SPコネクタの同意情報を反映
		$userAgreeList = $this->TakerAgreement->reflectSPCAgreement($userAgreeList, $useSpcIdList, $id);
		if (!empty($userAgreeList) && !empty($userAgreeList['Agreement'])) {
			$this->set('userAgreeList', $userAgreeList['Agreement']);
		} else {
			$this->set('userAgreeList', array());
		}


		// 接続していないSPコネクタを取得
		$notConnectSPCIdList = $this->GroupSphost->getNotConnectSPC($id);
		$this->set('notConnectSPCIdList', $notConnectSPCIdList);

		$administrators = $this->_getAllAdministrators($id);
		$administrator_names = array();
		foreach ($administrators as $administrator) {
			$administrator_names[] = $administrator['Account']['name'].$administrator['Account']['name_en'];
		}
		$this->set('administrator_names', $administrator_names);


		// 参加人数(仮アカウントも考慮)
		$accountIdList = $this->Mygroup->find('list', array('fields' => array('account_id', 'account_id'),
			'conditions' => array('groupid' => $id)));
		$tempCount = $this->Account->find('count', array(
			'conditions' => array('id' => $accountIdList, 'is_temp' => array(1, 2, 3))));

		// グループ管理者判定
		if ($isAdmin) {
			// メンバー数
			$this->set('memberCount', count($accountIdList) - $tempCount);

			$this->set('groupAdmin', true);
			// SPコネクタ、上位グループ接続確認
			$count = $this->AddGroup->find('count', array('conditions' => array('bottom_id' => $id)));
			if ($count > 0) {
				$this->set('spconnnect', true);
			}
			// 機関グループのentityID
			if ($this->AutoGroup->isIdPGroup($id)) {
				$entityid = $this->AutoGroup->findEntityIdForIdpGroup($id);
				$this->set('entityid', $entityid);
			}
			// メンバー用画面へ
			$this->setFavariteFiveGroup($id);
			$this->render('/group_lists/top_member');
		} else {
			// メンバー数
			$this->set('memberCount', count($accountIdList) - $tempCount);

			$mygroup = $this->Mygroup->find('first', array(
				'fields' => array('id'),
				'conditions' => array('groupid' => $id, 'account_id' => $this->getAccountID())));
			if (!empty($mygroup)) {
				// メンバー用画面へ
				$this->setFavariteFiveGroup($id);
				$this->render('/group_lists/top_member');
			} else {
				// 非メンバー用グループトップ画面へ
				if ($group['Group']['inspectjoin'] == 1) {
					// 入会申請中か確認
					$count = $this->ManageInspect->find('count', array(
						'conditions' => array('groupid' => $id, 'account_id' => $this->getAccountID(),
							'kind' => 0, 'status' => 0)));
					if ($count > 0) {
						$this->set('requestJoin', 1);
					}
				}
				$this->render('/group_lists/top');
			}
		}
	}

	/**
	 * サービスの並び替えをリセットする
	 */
	function reset($groupid) {
		// IdPグループの並び順に変更する
		$autoGroup = $this->AutoGroup->findIdPGroup($this->getIdentityProvider());
		if (!empty($autoGroup)) {
			$this->SpcSort->copyIdpGroupToTargetGroup(
					$autoGroup['AutoGroup']['group_id'], $groupid, $this->getAccountID());
			$this->UserSpSort->copyIdpGroupToTargetGroup(
					$autoGroup['AutoGroup']['group_id'], $groupid, $this->getAccountID());
		}

		$this->redirect('/group_lists/detail/'.$groupid);
	}

	/**
	 * 入会確認を行う。
	 * @param integer $groupid group ID
	 */
	function join_confirm($groupid) {
		switch ($this->getSubmitName()) {
			case '': // 確認画面の表示
				$this->join_confirm_show($groupid);
				break;
			case 'join': // 入会
				if (isset($this->data['GroupList']['agreement']) && empty($this->data['GroupList']['agreement'][0])) {
					$this->data['GroupList']['error_agreement'] = true;
					$this->Session->setFlash(__("message.agreement.sending.attribute", true), 'message');
					$this->join_confirm_show($groupid);
					return;
				}
				$this->join();
				break;
			case 'cancel': // キャンセル
				if (!$this->Group->isSpConnector($groupid)) {
					$this->redirect('/group_lists/detail/'.$groupid);
				} else  {
					$this->redirect('/sp_groups/detail/'.$groupid);
				}
				return;
		}
	}

	private function join_confirm_show($groupid) {
		$group = $this->Group->find('first', array('conditions' => array('id' => $groupid)));
		$this->set('group', $group);

		// 入会画面を表示してよいか確認
		if ($group['Group']['inspectjoin'] == 2) {
			$this->cakeError('illegalAccess', array());
			return;
		}

		$this->set('alreadyApplication', false);
		if (!$this->validatetJoin($groupid, $group['Group']['inspectjoin'], $group['Group']['sp'])) {
			$this->set('alreadyApplication', true);
		}

		// 自グループID＋上位グループIDを検索
		$groupIdList = $this->AddGroup->getTopIdMyIdList($groupid, $onlyActive=false);
		$upperGroup = 0;
		if (count($groupIdList) >= 2) {	// 上位グループあり
			$upperGroup = 1;
		}
		$this->set('upperGroup', $upperGroup);

		// 自グループIDと上位グループに接続しているSPコネクタのIDを取得
		$spcIdList = $this->AddGroup->getConnectedSPCIdList($groupIdList, $onlyActive=false, true);

		if (empty($spcIdList)) {
			$agreedList = array('Agreement' => array());
		} else {
			// SPコネクタの同意情報を設定
			$lang = $this->getLanguage();
			$agreedList = $this->TakerAgreement->servData($this->getAccountID(), $lang, $spcIdList, $groupIdList, true);
		}
		$this->set('agreedList', $agreedList);

		if ($this->data) {
			$this->data = array();
		}
		if (!isset($this->data['GroupList'])) {
			$this->data['GroupList'] = array();
		}
		$this->data['GroupList']['id'] = $groupid;
	}

	private function validatetJoin($groupid, $inspectJoin, $sp) {
		if (!$this->validateInspect($groupid, $inspectJoin, 0)) {
			if ($sp == 0) {
				$this->Session->setFlash(__("group_lists.message.already.application.join", true), 'message');
			} else {
				$this->Session->setFlash(__("group_lists.message.already.application.admin", true), 'message');
			}
			return false;
		}
		return true;
	}

	private function validatetQuit($groupid, $inspectQuit, $sp) {
		if (!$this->validateInspect($groupid, $inspectQuit, 1)) {
			if ($sp == 0) {
				$this->Session->setFlash(__("group_lists.message.already.application.quit", true), 'message');
			} else {
				$this->Session->setFlash(__("group_lists.message.already.application.admin", true), 'message');
			}
			return false;
		}
		return true;
	}

	private function validateInspect($groupid, $inspect, $kind) {
		if ($inspect != 1){
			return true;
		}

		// 既に申請済みかを調べる
		$count = $this->Inspect->find('count', array(
			'conditions' => array(
				'groupid' => $groupid,
				'account_id' => $this->getAccountID(),
				'kind' => $kind,
				'status' => 0)));
		if ($count > 0) {
			return false;
		}
		return true;
	}

	/**
	 * 入会する
	 */
	function join() {
		$groupid = $this->data['GroupList']['group_id'];
		if (empty($groupid)) {
			$this->redirect('/mygroups/view');
		}

		$group = $this->Group->find('first', array(
			'fields' => array('id', 'name', 'inspectjoin', 'sp'),
			'conditions' => array('id' => $groupid)));

		// validate
		if ($group['Group']['inspectjoin'] == 2) {
			$this->cakeError('illegalAccess', array());
			return;
		}

		$controllerName = 'group_lists';
		if ($group['Group']['sp'] == 0) {
			$controllerName = 'group_lists';
		} else if ($group['Group']['sp'] == 1) {
			$controllerName = 'sp_groups';
		}

		// キャンセル
		if ($this->getSubmitName() === "cancel") {
			$this->redirect('/'.$controllerName.'/detail/'.$groupid);
			return;
		}
		$this->set('group', $group);

		// 承認が必要な場合は申請
		if ($group['Group']['inspectjoin'] == 1) {
			// 既に申請済みかを再度調べる
			if (!$this->validatetJoin(
			$this->data['GroupList']['group_id'], $group['Group']['inspectjoin'], $group['Group']['sp'])) {
				$this->redirect('/'.$controllerName.'/detail/'.$groupid);
				return;
			}

			// 申請を登録
			$this->Transaction->begin();
			$inspect = array(
				'Inspect' => array(
					'groupid' => $groupid,
					'account_id' => $this->getAccountID(),
					'kind' => 0,
					'status' => 0,
					'sent' => 0,
				)
			);
			$resultInspect = $this->Inspect->save($inspect);
			if (!$resultInspect) {
				$this->Transaction->rollback();
				$this->cakeError('systemError');
				return;
			}

			// 利用属性、委任を登録
			$entrustment = false;
			if (!empty($this->data['GroupList']['entrustment'][0])
				&& $this->data['GroupList']['entrustment'][0] == true) {
				$entrustment = true;
			}
			$result = $this->joinAttribute($groupid, $entrustment);
			if (!$result) {
				$this->Transaction->rollback();
				$this->cakeError('systemError');
				return;
			}

			$this->Transaction->commit();

			// SPコネクタの場合は、管理者にメールする
			if ($group['Group']['sp'] == 1) {
				$this->sedMailForManager($groupid);
			}

			if ($group['Group']['sp'] == 0) {
				$this->Session->setFlash(__("group_lists.message.after.approval", true), 'message');
			} else {
				$this->Session->setFlash(__("group_lists.message.after.approval.admin", true), 'message');
			}
		} else {
			// 既に入会しているか調べる
			$count = $this->Mygroup->find('count', array('conditions' => array(
				'groupid' => $this->data['GroupList']['group_id'],
				'account_id' => $this->getAccountID())));
			if ($count > 0) {
				if ($group['Group']['sp'] == 0) {
					$this->Session->setFlash(__("group_lists.message.already.join", true), 'message');
				} else {
					$this->Session->setFlash(__("group_lists.message.already.admin", true), 'message');
				}
				$this->redirect('/'.$controllerName.'/detail/'.$groupid);
				return;
			}

			$admin = 0;  // メンバーとして
			if ($group['Group']['sp'] == 1) {
				$admin = 1; // 管理者として
			}

			$this->Transaction->begin();
			$mygroup = array(
				'Mygroup' => array(
					'groupid' => $groupid,
					'account_id' => $this->getAccountID(),
					'admin' => $admin,
				)
			);
			$resultMygroup = $this->Mygroup->save($mygroup);
			if (!$resultMygroup) {
				$this->Transaction->rollback();
				$this->cakeError('systemError');
				return;
			}
			if ($group['Group']['sp'] == 0) {
				$this->Session->setFlash(__("group_lists.message.join.finish", true), 'message');
			} else {
				$this->Session->setFlash(__("group_lists.message.join.finish.admin", true), 'message');
			}

			// 利用属性、委任を登録
			$entrustment = false;
			if (!empty($this->data['GroupList']['entrustment'][0]) && $this->data['GroupList']['entrustment'][0] == true) {
				$entrustment = true;
			}
			$result = $this->joinAttribute($groupid, $entrustment);
			if (!$result) {
				$this->Transaction->rollback();
				$this->cakeError('systemError');
				return;
			}
			$this->Transaction->commit();
		}
		$this->redirect('/'.$controllerName.'/detail/'.$groupid);
	}

	private function joinAttribute($groupid, $entrustment) {
		// 自グループID＋上位グループIDを検索
		$groupIdList = $this->AddGroup->getTopIdMyIdList($groupid, $onlyActive=false);

		foreach ($groupIdList as $gid) {
			// このグループに接続しているSPコネクタのIDを取得
			$spcIdList = $this->AddGroup->getConnectedSPCIdList($gid, $onlyActive=false);
			$attributeList = $this->TakerAgreement->getAttributeListForSave($spcIdList);

			// 利用属性を登録
			$result = $this->TakerAgreement->saveAttribute(
				$gid, $attributeList, $spcIdList, $this->getAccountID());
			if (!$result) {
				return false;
			}
		}

		// 委任登録
		$result = $this->JoinOperator->entrust(
			$groupid, $this->getAccountID(), $entrustment);
		if (!$result) {
			return false;
		}
		return true;
	}

	/**
	 *
	 * @param integer $id group ID
	 */
	function members($id) {
		if (!$this->isSuperManager()) {
			// メンバーリストを閲覧することができるかを確認
			$group = $this->Group->find('first', array('fields' => array('id', 'openmember'),
				'conditions' => array('id' => $id)));
			// 全公開ではない場合は、グループに入会しているかを確認
			if ($group['Group']['openmember'] != 2) {
				$count = $this->Mygroup->find('count', array('conditions' => array(
					'account_id' => $this->getAccountID(), 'groupid' => $id)));
				if ($count == 0) {
					$this->cakeError('cannotAccess', array('url' => $this->here));
					return;
				}
			}
		}

		if (isset($this->params['url']['page'])) {	// 前へ 次へ redirect時
			$page = (int)$this->params['url']['page'];
		}
		else if (isset($this->data['GroupList']['page'])) {	// submit時
			$page = $this->data['GroupList']['page'];
		}
		else {
			$page = 1;
		}
		if ($page <= 0) { $page = 1; }
		$word = '';
		if (isset($this->params['url']['word'])) {	// 前へ 次へ redirect時
			$word = $this->params['url']['word'];
		}
		else if (isset($this->data['GroupList']['word'])) {	// 検索以外のsubmit時
			$word = $this->data['GroupList']['word'];
		}
		$word = trim($word);
		$this->set('word', $word);
		$this->memberlist($page, $id, 'GroupList', $word);
		$this->setFavariteFiveGroup($id);
		$this->render('members');
	}

	/**
	 *
	 * @param integer $page page number
	 * @param integer $id group ID
	 * @param string $modelName model name(SpGroupListsControllerから呼び出しで利用している)
	 */
	protected function memberlist($page = 1, $id, $modelName = null, $word = '') {
		if ($modelName == null) {
			$modelName = 'GroupList';
		}

		if ($page === 'all') {
			$page = 0;
		}

		switch ($this->getSubmitName()) {
			case '': //表示
				$this->membersShow($id, $page, null, $modelName, $word);
				break;
			case 'invite': // 招待
				$this->redirect('/admin/manage_groups/invite/'.$id);
				break;
			case 'edit': // 権限変更
				// 管理権限を持っているか確認
				$isAdmin = $this->isGroupAdmin($id);
				if (!$isAdmin) {
					return;
				}
				// 管理者は最低１人いるかを確認
				$existManager = false;
				$editIdList = array();	// 権限変更対象者一覧
				foreach ($this->data['admin'] as $key => $acountId) {
					array_push($editIdList, key($acountId));
					$value = array_shift($acountId);
					if ($value >= 1){
						$existManager = true;
						continue;
					}
				}
				if (!$existManager) {	// 画面内に管理者がいない
					// 権限変更対象者以外に管理者がいるかを検索
					$count = $this->Mygroup->find('count', array('conditions' =>
						array('NOT' => array('account_id' => $editIdList),
							'groupid' => $id, 'admin' => array(1, 2))));
					if ($count > 0) {
						$existManager = true;
					}
				}
				if (!$existManager) {
					$this->Session->setFlash(__("group_lists.no.exit.admin", true), 'message');
					$this->membersShow($id, $page, null, $modelName);
					return false;
				}

				$serviceLinkList = $this->GroupSphost->getServiceLink($id, $this->getLanguage(), true);
				$privateServiceIdList = $this->Group->find('list', array(
					'fields' => array('id'),
					'conditions' => array(
						'id' => array_keys($serviceLinkList),
						'sp' => 2)));

				foreach ($this->data['admin'] as $key => $acountId) {
					$value = array_shift($acountId);
					$mygroup = $this->Mygroup->find('first', array('conditions' => array('id'=>$key)));
					unset($mygroup['Mygroup']['modified']);

					$beforeAuthority = $mygroup['Mygroup']['admin'];
					if ($beforeAuthority == $value) {
						continue;
					}

					$mygroup['Mygroup']['admin'] = $value;
					$this->Mygroup->save($mygroup);

					if ($beforeAuthority == 0 && ($value == '1' || $value == '2')) {
						foreach ($privateServiceIdList as $psid) {
							$psMygroup = array(
								'groupid' => $psid,
								'account_id' => $mygroup['Mygroup']['account_id'],
								'admin' => 1);
							$this->Mygroup->create();
							$this->Mygroup->save(array('Mygroup' => $psMygroup));
						}
					} elseif  (($beforeAuthority == 1 || $beforeAuthority == 2) && $value == '0') {
						$this->Mygroup->deleteAll(array('groupid' => $privateServiceIdList, 'account_id' => $mygroup['Mygroup']['account_id']));
					}
				}
				$this->Session->setFlash(__("message.group_lists.change.authority", true), 'message');
				$this->redirect('/group_lists/members/'.$id.'?word='.$word);
				break;

			case 'withdraw': // 退会
				// 退会者を取得
				$withdrawIdList = array(); // 退会者一覧
				foreach ($this->data['withdraw'] as $value) {
					if (empty($value[0])) {
						continue;
					}
					array_push($withdrawIdList, $value[0]);
				}

				// 退会者以外に管理者がいるかを検索
				$count = $this->Mygroup->find('count', array('conditions' =>
				array('NOT' => array('account_id' => $withdrawIdList),
						'groupid' => $id, 'admin' => array(1, 2))));
				if ($count == 0) {
					$this->Session->setFlash(__("group_lists.no.exit.admin", true), 'message');
					$this->membersShow($id, $page, null, $modelName);
					return false;
				}

				// 退会処理
				$serviceLinkList = $this->GroupSphost->getServiceLink($id, $this->getLanguage(), true);
				$privateServiceIdList = $this->Group->find('list', array(
					'fields' => array('id'),
					'conditions' => array(
						'id' => array_keys($serviceLinkList),
						'sp' => 2)));

				$quitFlg = false;
				foreach ($this->data['withdraw'] as $value) {
					if (empty($value[0])) {
						continue;
					}
					if($value){
						// Mygroup
						$this->Mygroup->deleteAll(array('groupid' => $privateServiceIdList, 'account_id' => $value[0]));
						$this->Mygroup->deleteAll(array('groupid' => $id, 'account_id' => $value[0]));
						// AgreementEntrustmen
						$this->AgreementEntrustment->deleteAll(array('group_id' => $id, 'account_id' => $value[0]), false);
						// 仮アカウントを削除（仮アカウントでなければ何もしない）
						$this->Account->deleteTemporaryAccount($value[0], $id);
						// Invite
						$invite = $this->Invite->findByTempAccountIDGroupID($value[0], $id);
						$this->Invite->updateInvalid($invite['Invite']['id']);
						$quitFlg = true;
					}
				}
				if ($quitFlg) {
					if ($modelName == 'GroupList') {
						$this->Session->setFlash(__("message.group_lists.quit.member", true), 'message');
						$this->redirect('/group_lists/members/'.$id.'?word='.$word.'&page='.$page);
					} else if ($modelName == 'SpGroupList') {
						$this->Session->setFlash(__("message.group_lists.quit.admin", true), 'message');
						$this->redirect('/sp_group_lists/administrators/'.$id);
					}
				} else {
					$this->Session->setFlash(__("message.group_lists.not.quit.member", true), 'message');
					if ($modelName == 'GroupList') {
						$this->redirect('/group_lists/members/'.$id.'?word='.$word.'&page='.$page);
					} else if ($modelName == 'SpGroupList') {
						$this->redirect('/sp_group_lists/administrators/'.$id);
					}
				}
				return;
			case 'search': //検索
				$word = isset($this->data['GroupList']['search']) ? trim($this->data['GroupList']['search']) : '';
				$this->set('word', $word);
				$page = 1;
				$this->membersShow($id, $page, null, $modelName, $word);
				break;
			default:
				break;
		}
	}

	/**
	 * メンバーリストを表示する。
	 * @param integer $id group ID
	 * @param integer $page page number
	 * @param array $selectList 選択状態のメンバー一覧
	 * @param array $modelName model name
	 */
	private function membersShow($id, $page, $selectList = null, $modelName, $word = '') {
		// グループメンバー公開設定を取得
		$Group = $this->Group->find('first', array(
			'fields' => array('openmember', 'group_key'),
			'conditions' => array('id' => $id)));
		$this->data[$modelName]['openmember'] = $Group['Group']['openmember'];

		// 管理権限を設定
		$manager = false;
		$isAdmin = $this->isGroupAdmin($id);
		if ($isAdmin) {
			$manager = true;
		}
		$this->data[$modelName]['manager'] = $manager;

		// 非公開でメンバーでも管理者でもなければエラー
		if ($Group['Group']['openmember'] == 0 && !$manager) {
			if (!$this->Mygroup->isMember($this->getAccountID(), $id)) {
				$this->cakeError('cannotAccess', array('url' => $this->here));
				return;
			}
		}

		// 代理グループの有無を設定
		$agency = $this->AgencyUtils->isAgencyGroup($Group['Group']['group_key']);
		$this->data['GroupList']['agency'] = $agency;

		$dispTemp = false;	// 仮アカウント表示フラグ
		if ($modelName === 'GroupList' && $manager && !$agency) {	// グループリストで管理者で代理グループでなければ仮アカウントを表示する
			$dispTemp = true;
		}

		// メンバーを検索
		$conditions = array();
		if ($word != '') {
			$word = str_replace('　', ' ', $word);
			$word = trim($word);
			$word = preg_replace('/\s+/', ' ', $word);
		}
		if ($word != '') {
			$nameConditions = array();
			// 名前の検索条件
			if (!$dispTemp) {	// 仮アカウントを表示しない
				$nameConditions['is_temp'] = 0;
			}
			else {
				$nameConditions['is_temp !='] = 1;	// 仮アカウント(メールアドレスで招待)の場合、氏名は検索しない
			}
			$word = str_replace(array('\\', '%', '_'), array('\\\\', '\%', '\_'), $word);	// '\'と'%'と'_'のエスケープ
			$words = explode(' ',$word);
			$nameConditions['AND'] = array();
			foreach ($words as $wd) {
				if ($this->getLanguage() === 'ja') {
					$nameConditions['AND'][] = array(
						'OR' => array(
							'name like' => '%'.$wd.'%',
							'AND' => array(
								'IFNULL(name, \'\')' => '',
								'name_en like' => '%'.$wd.'%')));
				}
				else {
					$nameConditions['AND'][] = array(
						'OR' => array(
							'name_en like' => '%'.$wd.'%',
							'AND' => array(
								'IFNULL(name_en, \'\')' => '',
								'name like' => '%'.$wd.'%')));
				}
			}
			if ($dispTemp) {	// 仮アカウントを検索
				// アドレス検索条件
				$mailConditions = array();
				$mailConditions['is_temp'] = array(1, 3);	// 仮アカウント(名前で招待)の場合、メールアドレスは検索しない
				foreach ($words as $wd) {
					$mailConditions['AND'][] = array('mail like' => '%'.$wd.'%');
				}
				$conditions = array();
				$conditions['OR'] = array(
					$nameConditions,
					$mailConditions
				);
			}
			else {	// 仮アカウントを表示しない
				$conditions = $nameConditions;
			}
		}

		$params = array(
			'joins' =>array(array(
				'type' => 'INNER', 'table' => 'mygroups', 'alias' => 'Mygroup',
				'conditions' => array(
					'Account.id = Mygroup.account_id',
					'Mygroup.groupid' => $id))
			),
			'conditions' => $conditions,
		);
		if ($modelName === 'GroupList') {
			if (!$dispTemp) {        // 仮アカウントを表示しない
				$params['joins'][0]['conditions']['Account.is_temp'] = 0;
			}
		}
		$memberCount = $this->Account->find('count', $params);
		if ($modelName === 'GroupList') {
			$maxPage = ceil($memberCount / Configure::read("page.memberlimit"));
			if ($page > $maxPage) {
				$page = $maxPage;
			}
		}
		$params = array(
			'fields'=>array('Account.id', 'Account.name', 'Account.name_en', 'Account.mail', 'Account.is_temp',
					'Account.university', 'Account.university_en', 'Account.introduction', 'Account.introduction_en',
					'Account.arrival', 'Account.is_temp','Mygroup.id', 'Mygroup.created', 'Mygroup.admin'),
			'joins' =>array(array(
				'type' => 'INNER', 'table' => 'mygroups', 'alias' => 'Mygroup',
				'conditions' => array(
					'Account.id = Mygroup.account_id',
					'Mygroup.groupid' => $id))
			),
			'conditions' => $conditions,
			'order' => array('Mygroup.admin DESC', 'Account.is_temp ASC', 'Mygroup.created'),
		);
		if ($modelName === 'GroupList') {
			$params['limit'] = Configure::read("page.memberlimit");
			$params['page'] = $page;
			if (!$dispTemp) {        // 仮アカウントを表示しない
				$params['joins'][0]['conditions']['Account.is_temp'] = 0;
			}
		}
		$members = $this->Account->find('all', $params);
		$this->removeUnnecessaryLanguage($members);
		$this->IdpTranslator->setIdP($members, $this->getLanguage());

		// ページ制御
		$this->controllPage($page, $memberCount, Configure::read("page.memberlimit"));

		foreach ($members as $key => $member) {
			$members[$key]['Account']['display_mail'] = null;
		}
		// 表示名の設定
		if ($isAdmin) {
			foreach ($members as $key => $member) {
				switch($member['Account']['is_temp']) {
				case 1: // メールアドレスで招待
					$members[$key]['Account']['name'] = $member['Account']['mail'];
					break;
				case 3: // 名前＋メールアドレス入力で招待
					$members[$key]['Account']['display_mail'] = $member['Account']['mail'];
					break;
				}
			}
		}

		// 削除対象になっているアカウントと予定日の設定
		$deleteIdList = $this->SentNoticeMailLog->find('list', array(
			'fields' => array('account_eppn_id'),
			'gruop' => 'account_eppn_id',
		));
		$accountIdList = array();
		foreach ($members as $key => $member) {
			$members[$key]['Account']['delete_date'] = '';
			$accountEppnList = $this->AccountEppn->find('all', array(
				'fields' => array('id', 'last_login_approx'),
				'conditions' => array('account_id' => $member['Account']['id']),
				'order' => array('last_login_approx ASC')));
			$delete = true;
			foreach ($accountEppnList as $eppn) {
				if (!in_array($eppn['AccountEppn']['id'], $deleteIdList)) {
					$delete = false;
					break;	// 削除予定ではない
				}
				$last_login_approx = $eppn['AccountEppn']['last_login_approx'];
			}
			if ($accountEppnList && $delete) {	// 削除予定なので、予定日を設定する
				$targetTime = strtotime($last_login_approx);
				$members[$key]['Account']['delete_date'] = date('Y/m/d', strtotime('+2 year', $targetTime));
			}
		}

		// 画面で選択した状態にする
		$member_count = count($members);
		if ($selectList != null) {
			for ($i = 0; $i < $member_count; $i++) {
				// 権限変更
				$val = array_shift($selectList['admin'][ $members[$i]['Mygroup']['id'] ]);
				$members[$i]['Mygroup']['admin'] = $val;

				// 退会
				if (!empty($selectList['withdraw'][ $members[$i]['Account']['id'] ][0] )) {
					$members[$i]['Account']['selectedWithdraw'] = $selectList['withdraw'][$members[$i]['Account']['id']][0];
				} else {
					$members[$i]['Account']['selectedWithdraw'] = '';
				}
			}
		} else {
			for ($i = 0; $i < $member_count; $i++) {
				$members[$i]['Account']['selectedWithdraw'] = '';
			}
		}

		// 下位グループ名の取得
		$bottomGroupList = array();
		if ($isAdmin) {
			$bottomIdList = $this->AddGroup->getBottomIdList($id, $onlyActive=false);
			$bottomGroupList = $this->Group->findGroupList($bottomIdList);
		}
		$this->set('bottomGroupList', $bottomGroupList);

		$this->set('members', $members);
		$this->data[$modelName]['id'] = $id;
		$this->data[$modelName]['page'] = $page;

		// 自分のアカウントIDを設定
		$this->set('myaccountid', $this->getAccountId());
	}

	/**
	 * グループからの退会
	 * @param integer $groupid group ID
	 */
	function quit($groupid) {
		if (!$this->isSuperManager()) {
			// 入会しているかを確認
			$count = $this->Mygroup->find('count', array('conditions' => array(
					'account_id' => $this->getAccountID(),
					'groupid' => $groupid)));
			if ($count == 0) {
				$this->cakeError('cannotAccess', array('url' => $this->here));
				return;
			}
		}

		$this->data['GroupList']['id'] = $groupid;
		$this->set('message', '');

		$group = $this->Group->find('first', array('fields' => array('name', 'inspectquit', 'sp'),
			'conditions' => array('id' => $groupid)));
		$sp = $group['Group']['sp'];
		$this->set('Group', $group);
		$this->setFavariteFiveGroup($groupid);

		switch ($this->getSubmitName()) {
			case '': // 表示
				$otherCount = $this->getOtherManagerCount($groupid);
				if ($otherCount == 0) {
					if (!$sp)
						$this->set('message', __("message.group_lists.not.quit.admin", true));
					else
						$this->set('message', __("message.group_lists.not.exit", true));
				}

				// 既に申請しているかを確認
				$this->set('alreadyApplication', false);
				if (!$this->validatetQuit($groupid, $group['Group']['inspectquit'], $group['Group']['sp'])) {
					$this->set('alreadyApplication', true);
				}
				break;
			case 'quit': // 退会
				$count = $this->Mygroup->find('count', array('conditions' => array(
						'account_id' => $this->getAccountID(),
						'groupid' => $groupid, 'admin' => array(1, 2))));
				if ($count > 0) {
					// 自分以外に管理者が最低１人いるかを確認
					$otherCount = $this->getOtherManagerCount($groupid);
					if ($otherCount == 0) {
						if (!$sp) {
							$this->Session->setFlash(__("message.group_lists.not.quit.admin", true), 'message');
						} else {
							$this->Session->setFlash(__("message.group_lists.not.exit", true), 'message');
						}
						return false;
					}
				}

				if ($group['Group']['inspectquit'] == 1) {
					// 退会申請をする
					$inspect = array(
						'Inspect' => array(
							'groupid' => $groupid,
							'account_id' => $this->getAccountID(),
							'kind' => 1,
							'status' => 0,
							'sent' => 0,
						)
					);
					$this->Inspect->save($inspect);
					if (!$sp) {
						$this->Session->setFlash(__("message.group_lists.quit.approval", true), 'message');
					} else {
						$this->Session->setFlash(__("message.group_lists.quit.other", true), 'message');
					}
					$this->redirect('/mygroups/view');
				} else {
					// プライベートサービスの管理者情報を削除
					$privateServiceIdList = $this->GroupSphost->getPrivateServiceId($groupid);
					$this->Mygroup->deleteAll(array('groupid' => $privateServiceIdList, 'account_id' => $this->getAccountID()));
					// Myグループ情報を削除
					$this->Mygroup->deleteAll(array('groupid' => $groupid, 'account_id' => $this->getAccountID()), false);
					if (!$sp) {
						$this->Session->setFlash(__("message.group_lists.quit.group", true), 'message');
					} else {
						$this->Session->setFlash(__("message.group_lists.quit.gadmin", true), 'message');
					}

					// 委任情報を削除
					$this->AgreementEntrustment->deleteAll(array('group_id' => $groupid, 'account_id' => $this->getAccountID()), false);

					// 管理者に退会したことをメールする
					$template = 'quit_group_template';
					$Group = $this->Group->find('first', array('conditions' => array('id' => $groupid)));
					$Mail = array(
						'groupName' => $Group['Group']['name'],
						'groupNameJp' => $this->getGroupNameForMail('ja', $Group),
						'groupNameEn' => $this->getGroupNameForMail('eng', $Group),
						'url' => Router::url('/group_lists/detail/', true).$Group['Group']['id'],
					);
					$account = $this->Account->find('first', array(
						'fields' => array('id', 'name', 'name_en'),
						'conditions' => array('id' => $this->getAccountID())));
					$Mail['AccountName'] = $account['Account']['name'];
					$Mail['AccountNameJp'] = $this->getAccountName('ja', $account);
					$Mail['AccountNameEn'] = $this->getAccountName('eng', $account);
					$invites = $this->Invite->find('first', array(
						'fields' => array('address'),
						'conditions' => array('groupid' => $groupid, 'joined_account_id' => $this->getAccountID()),
						'order' => array('created' => 'desc')
					));
					$Mail['MailAddressJp'] = '';
					$Mail['MailAddressEn'] = '';
					if (!empty($invites['Invite']['address'])) {
						$address = $invites['Invite']['address'];
						$Mail['MailAddressJp'] = sprintf(__("mail.once.invited.as.jp", true), $address);
						$Mail['MailAddressEn'] = sprintf(__("mail.once.invited.as.en", true), $address);
					}
					$this->sendMailAllAdmin($groupid, 'quit.group', $template, $Mail);
					$this->redirect('/mygroups/view');
				}
				break;
			case 'cancel': // キャンセル
				if ($group['Group']['sp'] == 0) {
					$this->redirect('/group_lists/detail/'.$groupid);
				} else if ($group['Group']['sp'] == 1) {
					$this->redirect('/sp_groups/detail/'.$groupid);
				}
				return;
			default:
				break;
		}
	}

	/**
	 * 自分以外に管理者が最低１人いるかを確認
	 * @param integer $groupid group ID
	 */
	private function getOtherManagerCount($groupid) {
		$otherCount = $this->Mygroup->find('count', array('conditions' => array(
			'NOT' => array('account_id' => $this->getAccountID()),
			'groupid' => $groupid, 'admin' => array(1, 2))));
		return $otherCount;
	}

	/**
	 *
	 * @param integer $id group ID
	 */
	function administrators($id) {
		$this->administratorlist(1, $id);
	}

	function _getAllAdministrators($id)
	{
		return $this->_getAdministrators($id, null);
	}

	function _getAdministrators($id, $page = 1)
	{
		$page_options = array();
		if (!is_null($page) && (int)$page > 0) {
			$page_options = array(
				'limit' => Configure::read("page.memberlimit"),
				'page' => (int)$page,
			);
		}
		$administrators = $this->Account->find('all', array(
			'fields' => array('Account.id', 'Account.name', 'Account.name_en',
				'Account.university', 'Account.university_en',
				'Account.introduction', 'Account.introduction_en', 'Account.arrival',
				'Mygroup.id','Mygroup.created', 'Mygroup.admin'),
			'joins' => array(array(
				'type' => 'INNER', 'table' => 'mygroups', 'alias' => 'Mygroup',
				'conditions' => array(
					'Account.id = Mygroup.account_id',
					'Mygroup.groupid' => $id))
			 ),
			'conditions' => array('admin' => array(1, 2)),
			'order' => array('Mygroup.created'),
		) + $page_options);
		$this->removeUnnecessaryLanguage($administrators);
		return $administrators;
	}

	/**
	 *
	 * @param integer $page page number
	 * @param integer $id group ID
	 */
	function administratorlist($page = 1, $id) {
		// 管理者を検索
		$adminCount = $this->Mygroup->find('count', array(
			'conditions' => array('groupid' => $id, 'admin' => array(1, 2))));
		$administrators = $this->_getAllAdministrators($id);	// 画面にページ制御が無いので、全表示にする
//		$administrators = $this->setIdP($administrators);
		$this->IdpTranslator->setIdP($administrators, $this->getLanguage());

		// ページ制御
		$this->controllPage($page, $adminCount, Configure::read("page.memberlimit"));

		$this->data['GroupList']['id'] = $id;
		$this->data['GroupList']['page'] = $page;
		$this->set('administrators', $administrators);
		$this->setFavariteFiveGroup($id);
		$this->render('administrators');
	}

	protected function _getGroupAdministrators($groups, $lang)
	{
		$groupIds = array();
		foreach ($groups as $group) {
			$groupIds[] = $group['Group']['id'];
		}
		return $this->GroupAdministrator->get($groupIds, $lang);
	}


	/**
	 * サービスの並び順を表示する
	 * @param integer $groupid group ID
	 */
// 	function service_sort($groupid) {
// 		$spcSortType = $this->SpcSort->getSortType($this->getAccountID(), $groupid);
// 		$this->set('groupid', $groupid);
// 		$this->set('spcSortType', $spcSortType);
// 	}

	/**
	 * サービスの並び順を保存する
	 * @param integer $groupid group ID
	 */
	function service_sort_save() {
		$groupid = $this->data['GroupList']['id'];
		$spcSortType = $this->data['GroupList']['spc_sort_type'];
		$this->SpcSort->updateType($this->getAccountID(), $groupid, $spcSortType);
		$this->redirect('/group_lists/detail/'.$groupid);
	}

	/**
	 * 入会申請のキャンセルを行う。
	 * @param integer $groupid group ID
	 */
	function join_cancel($groupid) {
		$group = $this->Group->find('first', array('conditions' => array('id' => $groupid)));
		$conditions =  array('groupid' => $groupid, 'account_id' => $this->getAccountID(), 'kind' => 0, 'status' => 0);
		$inspect = $this->ManageInspect->find('all', array('conditions' => $conditions));
		if (!empty($inspect)) {
			$this->ManageInspect->deleteAll($conditions, false);
			$this->log('GroupListsController::join_cancel Delete inspect data groupid='.$groupid.' account_id='.					$this->getAccountID(), 'debug');
			$this->Session->setFlash(__("group_lists.message.join.canceled", true), 'message');
			$this->redirect('/mygroups/view');
		}
	}
}
?>
