<?php
/**
 * Group Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Controller', 'Manages');

class GroupsController extends ManagesController {
	var $name = 'Groups';
	var $uses = array('Transaction', 'Group', 'Mygroup',
			'SpAutoConnector', 'AddGroup', 'AgreementIsmemberof', 'Agreement', 'SpGroup', 'GroupSphost', 'Account');
	var $components = array('JoinOperator', 'AdminGroup', 'GroupInvite', 'TakerAgreement', 'Csrf',
			'Callback', 'ParameterReplacer');
	var $helpers = array('Javascript');

	function beforeFilter() {
		$this->Csrf->validatePost = false;
		parent::beforeFilter();
	}

	/**
	 * create group
	 */
	function create() {
		// 利用可能Webサービスの取得
		$spcIdList = $this->SpAutoConnector->findSpcList();
		$spcList = $this->SpGroup->findByIdList($spcIdList);
		$this->set('spcList', $spcList);

		switch ($this->getSubmitName()) {
		case '': // 初期表示(入力画面)
			$this->data['Group'] = array();
			break;
		case 'save':
			if (empty($this->data)) {
				return;
			}
			$this->_save();
			break;
		case 'cancel':
			$this->redirect('/mygroups/view');
			break;
		case 'input':
			if (empty($this->data)) {
				return;
			}
			$this->_adminInput();
			break;
		}
	}

	private function validateAdminInput()
	{
		if (isset($this->data['Group']['adminInput'])
			&& is_array($this->data['Group']['adminInput'])
			&& $this->data['Group']['adminInput']) {
			$invalid_addresses = $this->GroupInvite->getInvalidAddresses($this->data['Group']['adminInput']);
			if ($invalid_addresses) {
				$message = $this->createMessage('message.manage_groups.invite.invalid.email', implode(', ', $invalid_addresses));
				$this->Session->setFlash($message, 'message');
				return false;
			}
		}
		return true;
	}

	private function _adminInput() {
		if (!$this->validateAdminInput()) {
			$this->setAdminInputPageData();
			$this->render('admin_input');
			return false;
		}

		// memberの値を変換 "$id,$id" => array($id => "1")
		$memberIdKeys = array();
		if (isset($this->data['Group']['member'])) {
			foreach (explode(',', $this->data['Group']['member']) as $memberId) {
				$memberIdKeys[$memberId] = '1';
			}
		}

		if ($memberIdKeys) {
			$this->data['Group']['member'] = $memberIdKeys;
		} else {
			unset($this->data['Group']['member']);
		}

		$confirmed_admin_input = true;
		$this->_save($confirmed_admin_input);
	}

	private function setAdminInputPageData() {
		$addMemberIdList = array();
		if (isset($this->data['Group']['member'])) {
			$validMemberNameList = $this->getValidMemberNameList(explode(',', $this->data['Group']['member']));
			$addMemberIdList = array_keys($validMemberNameList);
		}

		$notAgreedMemberIds = $this->TakerAgreement->getNotAgreedToMail($addMemberIdList);
		$this->set('notAgreedMemberIds', $notAgreedMemberIds);
		$this->set('addMemberNameList', $validMemberNameList);
	}

	private function _validateInviteAddress() {
		// 無効なメールアドレスの確認
		$invalid_addresses = $this->GroupInvite->getInvalidAddresses(explode("\n", $this->data['Group']['mailaddress']));
		$invalid_addresses_ml = $this->GroupInvite->getInvalidAddresses(array($this->data['Group']['mailaddress_ml']));
		if ($invalid_addresses || $invalid_addresses_ml) {
			$error_addresses = array_merge($invalid_addresses, $invalid_addresses_ml);
			$this->Session->setFlash($this->createMessage('message.manage_groups.invite.invalid.email', implode(', ', $error_addresses)), 'message');

			return false;
		}
		return true;
	}

	private function renderAdminInputPage($notAgreedMemberIds, $validMemberNameList) {
		if (isset($this->data['Group']['member']) && is_array($this->data['Group']['member'])) {
			$this->data['Group']['member'] = implode(',', array_keys($this->data['Group']['member']));
		}
		// アップロードされたファイルがあれば一時的に保存
		$temporary_file = $this->Group->transferTemporary();
		$this->data['Group']['file'] = is_null($temporary_file) ? null : basename($temporary_file);

		$this->set('notAgreedMemberIds', $notAgreedMemberIds);
		$this->set('addMemberNameList', $validMemberNameList);
		$this->render('admin_input');
	}

	private function _save($confirmed_admin_input = false) {
		if (!$this->_validateInviteAddress()) {
			return;
		}

		$addMemberIdList = array();
		if (isset($this->data['Group']['member'])) {
			$validMemberNameList = $this->getValidMemberNameList(array_keys($this->data['Group']['member']));
			$addMemberIdList = array_keys($validMemberNameList);
		}

		if (!$confirmed_admin_input) {
			$notAgreedMemberIds = $this->TakerAgreement->getNotAgreedToMail($addMemberIdList);
			// 同意がないユーザーがある場合は入力ページを表示
			if ($notAgreedMemberIds) {
				// 他の項目の入力値のエラーがあるか先にチェック
				$this->Group->set($this->data);
				$errors = $this->validateErrors($this->Group);
				if ($errors) {
					$this->Session->setFlash(__("message.input.error", true), 'message');
					return;
				}
				return $this->renderAdminInputPage($notAgreedMemberIds, $validMemberNameList);
			}
		}

		$this->Transaction->begin();
		$group = $this->Group->registerGroup($this->data['Group']);
		if (!$group) {
			$this->Session->setFlash(__("message.input.error", true), 'message');
			$this->Transaction->rollback();
			return;
		}
		$groupid = $this->Group->getLastInsertID();

		// 作成者を管理者＋メンバーとして初期登録
		$mygroup = array(
			'Mygroup' => array(
				'groupid' => $groupid,
				'account_id' => $this->getAccountID(),
				'admin' => 2,
			),
		);
		$result = $this->Mygroup->save($mygroup);
		if (!$result) {
			$this->Transaction->rollback();
			$this->log('GroupsController#create Mygroup save error', 'error');
			$this->cakeError('systemError');
			return;
		}

		// SPコネクタと接続
		$result = $this->connectSPC($groupid, $group['Group']['group_key']);
		if (!$result) {
			$this->Transaction->rollback();
			$this->log('GroupsController#create SPC connect error', 'error');
			$this->cakeError('systemError');
			return;
		}

		$result = $this->_invite($groupid, $group, $addMemberIdList);
		if (!$result) {
			$this->Transaction->rollback();
			$this->cakeError('systemError');
			return;
		}
		$this->Transaction->commit();

		$this->Session->setFlash(__("message.groups.create", true), 'message');
		$this->redirect('/group_lists/detail/'.$groupid);
	}

	private function getValidMemberNameList($memberIdList) {
		$members = $this->getMymembers(null);
		$checkKeys = $this->GroupInvite->getCheckKeys(false);
		$memberNameList = array();
		foreach ($memberIdList as $accountId) {
			foreach ($checkKeys as $checkKey) {
				if (isset($members[$checkKey][$accountId])) {
					$memberNameList[$accountId] = $members[$checkKey][$accountId];
				}
			}
		}
		return $memberNameList;
	}

	private function _invite($groupid, $group, $addMemberIdList) {
		$account = $this->Account->find('first', array('fields' => array('name', 'name_en'),
			'conditions' => array('id' => $this->getAccountID())));
		$group = $this->GroupInvite->findGroup($groupid);
		$group_data = $this->data['Group'] + array('adminInput' => array());
		// address (invite_lang_individual)
		$result = $this->GroupInvite->execute(array(
			'accountId' => $this->getAccountID(),
			'group' => $group,
			'addresses' => explode("\n", $this->data['Group']['mailaddress']),
			'addressMl' => '',
			'memberIdList' => '',
			'adminInput' => $group_data['adminInput'],
			'body' => $this->GroupInvite->getTemplate(
				$this->data['Group']['invite_lang_individual'], $group, $account['Account']),
			'lang' => $this->data['Group']['invite_lang_individual'],
			'accountLang' => $this->getLanguage(),
			'forcibleAdmission' => false,
		));
		if (!$result) {
			return $result;
		}

		// ML + memberIdList (invite_lang)
		$result = $this->GroupInvite->execute(array(
			'accountId' => $this->getAccountID(),
			'group' => $group,
			'addresses' => array(),
			'addressMl' => $this->data['Group']['mailaddress_ml'],
			'memberIdList' => $addMemberIdList,
			'adminInput' => $group_data['adminInput'],
			'body' => $this->GroupInvite->getTemplate($this->data['Group']['invite_lang'], $group, $account['Account']),
			'lang' => $this->data['Group']['invite_lang'],
			'accountLang' => $this->getLanguage(),
			'forcibleAdmission' => false,
		));
		return $result;
	}

	// 自動接続のSPコネクタと接続する
	private function connectSPC($groupid, $groupKey) {
		$spcidList = $this->SpAutoConnector->findSpcList();
		foreach ($spcidList as $spcid) {
			// SPコネクタ接続
			$AddGroup = array(
				'top_id' => $spcid,
				'bottom_id' => $groupid,
				'top' => 1,
				'bottom' => 0,
			);
			$this->AddGroup->create();
			$result = $this->AddGroup->save(array('AddGroup' => $AddGroup));
			if (!$result) {
				return false;
			}

			// SPの利用属性に合わせて、自身を同意させる
			$attributeList = $this->TakerAgreement->getAttributeListForSave($spcid);
			// 利用属性を登録
			$result = $this->TakerAgreement->saveAttribute(
					$groupid, $attributeList, array($spcid), $this->getAccountID());
			if (!$result) {
				return false;
			}

			// calbackUrlを取得
			$serviceInfoList = $this->GroupSphost->findList($spcid);
			foreach ($serviceInfoList as $serviceInfo) {
				$spid = $serviceInfo['GroupSphost']['sp_id'];
				$callbackUrl = $serviceInfo['GroupSphost']['callback_url'];
				if (empty($callbackUrl)) {
					continue;
				}
				$callbackUrl = $this->ParameterReplacer->replaceGroupKey($groupKey, $callbackUrl);
//				$callbackUrl = $this->ParameterReplacer->replaceEntityId($callbackUrl);
				$account = $this->Account->find('first', array('conditions' => array('id' => $this->getAccountID())));
				$callbackUrl = $this->ParameterReplacer->replaceAdminEmail($account['Account']['mail'], $callbackUrl);
				// サービスのAPI呼び出し
				$json = $this->Callback->call($spid, $callbackUrl);
				if (!isset($json) || ($json['result'] != 'OK' && $json['result'] != 'ok')) {
					$this->Session->setFlash(__("message.failed.service.api", true), 'message');
				}
			}
		}
		return true;
	}

	private function getMymembers($groupid) {
		return $this->AdminGroup->getMembers($this->getAccountID(), $this->getLanguage(), $groupid, false);
	}
}
?>
