<?php
/**
 * Mygroup Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Controller', 'Manages');

class MygroupsController extends ManagesController {
	var $name = 'Mygroups';
	var $uses = array('Group', 'SpGroup', 'AddGroup', 'GroupSphost',
			'SpHost', 'AgreementEntrustment', 'Account', 'Mygroup', 'ProvideAttribute',
			'AutoGroup');
	var $helpers = array('Javascript');
	var $components = array('ParameterReplacer', 'GroupAdministrator', 'GroupInvite',
			'AdminGroup', 'TakerAgreement', 'SpcSortor', 'Csrf');

	function beforeFilter() {
		$this->Csrf->validatePost = false;
		parent::beforeFilter();
	}

	/**
	 * MYグループを表示する。
	 */
	function view() {
		// アカウント名が未設定なら、アカウント画面に遷移する
		$exitAccount = $this->Account->hasAccountName($this->getAccountID());
		if (!$exitAccount) {
			$this->Session->setFlash(__("accounts.need.regist", true), 'message');
			$this->redirect('/accounts/edit');
			return;
		}

		// 自分が所属しているグループを検索
		$mygroups = $this->getMyGroupList();

		// グラフ設定
		$mygroupIds = array();
		foreach ($mygroups as $group) {
			array_push($mygroupIds, $group['Group']['id']);
		}

		// MyグループのグループID
		$addGroupIds = array();
		// Myグループと上位グループのグループID
		$myUpGroupIds = array();
		foreach (array_keys($mygroups) as $i) {
			$addGroupIds[ $mygroups[$i]['Group']['id'] ] = $mygroups[$i]['Group']['id'];
			array_push($myUpGroupIds, $mygroups[$i]['Group']['id']);
		}

		// Myグループの上グループ、下グループを取得
		$addList = $this->AddGroup->find('all', array(
			'fields' => array('id', 'top_id', 'bottom_id', 'top', 'bottom'),
			'conditions' => array('OR' => array('top_id' => $mygroupIds, 'bottom_id' => $mygroupIds)),
			));
		foreach ($addList as $group) {
			$addGroupIds[ $group['AddGroup']['top_id'] ] = $group['AddGroup']['top_id'];
			$addGroupIds[ $group['AddGroup']['bottom_id'] ] = $group['AddGroup']['bottom_id'];
		}

		// Myグループの上グループを検索
		$upGroups = $this->AddGroup->find('all', array(
			'fields' => array('top_id', 'bottom_id'),
			'conditions' => array('bottom_id' => $mygroupIds)));
		$ids = array();
		foreach($upGroups as $group) {
			array_push($ids, $group['AddGroup']['top_id']);
			array_push($myUpGroupIds, $group['AddGroup']['top_id']);
		}
		// Myグループの上グループの上SPグループを検索
		$addList2 = $this->AddGroup->find('all', array(
			'fields' => array('id', 'top_id', 'bottom_id', 'top', 'bottom'),
			'conditions' => array('bottom_id' => $ids, 'top' => 1)
			));
		foreach ($addList2 as $group) {
			$addGroupIds[ $group['AddGroup']['top_id'] ] = $group['AddGroup']['top_id'];
			$addGroupIds[ $group['AddGroup']['bottom_id'] ] = $group['AddGroup']['bottom_id'];
			array_push($myUpGroupIds, $group['AddGroup']['top_id']);
		}
		$this->set('group_administrators', $this->GroupAdministrator->get($addGroupIds, $this->getLanguage()));


		// 委任状態を設定
		$entrustmentList = $this->AgreementEntrustment->find('list', array(
			'fields' => array('group_id', 'group_id'),
			'conditions' => array('account_id' => $this->getAccountID())));
		$this->set('entrustmentList', $entrustmentList);
//return;

		// サービスのリンクを取得
		$linkGroupIdList = array();
		foreach ($mygroups as $mygroup) {
			array_push($linkGroupIdList, $mygroup['Group']['id']);
		}
		// Myグループの上グループを検索
		$upIdList = $this->AddGroup->find('list', array(
			'fields' => array('top_id', 'top_id'), 'conditions' => array('bottom_id' => $linkGroupIdList, 'top' => '0')));
		foreach ($upIdList as $id) {
			array_push($linkGroupIdList, $id);
		}
		$linkGroupIdList = array_unique($linkGroupIdList);
//return;
		
		$iconList = $this->SpGroup->findIconList();
		$groupIdSpcIdList = $this->getGroupIdSpcIdList($linkGroupIdList);
		$relatedUrlList = array();

		// SPホスト名一覧を事前に取得
		$spHostList = $this->SpHost->getSpHostList();
		foreach ($linkGroupIdList as $id) {
			if ($this->AutoGroup->isIdPGroup($id)) {
				continue;
			}
			$urlList = $this->GroupSphost->getServiceLink($id, $this->getLanguage(), false,
					$iconList, $groupIdSpcIdList, $spHostList);
			$privateServiceIdList = $this->Group->find('list', array(
				'fields' => array('id'),
				'conditions' => array(
					'id' => array_keys($urlList),
					'sp' => 2)));
			foreach ($privateServiceIdList as $private_service_id) {
				if (isset($urlList[$private_service_id])) {
					unset($urlList[$private_service_id]);
				}
			}
			$relatedUrlList[$id] = $urlList;
		}
//return;
		// ユーザ同意
		// MYグループに表示するSPコネクタを取得
		$useSpcIdList = array();
		foreach($relatedUrlList as $spcList) {
			foreach($spcList as $spc) {
				$useSpcIdList[ $spc[2] ] = $spc[2];
			}
		}
//debug($relatedUrlList);
//debug($useSpcIdList);
		// ユーザの同意情報を取得
		$userAgreeList = $this->TakerAgreement->servData($this->getAccountID(), $this->getLanguage(), $useSpcIdList);
//debug($userAgreeList);
//return;
		// SPコネクタの同意情報を反映
		$provideList = $this->ProvideAttribute->find('all',
				array('conditions' => array('group_id' => $useSpcIdList)));
		foreach($mygroupIds as $id) {
			$userAgreeList = $this->TakerAgreement->reflectSPCAgreement(
						$userAgreeList, $useSpcIdList, $id, $provideList);
		}
		if (!empty($userAgreeList) && !empty($userAgreeList['Agreement'])) {
			$this->set('userAgreeList', $userAgreeList['Agreement']);
		} else {
			$this->set('userAgreeList', array());
		}
//debug($userAgreeList);
//debug($relatedUrlList);
//return;
		// SPコネクタの並び順はデフォルト名前順になっている
		// アクセス頻度順、最終アクセス順が設定されていたら、自力で並び変える
		$relatedUrlList = $this->SpcSortor->updateSortForMyGroup(
			$relatedUrlList, $this->getAccountID(), $this->getIdentityProvider());

//debug($relatedUrlList);

		$this->set('mygroups', $mygroups);
		$this->set('relatedUrlList', $relatedUrlList);

		// SPアイコンリストを取得
		$spcIconList = $this->SpGroup->findIconList();
		$this->set('spcIconList', $spcIconList);
	}

	/**
	 * グループIDごとのSPCID一覧を求める
	 */
	private function getGroupIdSpcIdList($groupIdList) {
		$addGroupList = $this->AddGroup->find('all', array(
				'fields' => array('bottom_id', 'top_id'),
				'conditions' => array(
					'bottom_id' => $groupIdList, 'top' => 1,
					'top_id in (select id from groups where sp=1 and active = 0)')));

		$groupIdSpcIdList = array();
		foreach($addGroupList as $addgroup) {
			if (empty($groupIdSpcIdList[ $addgroup['AddGroup']['bottom_id'] ])) {
				$groupIdSpcIdList[ $addgroup['AddGroup']['bottom_id'] ] = array();
			}
			array_push($groupIdSpcIdList[ $addgroup['AddGroup']['bottom_id'] ],
			$addgroup['AddGroup']['top_id']);
		}
		return $groupIdSpcIdList;
	}

	/**
	 * 指定したグループをお気に入りに登録する
	 */
	public function favoriteOn($groupid) {
		// 自分のグループかを確認
		if (!$this->Mygroup->isMember($this->getAccountID(), $groupid)) {
			$this->redirect('/mygroups/view');
			return;
		}

		$this->Mygroup->updateFavoriteOn($this->getAccountID(), $groupid);
		$this->redirect('/mygroups/view');
	}

	/**
	 * 指定したグループをお気に入りから解除する
	 */
	public function favoriteOff($groupid) {
		// 自分のグループかを確認
		if (!$this->Mygroup->isMember($this->getAccountID(), $groupid)) {
			$this->redirect('/mygroups/view');
			return;
		}

		$this->Mygroup->updateFavoriteOff($this->getAccountID(), $groupid);
		$this->redirect('/mygroups/view');
	}

	/**
	 * [既存グループからの選択]element用データ設定.
	 */
	public function member() {
		//	テンプレートから
		//		$this->element("mygroups_member", $this->requestAction('/mygroups/member', $params))
		//
		//	$this->params = array(
		//		'groupid' => $group_id,
		//		'model' => $modelName,
		//		'data' => array(),
		//	);
		$forcibleAdmission = $this->GroupInvite->canForce();
		$groupid = isset($this->params['groupid']) ? $this->params['groupid'] : null;
		$this->params['data'] += array('member' => array());

		return array(
			'model' => $this->params['model'],
			'members' => $this->getMymembers($groupid, $forcibleAdmission),
			'check_members' => $this->params['data']['member'],
		);
	}

	private function getMymembers($groupid, $forcibleAdmission) {
		return $this->AdminGroup->getMembers($this->getAccountID(), $this->getLanguage(), $groupid, $forcibleAdmission);
	}

//	/**
//	 *
//	 * @param string $groupNames
//	 * @param integer $spGroupId
//	 * @param array $add
//	 * @param array $groupSphosts
//	 * @param array $relatedUrlList
//	 * @param array $spHostList
//	 */
//	private function getSpLeadUrl($groupNames, $spGroupId, $add, $groupSphosts, $relatedUrlList, $spHostList) {
//		$spInfos = array();
//		foreach ($groupNames as $group) {
//			if ($group['Group']['id'] != $spGroupId || $group['Group']['sp'] != 1) {
//				continue;
//			}
//			$cnt = 0;
//			foreach ($groupSphosts as $host) {
//				if ($spGroupId != $host['GroupSphost']['group_id']) {
//					continue;
//				}
//				$spInfos[$cnt]['url'] = $host['GroupSphost']['lead_url'];
//
//				$serviceName = '';
//				if ($this->getLanguage() === 'ja') {
//					if (!empty($host['GroupSphost']['service_name']))
//						$serviceName = $host['GroupSphost']['service_name'];
//					else if (!empty($host['GroupSphost']['service_name_en']))
//						$serviceName = $host['GroupSphost']['service_name_en'];
//				} else {
//					if (!empty($host['GroupSphost']['service_name_en']))
//						$serviceName = $host['GroupSphost']['service_name_en'];
//					else if (!empty($host['GroupSphost']['service_name']))
//						$serviceName = $host['GroupSphost']['service_name'];
//				}
//				$spInfos[$cnt]['name'] = $serviceName;
//
//				if (empty($spInfos[$cnt]['name'])) {
//					$spInfos[$cnt]['name'] = $spHostList[ $host['GroupSphost']['sp_id'] ];
//				}
//				$cnt++;
//			}
//			break;
//		}
//		if (empty($spInfos)) {
//			return $relatedUrlList;
//		}
//		if (empty($relatedUrlList[ $add['AddGroup']['bottom_id'] ])) {
//			$relatedUrlList[ $add['AddGroup']['bottom_id'] ]['name'] = array();
//			$relatedUrlList[ $add['AddGroup']['bottom_id'] ]['url'] = array();
//		}
//
//		foreach ($spInfos as $info) {
//			$same = false;
//			$list = $relatedUrlList[ $add['AddGroup']['bottom_id'] ];
//			for ($i = 0; $i < count($list['name']); $i++) {
//				if ($list['name'][$i] === $info['name']
//					&& $list['url'][$i] === $info['url']) {
//					$same = true;
//					break;
//				}
//			}
//			if ($same) {
//				continue;
//			}
//			array_push($relatedUrlList[ $add['AddGroup']['bottom_id'] ]['name'], $info['name']);
//			array_push($relatedUrlList[ $add['AddGroup']['bottom_id'] ]['url'], $info['url']);
//		}
//		return $relatedUrlList;
//	}
}
?>
