<?php
/**
 * SP Connector Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Controller', 'Manages');

class SpGroupsController extends ManagesController {
	var $name = 'SpGroups';
	var $uses = array('SpGroup', 'Mygroup', 'ManageGroup', 'SpHost', 'AddGroup',
			'GroupSphost', 'SpAdministrator', 'Transaction', 'ProvideAttribute', 'Tag',
			'ExtendedAttribute');
	var $components = array('ParameterReplacer', 'ExtendedAttributeUtils');
	var $helpers = array('Javascript', 'Attribute');

	function beforeFilter() {
		parent::beforeFilter();
		$groupid = $this->_getGroupIdFromURL();
		$this->_checkAccessible($groupid);
	}

	protected function _checkAccessible($groupid) {
		// SPコネクタ作成画面は、SP管理者のみ操作できる。
		if ($this->params['action'] === 'create') {
			$count = $this->SpAdministrator->find('count', array('conditions' => array(
					'OR' => array('eppn' => $this->getEPPN(), 'eptid' => $this->getEPTID()))));
			if ($count <= 0) {
				$this->cakeError('cannotAccess', array('url' => $this->here));
			}
			return;
		}
		if (is_null($groupid) || !$this->Group->isSpConnector($groupid)) {
			$this->cakeError('notFound', array());
			return;
		}
		parent::_checkAccessible($groupid);
	}

	/**
	 * SP一覧を取得する。
	 * @param $groupid
	 */
	function getSpHostList($groupid = null) {
		// 既にSPコネクタの対象SPとなっていれば表示する(別管理者によって登録された)
		$selectedIdList = array();
		if ($groupid != null) {
			// 選択済みSP一覧を取得
			$groupSphosts =$this->GroupSphost->find('all', array(
					'fields' => array('sp_id'),
					'conditions' => array('group_id' => $groupid)));
			foreach($groupSphosts as $sphosts) {
				array_push($selectedIdList, $sphosts['GroupSphost']['sp_id']);
			}
		}

		// SP一覧を取得
		$spEntityIdList = $this->SpAdministrator->find('list', array(
			'fields' => array('id', 'entityid'),
			'conditions' => array('OR' => array('eppn' => $this->getEPPN(), 'eptid' => $this->getEPTID()))));
		$spHosts = $this->SpHost->find('all', array('fields' => array('id', 'name', 'entityid', 'service_name'),
				'conditions' => array(
					'OR' => array('entityid' => $spEntityIdList, 'id' => $selectedIdList),
					'status' => '0'),
				'order' => array('name')));
		$hostList = array();
		foreach($spHosts as $host) {
			$hostList[ $host['SpHost']['id'] ]['name'] = $host['SpHost']['name'];
			$hostList[ $host['SpHost']['id'] ]['entityid'] = $host['SpHost']['entityid'];
			$hostList[ $host['SpHost']['id'] ]['service_name'] = $host['SpHost']['service_name'];
		}
		return $hostList;
	}

	/**
	 * 対象SPは対象としてよいSPなのかを確認する(不正利用対策)。
	 * 既に他の管理者によって選択状態なら該当しなくてよい。
	 * @param unknown_type $myGroupid
	 */
	function canSetTargetSp($myGroupid = null) {
		$spIdList = array();
		$can = true;
		if (!empty($this->data['ManageSpGroup']['sp_host'])) {
			foreach ($this->data['ManageSpGroup']['sp_host'] as $spid) {
				array_push($spIdList, $spid);
			}
		}
		$spHostList = $this->SpHost->find('all', array('fields' => array('id', 'name', 'entityid'),
			'conditions' => array('id' => $spIdList)));
		$spAdminList = $this->SpAdministrator->find('all', array(
			'fields' => array('entityid'),
			'conditions' => array('OR' => array('eppn' => $this->getEPPN(), 'eptid' => $this->getEPTID()))));
		$badHostString= '';
		foreach ($spHostList as $host) {
			$same = false;
			foreach ($spAdminList as $admin) {
				if ($host['SpHost']['entityid'] === $admin['SpAdministrator']['entityid']) {
					$same = true;
					break;
				}
			}
			if (!$same) {
				// SPコネクタの更新時のみ
				if ($myGroupid != null) {
					// 既に他の管理者によって登録されていれば対象となる(上書き登録可能)
					$count = $this->GroupSphost->find('count', array('conditions' => array(
						'sp_id' => $host['SpHost']['id'], 'group_id' => $myGroupid)));
					if ($count == 0) {
						$can = false;
					}
				// SPコネクタ作成時はDBを確認しない(新規作成なので他の管理者が登録していない)
				} else {
					$can = false;
				}

				if (!$can) {
					if (!empty($badHostString)) {
						$badHostString = $badHostString.'　';
					}
					$badHostString = $badHostString.$host['SpHost']['name'];
				}
			}
		}

		if (!$can) {
			$this->Session->setFlash(__("message.sp_groups.cannot.select.target.sp", true).' （'.$badHostString.'）', 'message');
		}
		return $can;
	}

	/**
	 * グループ詳細画面を表示する。
	 * @param $groupid
	 */
	function detail($groupid) {
		$SpGroup = $this->SpGroup->find('first',
			array('conditions' => array('id' => $groupid)));
		// 多言語対応
		if ($this->getLanguage() === 'ja') {
			if (!empty($SpGroup['SpGroup']['name']))
				$SpGroup['SpGroup']['name_en'] = '';
			if (!empty($SpGroup['SpGroup']['introduction']))
				$SpGroup['SpGroup']['introduction_en'] = '';
		} else {
			if (!empty($SpGroup['SpGroup']['name_en']))
				$SpGroup['SpGroup']['name'] = '';
			if (!empty($SpGroup['SpGroup']['introduction_en']))
				$SpGroup['SpGroup']['introduction'] = '';
		}
		$SpGroup['spc_icon'] = $this->SpGroup->getIcon($groupid);
		$this->set('SpGroup', $SpGroup);

		// 自グループの子グループ(SPコネクタ、グループ)数を検索
		$count = $this->AddGroup->find('count',
			array('conditions' => array('top_id' => $groupid),
		));
		$this->set('groupCounts', $count);

		// 自SPコネクタの参加メンバー数を検索
		$count = $this->Mygroup->find('count', array('fields' => 'count(DISTINCT account_id) as count',
			'conditions' => array('groupid in (
			SELECT bottom_id FROM add_groups  where top_id= ? '.
			' union '.
			'select bottom_id from add_groups where top_id
			    in (SELECT bottom_id FROM add_groups  where top_id=?) '.
			' union select bottom_id from add_groups where top_id '.
				'in (select bottom_id from add_groups where top_id '.
					'in (SELECT bottom_id FROM add_groups where top_id=?)))' => array($groupid, $groupid, $groupid)
			)
		));
		$this->set('memberCounts', $count);

		// 対象SP検索
		$spHostList = $this->SpHost->find('all', array(
			'conditions' => array(
				'id in (select sp_id from group_sphosts where group_id = ?)' => $groupid,
				'status' => 0
				),
			'order' => array('name')
			));
		$groupSpHostList = $this->GroupSphost->find('all', array(
				'conditions' => array('group_id' => $groupid)));
		$spList = array();
		foreach ($spHostList as $spHost) {
			$id = $spHost['SpHost']['id'];
			foreach ($groupSpHostList as $rec) {
				if ($rec['GroupSphost']['sp_id'] != $id) {
					continue;
				}

				$serviceName = $this->GroupSphost->getServiceName($this->getLanguage(), $rec);
				// URL判定
				$nolink = false;
				if (!preg_match('/^(http:|https:)/i', $rec['GroupSphost']['lead_url'])) {
					$nolink = true;
				} else if (preg_match('/\$groupID|\$entityID/i', $rec['GroupSphost']['lead_url'])) {
					$nolink = true;
				}

				// 未設定の場合はホスト名を設定する
				if (empty($rec['GroupSphost']['service_name'])) {
					$serviceName = $spHost['SpHost']['name'];
				}
				// 未設定の場合はentityidを利用する
				if (empty($rec['GroupSphost']['lead_url'])) {
					$nolink = true;
				}
				if ($nolink) {
					array_push($spList, array(
						'service_name' => $serviceName,
						'lead_url' => str_replace('$name', '', $rec['GroupSphost']['lead_url']),
						'nolink' => $nolink,
						'entityid' => $spHost['SpHost']['entityid']));
				} else {
					array_unshift($spList, array(
						'service_name' => $serviceName,
						'lead_url' => str_replace('$name', '', $rec['GroupSphost']['lead_url']),
						'nolink' => $nolink,
						'entityid' => $spHost['SpHost']['entityid']));
				}
				break;
			}
		}
		$this->set('spList', $spList);
		// SPコネクタ管理者の設定
		$this->set('spAdmin', false);
		$isAdmin = $this->isGroupAdmin($groupid);
		if ($isAdmin) {
			$this->set('spAdmin', true);
		}

		// 利用属性を設定
		$providerAttribute = $this->ProvideAttribute->find('first',
				array('conditions' => array('group_id' => $groupid)));
		$this->set('providerAttribute', $providerAttribute);

		$tagNames = array_values($this->Tag->getPublicTagNames(null, $groupid));
		$this->set('tagNames', $tagNames);
	}

	/**
	 * SPコネクタを作成する。
	 */
	function create() {
		switch ($this->getSubmitName()) {
			case '':
				// SP一覧を取得
				$hostList = $this->getSpHostList();
				$this->set('hostList', $hostList);

				// 予めサービス名とサービスのリンクを設定する
				$keyList = array_keys($hostList);
				foreach($keyList as $key) {
					$this->data['SpGroup']['service_name'.$key] = $hostList[$key]['service_name'];
					$url = parse_url($hostList[$key]['entityid']);
					if (preg_match('/^https?$/', $url['scheme'])) {
						$leadUrl = $url['scheme'].'://'.$url['host'].'/';
					} else {
						$leadUrl = null;
					}
					$this->data['SpGroup']['lead_url'.$key] = $leadUrl;
				}

				$candidateTagNames = array_values($this->Tag->getPublicTagNames($this->getAccountID()));
				$this->set('candidateTagNames', $candidateTagNames);

				// 拡張属性を設定
				$extendedAttributes = array();
				$extendedAttrNames = array();
				$this->data['ManageSpGroup']['attr_extended'] = 0;
				$this->set('extendedAttrNames', $extendedAttrNames);
				$this->set('extendedAttributes', $extendedAttributes);
				break;
			case 'save':
				// 対象SPは対象としてよいSPなのかを確認(不正利用対策)
				$isSetTargetSP = $this->canSetTargetSp();
				if (!$isSetTargetSP) {
					$this->set('hostList', $this->getSpHostList());
					return;
				}

				if (empty($this->data)) {
					return;
				}

				$this->Transaction->begin();
				$group_id = $this->save();
				if ($group_id === 'validate_error') {
					$this->Transaction->rollback();
					$this->set('hostList', $this->getSpHostList());
					$this->ExtendedAttributeUtils->extendedShow('SpGroup');
					$this->Session->setFlash(__("message.input.error", true), 'message');
					return;
				} elseif (!$group_id) {
					$this->Transaction->rollback();
					$this->Session->setFlash(__("common.error.occurred", true), 'message');
					$this->redirect('/sp_groups/create');
					return;
				}
				$this->Transaction->commit();

				$this->Session->setFlash(__("message.sp_groups.create", true), 'message');
				$this->redirect('/sp_groups/detail/'.$group_id);
				break;
			case 'cancel':
				$this->redirect('/mygroups/view');
				return;
			default:
				break;
		}
	}

	private function save() {
		// グループに登録
		$this->data['SpGroup']['sp'] = 1;
		$this->data['SpGroup']['inspectjoin'] = 1; // 入会時、管理者の承認が必要
		$result = $this->SpGroup->save($this->data['SpGroup']);
		// バリデーションエラー
		if (!$result) {
			return 'validate_error';
		}
		$group_id = $this->SpGroup->getLastInsertID();

		// グループSPホストに登録
		if (!empty($this->data['SpGroup']['sp_host'])) {
			$isError = false;
			foreach ($this->data['SpGroup']['sp_host'] as $spid) {
				$group_sp = array(
					'group_id' => $group_id,
					'sp_id' => $spid,
					'service_name' => $this->data['SpGroup']['service_name'.$spid],
//					'service_name_en' => $this->data['SpGroup']['service_name_en'.$spid],	画面にservice_name_enが無いのでコメントアウト
					'lead_url' => $this->data['SpGroup']['lead_url'.$spid],
					'lead_non_map_url' => $this->data['SpGroup']['lead_non_map_url'.$spid],
					'callback_url' => $this->data['SpGroup']['callback_url'.$spid],
					'logout_option' => $this->data['SpGroup']['logout_option'.$spid],
					'relogin_url' => $this->data['SpGroup']['relogin_url'.$spid],
				);
				if (isset($this->data['SpGroup']['service_link_both'.$spid][0])) {
					$group_sp['service_link_both'] = $this->data['SpGroup']['service_link_both'.$spid][0];
				}
				$this->GroupSphost->create();
				$result = $this->GroupSphost->save(array('GroupSphost' => $group_sp));
				if (!$result) {
					$isError = true;
					foreach ($this->GroupSphost->validationErrors as $k => $validationError) {
						$this->SpGroup->validationErrors[$k.$spid] = $validationError;
					}
				}
			}
			if ($isError) {
				return  'validate_error';
			}
		}

		// 提供属性を登録
		$attribute = array(
			'group_id' => $group_id,
			'ismemberof' => 0,
			'eptid' => 0,
			'name' => 0,
			'mail' => 0,
			'eppn' => 0,
			'organization' => 0,
			'idp' => 0,
			'language' => 0,
			'introduction' => 0,
		);
		if (!empty($this->data['SpGroup']['attr_ismemberof'][0]))
			$attribute['ismemberof'] = $this->data['SpGroup']['attr_ismemberof'][0];
		if (!empty($this->data['SpGroup']['attr_eptid'][0]))
			$attribute['eptid'] = $this->data['SpGroup']['attr_eptid'][0];
		if (!empty($this->data['SpGroup']['attr_name'][0]))
			$attribute['name'] = $this->data['SpGroup']['attr_name'][0];
		if (!empty($this->data['SpGroup']['attr_mail'][0]))
			$attribute['mail'] = $this->data['SpGroup']['attr_mail'][0];
		if (!empty($this->data['SpGroup']['attr_eppn'][0]))
			$attribute['eppn'] = $this->data['SpGroup']['attr_eppn'][0];
		if (!empty($this->data['SpGroup']['attr_organization'][0]))
			$attribute['organization'] = $this->data['SpGroup']['attr_organization'][0];
		if (!empty($this->data['SpGroup']['attr_idp'][0]))
			$attribute['idp'] = $this->data['SpGroup']['attr_idp'][0];
		if (!empty($this->data['SpGroup']['attr_language'][0]))
			$attribute['language'] = $this->data['SpGroup']['attr_language'][0];
		if (!empty($this->data['SpGroup']['attr_introduction'][0]))
			$attribute['introduction'] = $this->data['SpGroup']['attr_introduction'][0];
		$result = $this->ProvideAttribute->save(array('ProvideAttribute' => $attribute));
		if (!$result) {
			return false;
		}

		// 作成者を管理者として初期登録
		$mygroup = array(
			'groupid' => $group_id,
			'account_id' => $this->getAccountID(),
			'admin' => 1,
		);
		$result = $this->Mygroup->save(array('Mygroup' => $mygroup));
		if (!$result) {
			return false;
		}

		$result = $this->Tag->updatePublicTags($this->getAccountID(), $group_id, $this->data['tagNames']);
		if (!$result) {
			return false;
		}

		// 拡張属性を保存する
		$result = $this->ExtendedAttributeUtils->saveExtended($group_id, 'SpGroup');
		if (!$result) {
			return 'validate_error';
		}

		return $group_id;
	}
}
?>
