<?php
/**
 * Top Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class TopController extends AppController {
	var $name = 'Top';
	var $uses = array('Transaction', 'Group', 'SpGroup', 'AutoGroup', 'GroupSphost', 'ProvideAttribute',
			'Mygroup', 'AddGroup', 'UserSpSort', 'SpcSort', 'Account', 'AccountEppn', 'SpHost',
			'IdpAdministrator', 'Tag', 'Organization');
	var $components = array('SpcSortor', 'RequestHandler');

	var $helpers = array('Javascript');

	function beforeRender() {
		parent::beforeRender();
		$this->layout = 'cloudgateway';
	}

	var $accountIdForServices = null;
	/* for connector controller */
	protected function setAccountIdForServices($idpEntityid, $eppn) {
		$account_id = false;
		if ($eppn !== '@me') {
			$account_id = $this->AccountEppn->field('account_id',
				array('eppn' => $eppn,
				      'entityid' => $idpEntityid));
		}
		// false(@meまたはmAPのアカウントが存在しない)の場合は、@meと同じ挙動とする。
		// CGP/mAPにログインしたユーザしかDBに登録されないため未ログインユーザのeppnを指定した際にNotFoundとなるのを回避するため。
		$this->accountIdForServices = $account_id;
	}
	/* override for connector controller */
	protected function getAccountID() {
		if (isset($this->accountIdForServices)) {
			return $this->accountIdForServices;
		}
		return parent::getAccountID();
	}

	/**
	 * トップURLの処理
	 * CGポータル画面の表示または、MYグループ画面を表示する
	 */
	function index() {
		if (!$this->Session->read('useCG')) {
			// 参加機関か確認する
			if (!$this->_isParticipant()) {
				$this->Session->write('nonParticipant', true);
				$this->Session->write('gakuninParticipant', !$this->_isNonGakuninParticipant());
				$this->redirect('/mygroups/view');
			}

			$autoGroup = $this->AutoGroup->findIdPGroup($this->getIdentityProvider());
			if (empty($autoGroup)) {
				// IdPグループが存在しないのでMYグループ画面を表示する
				$this->redirect('/mygroups/view');
				return;
			}

			// CGポータルを利用可能にする
			$idpGroupId = $autoGroup['AutoGroup']['group_id'];
			$this->useCGPortal($idpGroupId);
			$this->Session->write('useCG', true);
		}

		// 検証OKなので、CGポータル画面を表示する
		$this->showCGPortal();
	}

	/**
	 * CGポータルを利用可能にします。
	 * 自動アカントを作成し、IdPグループに参加させます。
	 */
	private function useCGPortal($idpGroupId) {
		$this->Transaction->begin();

		$eppn = $this->getEPPN();
		$eptid = $this->getEPTID();
		if (empty($eppn) && empty($eptid)) {
			$result = $this->useGuestUser($idpGroupId);
		} else {
			$result = $this->useNormalUser($idpGroupId);
		}
		if (!$result) {
			$this->Transaction->rollback();
			$this->cakeError('systemError');
			return;
		}

		$this->Transaction->commit();
	}

	/**
	 * eppn,eptidがないのでゲストユーザとしてCGポータルを利用します。
	 */
	private function useGuestUser($idpGroupId) {
		// eppn eptidの両方ともに無い場合は、GUESTとして動作させる
		// 大学毎のゲストアカントがない場合は、ゲストアカウントを作成
		// ゲストアカウントでログイン済みにする
		// ゲストアカウントの場合、サービスの並び順モーダルは表示しない
		// ゲストアカウントの場合、sortableもできないようにする
		$entityId = $this->getEntityID();
		$accountEppn = $this->AccountEppn->findGuest($entityId);
		if (empty($accountEppn)) {
			$accountId = $this->Account->createGuestAccount($this->getOrganizationName());
			if (!$accountId) {
				return false;
			}
			$result = $this->AccountEppn->createGuestAccountEppn($accountId, $entityId);
			if (!$result) {
				return false;
			}

			// IdPグループに入会させる
			if (!$this->Mygroup->isMember($accountId, $idpGroupId)) {
				$result = $this->Mygroup->createMygroup($accountId, $idpGroupId);
				if (!$result) {
					return false;
				}
			}
		} else {
			$accountId = $accountEppn['AccountEppn']['account_id'];
		}

		// ログイン済みに
		$this->Session->write('Login', 'true');
		$this->Session->write('LoginKey', 'guestguest');
		$this->Session->write('accountID', $accountId);
		$this->Session->write('guestMode', true);

		return true;
	}

	/**
	 * 通常のアカウントでCGポータルを利用します。
	 * @return boolean
	 */
	private function useNormalUser($idpGroupId) {
		// アカウントが未作成なら自動で作成する
		$accountId = $this->getAccountID();
		if (empty($accountId)) {
			$accountId = $this->Account->createAutoyAccount();
			if (!$accountId) {
				return false;
			}

			$result = $this->AccountEppn->createAccountEppn(
					$accountId, $this->getEPPN(), $this->getEPTID(), $this->getIdentityProvider());
			if (!$result) {
				return false;
			}
			// ログイン済みに
			$this->setLoggedIn();
		}
		
		// IdPグループに入っていなかったら入会させる
		if (!$this->Mygroup->isMember($accountId, $idpGroupId)) {
			$result = $this->Mygroup->createMygroup($accountId, $idpGroupId);
			if (!$result) {
				return false;
			}
		}
		return true;
	}

	/**
	 * 参加機関か確認する
	 */
	private function _isParticipant() {
		$participants = Configure::read('participants');
		$entityId = $this->getEntityID();
		return in_array($entityId, $participants);
	}

	/**
	 * 非学認参加機関か確認する
	 */
	private function _isNonGakuninParticipant() {
		$non_gakunin_participants = Configure::read('non_gakunin_participants');
		$entityId = $this->getEntityID();
		return in_array($entityId, $non_gakunin_participants);
	}

	/**
	 * IdPグループが存在するかの検証を行います
	 * @param unknown $autoGroup
	 */
	private function validateIdpGroup($autoGroup, $allow_no_user) {
		if (empty($autoGroup)) {
			return false;
		}
		$idpGroupId = $autoGroup['AutoGroup']['group_id'];
		// IdPグループに参加しているか確認
		if (!$allow_no_user && !$this->Mygroup->isMember($this->getAccountID(), $idpGroupId)) {
			return false;
		}
		return $idpGroupId;
	}

	/**
	 * 以前との互換性のため、/top/sideは/top/indexにリダイレクトする
	 */
	public function side() {
		$this->redirect('/top/index');
	}

	/**
	 * CGポータル画面表示
	 * ・idpグループが接続しているSPコネクタのサービスを表示する
	 * ・meatwikiとmeatmailに関しては、利用属性の同意にかかわらず表示する
	 * ・これらのサービスが利用できるidpグループ以外のグループに参加していたら、
	 *   そのグループも表示する
	 * ・ismemberof属性が利用有りになっているSPコネクタ(サービス)はグループを表示する
	 *   ismemberof属性の利用なしになっているSPコネクタ(サービス)はグループを表示しない 例）cinii
	 */
	private function showCGPortal() {
		$organizationDisplayName = $this->getOrganizationDisplayName();
		if ($organizationDisplayName) {
			$organizationDisplayName = "(" . $organizationDisplayName . ")";

			$accountLanguage = $this->Session->read('accountLanguage');
			if ($accountLanguage == 'eng') {
				$organizationDisplayName = " " . $organizationDisplayName;
			}
		}
		$this->set("organizationDisplayName", $organizationDisplayName);

		if($this->_side() === false) {
			$this->redirect('/mygroups/view');
			return;
		}
		$this->render('/top/side');
	}
	
	//
	// isMatchGroupKey
	//
	// @param $settings 属性名=属性値の配列
	// @return すべての属性がサーバ変数と一致した場合trueを返す
	//         一組でも不一致の場合は、falseを返す
	//         =がない場合も、falseを返す
	//
	private function isMatchGroupKey($settings) {

		foreach ($settings as $set) {
			$pos = mb_strpos($set, '=');
			$key = mb_substr($set, 0, $pos);
			$val = mb_substr($set, $pos+1);

			if ($this->isServerVariable($key, $val) == false){
				return false;
			}
		}
		return true;
	}

	//
	// _get_subGroupId
	// 機関グループIDのgroup_keyと一致するサブグループIDを取得する
	// @param $idpGroupId  機関グループID
	//
    private function _get_subGroupId($idpGroupId) {

		$group = $this->Group->findGroup($idpGroupId);
		$group_key = $group['Group']['group_key'];
		$sub_groups = $this->Group->findGroup_bygroupkey($group_key);

		foreach($sub_groups as $id => $group_key) {
			$pos = mb_strpos($group_key, '/');
			$str = mb_substr($group_key, $pos+1);
			$settings = explode('&', $str);
			if($this->isMatchGroupKey($settings)) {
				return $id;
			}
		}
		return null;
	}

	protected function _side($idpEntityid=null) {
		// ユーザのIdPグループが利用できるサービスを取得
		// ＋ meatwikiとmeatmailも利用できることにする → IdPグループ作成時に接続しよう
		$allow_no_user = false;
		if (is_null($idpEntityid)) {
			$idpEntityid = $this->getIdentityProvider();
		} else {
			$allow_no_user = true;
		}
	
		// IdPグループを取得
		$autoGroup = $this->AutoGroup->findIdPGroup($idpEntityid);
		// IdPグループを検証
		$idpGroupId = $this->validateIdpGroup($autoGroup, $allow_no_user);
		if ($idpGroupId === false) {
			// IdPグループが存在しないのでMYグループ画面を表示する
			return false;
		}

        // 20201202 Add From
        // 機関サブグループIDを取得
		$subGroupId = $this->_get_subGroupId($idpGroupId);
		if (!is_null($subGroupId)){
			$idpGroupId=$subGroupId;
		}
		// 20201202 Add To

		// サービスの並び順
		$sortType = $this->servSortType($idpGroupId);

		// 表示データ
		//  'SPコネクタID' => array('グループID' => 'サービスのリンク情報')
		$resultList = array();

		// IdPグループに接続しているSPのサービスのリンクを取得
		$serviceLinkList = $this->GroupSphost->getServiceLink($idpGroupId, $this->getLanguage(), true);

		// 表示データに格納
		foreach ($serviceLinkList as $spcid => $serviceLink) {
			$resultList[$spcid] = array($idpGroupId => $serviceLink);
		}
		//debug($resultList);

		// IdPグループに接続していないSPコネクタのうち、IdPタイプのSPコネクタを
		// 使用不可サービスとしてresultListに追加する
		// 
		// TODO:
		// IdPタイプ以外のSPコネクタを使用不可サービスとして追加する際に
		// 使用不可サービスでは「グループ表示」を表示しない修正が必要
		$notConnectServiceLinkList = $this->GroupSphost->getServiceLinkNotConnect($idpGroupId, $this->getLanguage(), true);
		// 表示データに格納
		foreach ($notConnectServiceLinkList as $spcid => $serviceLink) {
			if (isset($resultList[$spcid])) {
				continue;
			}
			if (!$this->SpGroup->isIdPtype($spcid)) {
				continue;
			}
			$resultList[$spcid] = array(0 => $serviceLink);
			$resultList[$spcid]['unusable'] = true;
		}

		// isMemberOf属性を利用するSPコネクタIDの一覧
		$ismemberofSpcIdList = $this->ProvideAttribute->findIsMemberOfSPConnector();

		// メンバーとして所属しているグループに接続するサービスを取得
		$memberGroupIdList = $this->Mygroup->findMyGroupJoinedAsMemberIdList($this->getAccountID());
		$resultList = $this->_setOtherService($memberGroupIdList, $resultList, $ismemberofSpcIdList);

		// メンバーとして所属しているグループの上位グループに接続するサービスを取得
		$upGroupIdList = $this->AddGroup->getTopIdList($memberGroupIdList);
		$resultList = $this->_setOtherService($upGroupIdList, $resultList, $ismemberofSpcIdList);

		// 管理者として所属しているグループに接続するサービスを取得
		$adminGroupIdList = $this->Mygroup->findMyGroupJoinedAsAdminIdList($this->getAccountID());
		$resultList = $this->_setOtherService($adminGroupIdList, $resultList, $ismemberofSpcIdList);

		// PrivateServiceを取得
		$resultList = $this->_setPrivateService($resultList);

		// ハイブリッド型データの取得
		$spcIdGroupIdList = $this->AddGroup->findHybridSpcList($this->getAccountID());
//debug($spcIdGroupIdList);
		foreach ($spcIdGroupIdList as $spcid => $groupidList) {
//debug($groupidList);
			if (empty($groupidList)) {
				continue;
			}
			$resultList = $this->_setOtherService(
					$groupidList, $resultList, $ismemberofSpcIdList, true, $spcid);
		}

		$resultList = $this->SpcSortor->updateSortForDetail($idpGroupId, $resultList, $this->getAccountID());
		// グループの表示順をMYグループの表示順にする
		$resultList = $this->_sortGroup($resultList, $this->getAccountID());

		// 画面に表示するデータを整理する
		//debug($ismemberofSpcIdList);
		$memberGroupNameList = $this->Group->findGroupNameList($memberGroupIdList);
		$spcTypeList = $this->Group->findSpcTypeList();

		$upGroupNameList = $this->Group->findGroupNameList($upGroupIdList);
		$adminGroupNameList = $this->Group->findGroupNameList($adminGroupIdList);
		$idpGroupName = $this->Group->findGroupNameList($idpGroupId);
		$iconList = array();
		foreach ($resultList as $spcid => $result) {
			// isMemberOf属性を利用するSPコネクタを識別する
			// このSPコネクタのサービスは画面にグループ一覧を表示する
			if (isset($ismemberofSpcIdList[$spcid])) {
				$resultList[$spcid]['ismemberof'] = true;
			} else {
				$resultList[$spcid]['ismemberof'] = false;
			}

			$serviceName = '';
			$serviceUrl = '';
			$wkSpcId = '';
			$leadNonMapUrl = '';
			$icon = '';
			foreach ($result as $groupid => $service) {
				if (empty($icon)) {
					if (isset($service[3])) {
						$icon = $service[3];
					}
				}

				if ($groupid === 'unusable' || $groupid === 'private_service') {
					continue;
				}
				$newService = array();
				$newService['name'] = $service[0];
				$newService['url'] = $service[1];
				$newService['spcid'] = $service[2];
				$newService['groupid'] = $groupid;
				$newService['group_name'] = $this->_getGroupName($groupid, $idpGroupName,
								$memberGroupNameList, $upGroupNameList, $adminGroupNameList);
				if (!isset($iconList[$groupid])) {
					$iconList[$groupid] = $this->Group->getIcon($groupid);
				}
				$newService['group_icon'] = $iconList[$groupid];
				$wkSpcId = $service[2];
				$resultList[$spcid][$groupid] = $newService;
				if (empty($serviceName)) {
					$serviceName = $service[0];
					$serviceOrgUrl = $service['orgUrl'];
				}
				if (empty($leadNonMapUrl)) {
					if (isset($service['lead_non_map_url'])) {
						$leadNonMapUrl = $service['lead_non_map_url'];
					}
				}
			}
			$resultList[$spcid]['sp_type'] = $this->_getSpType($wkSpcId, $spcTypeList);

			// サービス名とサービスのURL(各グループでも同じURLの場合)を設定
			$resultList[$spcid]['serviceName'] = $serviceName;
			if ($resultList[$spcid]['ismemberof']
					&& $resultList[$spcid]['sp_type'] != 3 && $resultList[$spcid]['sp_type'] != 1) {
				// グループごとにURLが異なるので、サービス名にはリンクを貼らない（ハイブリッド型SPCは除く）
				$resultList[$spcid]['serviceUrl'] = '';
			} else {
				// isMemberOfを使わない⇒全グループでサービスのURLは同じなので、リンクを張る
				$resultList[$spcid]['serviceUrl'] = $serviceOrgUrl;
			}
			$resultList[$spcid]['lead_non_map_url'] = $leadNonMapUrl;
			$resultList[$spcid]['spc_icon'] = $icon;
			$resultList[$spcid]['spc_icon_logo'] = 0;
			if (empty($icon)) {
				$resultList[$spcid]['spc_icon_logo'] = $this->Group->getIconLogo($spcid);
			}
		}
		//		$this->set('resultList', $resultList);

		// 手動の場合、保存した並び順にする
		if ($sortType === "manual") {
			$sortList = $this->UserSpSort->findSideSpIdList($this->getAccountID(), $idpGroupId);
			//debug($sortList);
		}

		$sideResultList = array();
		$settledIdList = array();
		// 既に順番を設定している場合
		if (!empty($sortList)) {
			if (!empty($resultList)) {
				foreach ($sortList as $spcid => $sortNo) {
					if (isset($resultList[$spcid])) {
						$sideResultList[$spcid] = $resultList[$spcid];
						$settledIdList[$spcid] = $spcid;
					}
				}

				// ソート設定されていないサービスを追加する
				foreach($resultList as $spcid => $result) {
					if (isset($settledIdList[$spcid])) {
						continue;
					}
					$sideResultList[$spcid] = $result;
				}
			}
		// 順番を設定していない場合
		} else {
			foreach ($resultList as $spcid => $result) {
				$sideResultList[$spcid] = $result;
			}
		}
		//debug($resultList);
		//debug($sideResultList);
		//$this->set('sideResultList', $sideResultList);

// debug($sideResultList);
// exit;
		
//debug($sideResultList);
		$useSpcIdList = array();
		foreach ($sideResultList as $scpid => $result) {
			array_push($useSpcIdList, $scpid);
		}
//debug($useSpcIdList);
		// 同意情報を反映、同意していないグループは非表示に
		// isMemberOfを利用するサービスでグループが０なら、サービスも表示しない
		// ユーザの同意情報を取得
		$userAgreeList = $this->TakerAgreement->servData($this->getAccountID(), $this->getLanguage(), $useSpcIdList);
//debug($userAgreeList);
		// SPコネクタの同意情報を反映
		$provideList = $this->ProvideAttribute->find('all',
				array('conditions' => array('group_id' => $useSpcIdList)));
		$mygroupIds = $this->Mygroup->findMyGroupIdList($this->getAccountID());
		foreach($mygroupIds as $id) {
			$userAgreeList = $this->TakerAgreement->reflectSPCAgreement(
					$userAgreeList, $useSpcIdList, $id, $provideList);
		}

		// 同意していないグループを削除
		$idpGroupIdList = $this->AutoGroup->findIdPGroupIdList();
		if (!empty($userAgreeList) && !empty($userAgreeList['Agreement'])) {
			//  機関グループは同意を利用しない
			foreach ($sideResultList as $spcid => $result) {
				foreach ($result as $groupid  => $groupinfo) {
					if (!is_numeric($groupid)) {
						continue;
					}
					// 機関グループを除外
					if (isset($idpGroupIdList[$groupid])) {
						continue;
					}

					foreach ($userAgreeList['Agreement'] as $userAgree) {
						if ($userAgree['group_id'] != $spcid) {
							continue;
						}
						if ($userAgree['nothing_agree_item']) {
							// 提供属性を持たないＳＰコネクタ（ＣＧ用）
							
						} else if (isset($userAgree['top_group_select_id_list'][$groupid])
							&& $userAgree['top_group_select_id_list'][$groupid]) {
							// 同意済み
						} else {
							// 未同意
							unset($sideResultList[$spcid][$groupid]);
							// 全てのグループに対して未同意ならサービスを削除する
							if (count($sideResultList[$spcid]) <= 3) {
								$sideResultList[$spcid]['unusable'] = true;
							}
						}
						break;
					}
				}
			}
		}
//debug($userAgreeList);

		
		// CGフィードバック
		// サービスがIdPグループを接続しているかを設定(IdPグループと接続⇒SPと契約済み)
		// 自大学のIdPグループID : $idpGroupId
		foreach ($sideResultList as $spcid => $sideResult) {
			$connectIdpGroup = false;
			foreach ($sideResult as $groupid => $group) {
				if ($groupid == $idpGroupId) {
					$connectIdpGroup = true;
					break;
				}
			}
			$sideResultList[$spcid]['connectIdpGroup'] = $connectIdpGroup;
		}

		// HPCI型SPコネクタ
		//  * IdPグループが存在していなかったら(接続＋同意がない)、サービスを表示しない
		// AP(mAP対応),AP/IdP AND AP(read/write), IdP AND AP(HPCI型)
		//  * 機関グループは表示しない
		foreach ($sideResultList as $spcid => $sideResult) {
			if ($sideResultList[$spcid]['sp_type'] == 2 ||
				$sideResultList[$spcid]['sp_type'] == 5 ||
				$sideResultList[$spcid]['sp_type'] == 4) {
				foreach ($sideResult as $groupid => $group) {
					// mAP対応型
					if (isset($idpGroupIdList[$groupid])) {
						unset($sideResultList[$spcid][$groupid]);
					}
				}
			}
			if ($sideResultList[$spcid]['sp_type'] == 5
				&& !$sideResultList[$spcid]['connectIdpGroup']) {
					$sideResultList[$spcid]['unusable'] = true;
			}
		}

		// 活動中でなグループは表示しない
		// mAP非対応型：ismemberofは使わない
		// mAP対応型：グループ数０のサービスは表示しない
		// サービス名を設定する
		// IdPタイプ（mAP非対応）:機関グループと接続していないと表示しない
		$notActiveGroupIdList = $this->Group->findNotActiveIdList();
		foreach ($sideResultList as $spcid => $sideResult) {
			if (isset($notActiveGroupIdList[$spcid])) {
				$sideResultList[$spcid]['unusable'] = true;
				continue;
			}
			// 活動中でなグループは表示しない
			foreach ($sideResult as $groupid => $res) {
				if (isset($notActiveGroupIdList[$groupid])) {
					unset($sideResultList[$spcid][$groupid]);
				}
			}

			// サービス名の文字数を設定
			$serviceName = $sideResult['serviceName'];
			$len = mb_strwidth($serviceName);
			if ($len <= 18 && $len >= 14) {
				$sideResultList[$spcid]['serviceSize'] = 'middle_service_name';
			} else if ($len > 18) {
				$sideResultList[$spcid]['serviceSize'] = 'small_service_name';
			} else {
				$sideResultList[$spcid]['serviceSize'] = 'big_service_name';
			}

			// IdPタイプ(mAP非対応型)
			if ($sideResultList[$spcid]['sp_type'] == 1) {
				$sideResultList[$spcid]['ismemberof'] = false;

				$IdpGroupOK = false;
				foreach ($sideResult as $groupid => $res) {
					if ($idpGroupId == $groupid) {
						$IdpGroupOK = true;
						break;
					}
				}
				if (!$IdpGroupOK) {
					// IdPタイプ（mAP非対応）に機関グループがないので非表示
					$sideResultList[$spcid]['unusable'] = true;
				}
			}
		}

		// 表示するグループの数を設定
		foreach ($sideResultList as $spcid => $sideResult) {
			$count = 0;
			foreach ($sideResult as $key => $result) {
				if (is_numeric($key)) {
					$count++;
				}
			}
			$sideResultList[$spcid]['group_count'] = $count;

			// mAP対応型
			if ($sideResultList[$spcid]['sp_type'] == 2 
					|| $sideResultList[$spcid]['sp_type'] == 5
					|| $sideResultList[$spcid]['sp_type'] == 4) {
				if ($sideResultList[$spcid]['group_count'] == 0) {
					$sideResultList[$spcid]['unusable'] = true;
				}
			}
		}

		// get introduction
		$introductionList = $this->Group->getIntroductions($useSpcIdList, $this->getLanguage());
		foreach ($introductionList as $spcid => $introduction) {
			if (!empty($sideResultList[$spcid])) {
				$sideResultList[$spcid]['introduction'] = $introduction;
			}
		}

		// get link_for_administrator
		if ($this->Mygroup->isAdmin($this->getAccountID(), $idpGroupId)) {	// IdPグループの管理者だったら
			$splinks = $this->SpGroup->findLinkForIdpAdministrator();
			foreach ($splinks as $spcid => $splink) {
				if (!isset($sideResultList[$spcid])) {
					continue;
				}
				$sideResultList[$spcid]['link_for_idp_administrator'] = $splink;
			}
		}

		$allCandidateTagNames = array_values($this->Tag->getCandidateTagNames($this->getAccountID()));
		$candidateTagNames = array();

		// get user tag
		$userTags = $this->Tag->getUserTags($this->getAccountID());
		foreach ($userTags as $result) {
			$tag = $result['Tag'];
			if (!isset($sideResultList[$tag['group_id']])) {
				unset($allCandidateTagNames[array_search($tag['name'], $allCandidateTagNames)]);
				continue;
			}
			if ($tag['public'] == 1) {
				$sideResultList[$tag['group_id']]['public_tag'][] = $tag['name'];
			} else {
				$sideResultList[$tag['group_id']]['tag'][] = $tag['name'];
			}
			$key = array_search($tag['name'], $allCandidateTagNames);
			if (!isset($sideResultList[$tag['group_id']]['unusable'])
			    && isset($key) && $key !== false) {
				$candidateTagNames[] = $allCandidateTagNames[$key];
				unset($allCandidateTagNames[$key]);
			}
		}
		$candidateTagNamesUnusableOnly = $allCandidateTagNames;

		// search
		$match_spcid_list = $this->searchService();
		if ($match_spcid_list !== false) {
			$matchResultList = array();
			foreach($match_spcid_list as $spcid) {
				if (!isset($sideResultList[$spcid])) {
					continue;
				}
				$matchResultList[$spcid] = $sideResultList[$spcid];
			}
			$sideResultList = $matchResultList;
		}

		// generate unusable list
		$sideResultUnusableList = array();
		foreach($sideResultList as $spcid => $result) {
			if (isset($result['unusable'])) {
				$sideResultUnusableList[$spcid] = $result;
				unset($sideResultList[$spcid]);
			}
		}
		$this->set('sideResultUnusableList', $sideResultUnusableList);

		$this->set('candidateTagNames', $candidateTagNames);
		$this->set('candidateTagNamesUnusableOnly', $candidateTagNamesUnusableOnly);

		$this->set('show_unusable', $this->Account->field('show_unusable',
						array('id' => $this->getAccountID())));

		$this->set('sideResultList', $sideResultList);

		return true;
	}

	private function _setPersonalPrivateService($resultList) {
		$privateSpcIdList = $this->Group->find('list', array(
			'fields' => array('Group.id', 'Group.id'),
			'joins' => array(array(
				'table' => 'mygroups', 'alias' => 'Mygroup',
				'conditions' => array('Group.id = Mygroup.groupid'))),
			'conditions' => array(
				'Group.sp' => 2,
				'Mygroup.account_id' => $this->getAccountID())));

		if (empty($privateSpcIdList)) {
			return $resultList;
		}

		$privateServiceList = $this->GroupSphost->find('all', array(
			'fields' => array('group_id', 'service_name', 'lead_url', 'AddGroup.bottom_id'),
			'joins' => array(array(
				'type' => 'LEFT', 'table' => 'add_groups', 'alias' => 'AddGroup',
				'conditions' => array('GroupSphost.group_id = AddGroup.top_id'))),
			'conditions' => array(
				'group_id' => $privateSpcIdList)));

		if (empty($privateServiceList)) {
			return $resultList;
		}

		$groupIdList = $this->Mygroup->findMyGroupIdList($this->getAccountID());
		foreach ($privateServiceList as $ps) {
			$groupId = $ps['AddGroup']['bottom_id'];
			if (isset($groupId) && in_array($groupId, $groupIdList)) {
				// グループもしくは機関グループ用プライベートサービスは設定済のためスキップする。
				continue;
			}

			$resultList[$ps['GroupSphost']['group_id']] = array(
				array(
					$ps['GroupSphost']['service_name'],
					$ps['GroupSphost']['lead_url'],
					$ps['GroupSphost']['group_id'],
					$this->Group->getIcon($ps['GroupSphost']['group_id']),
					'orgUrl' => $ps['GroupSphost']['lead_url']),
				'private_service' => true);
		}

		return $resultList;
	}

	private function _setPrivateService($resultList) {
		// 個人用として登録したPrivateServiceを取得
		$resultList = $this->_setPersonalPrivateService($resultList);

		// グループもしくは機関グループ用として登録したPrivateServiceの管理者設定
		$privateServiceIdList = $this->Mygroup->find('list', array(
			'fields' => array('groupid', 'groupid'),
			'joins' => array(array(
				'table' => 'groups', 'alias' => 'Group',
				'conditions' => array('groupid = Group.id'))),
			'conditions' => array(
				'account_id' => $this->getAccountID(),
				'admin' => 1,
				'Group.sp' => 2)));

		if (empty($privateServiceIdList)) {
			return $resultList;
		}

		$privateServiceList = $this->GroupSphost->find('all', array(
			'fields' =>array('group_id', 'service_name', 'lead_url'),
			'conditions' => array(
				'group_id' => $privateServiceIdList)));

		foreach ($privateServiceList as $ps) {
			$ps_id = $ps['GroupSphost']['group_id'];
			if (isset($resultList[$ps_id])) {
				$resultList[$ps_id]['private_service'] = true;
			} else {
				$resultList[$ps_id] = array(
					array(
						$ps['GroupSphost']['service_name'],
						$ps['GroupSphost']['lead_url'],
						$ps['GroupSphost']['group_id'],
						$this->Group->getIcon($ps['GroupSphost']['group_id']),
						'orgUrl' => $ps['GroupSphost']['lead_url']),
					'private_service' => true);
			}
		}

		return $resultList;
	}

	/**
	 * 検索窓に入力した文字列でサービスを検索する。
	 * タグ名は、完全一致で検索する。
	 * サービス名は、部分一致で検索する。
	 *
	 * @return array|false 検索条件にマッチしたSPコネクタIDのarrayを返す。
	 *                     検索窓が空の場合は、falseを返す。
	 */
	private function searchService() {
		$encoded_search_str = $this->params['url']['search'];
		if (empty($encoded_search_str)) {
			return false;
		}
		$search_type = $this->params['url']['search_type'];
		$search_str = rawurldecode($encoded_search_str);

		if (is_null($search_type) || $search_type != 'tag_only') {
			$service_match = $this->GroupSphost->find('list', array(
					'fields' => array('group_id'),
					'conditions' => array(
						'OR' => array(
							'service_name LIKE' => "%".$search_str."%",
							'service_name_en LIKE' => "%".$search_str."%",
						)
					)
				)
			);
		} else {
			$service_match = array();
		}

		if (is_null($search_type) || $search_type != 'service_only') {
			$tag_match = $this->Tag->find('list', array(
					'fields' => array('group_id'),
					'conditions' => array(
						'name' => $search_str,
						'OR' => array(
							'account_id' => $this->getAccountID(),
							'public' => 1
						)
					)
				)
			);
		} else {
			$tag_match = array();
		}

		return array_unique(array_merge($service_match, $tag_match));
	}

	/**
	 * mygroupのfavorite順にグループIDを並べる。
	 * //mygroupに無かったグループIDは後ろに付ける。
	 * $resultList[SPコネクタID][グループID]
	 */
	private function _sortGroup($resultList, $accountId) {
		$favariteGroupIdList = $this->Mygroup->findGroupIdSortFavarite($accountId);
		$sortedResultList = array();
		foreach ($resultList as $spcid => $groupList) {
			$sortedGroupList = array();
			$groupCount = 0;
			foreach ($favariteGroupIdList as $groupid) {
				if (isset($groupList[$groupid])) {
					$sortedGroupList[$groupid] = $groupList[$groupid];
					$groupCount ++;
					if ($groupCount == count($groupList)) {
						break;
					}
				}
			}
			$sortedResultList[$spcid] = $sortedGroupList;
			// 利用できないサービスを表示するために、mygroupにないグループを削除しない
			foreach ($groupList as $groupid => $res) {
				if (isset($sortedResultList[$spcid][$groupid])) {
					continue;
				}
				$sortedResultList[$spcid][$groupid] = $res;
			}
		}

		return $sortedResultList;
	}

	private function _getGroupName($groupid, $idpGroupName, $memberGroupNameList, $upGroupNameList, $adminGroupNameList) {
		if (isset($memberGroupNameList[$groupid])) {
			return $memberGroupNameList[$groupid];
		}
		if (isset($upGroupNameList[$groupid])) {
			return $upGroupNameList[$groupid];
		}
		if (isset($adminGroupNameList[$groupid])) {
			return $adminGroupNameList[$groupid];
		}
		if (isset($idpGroupName[$groupid])) {
			return $idpGroupName[$groupid];
		}
		return "";
	}

	private function _getSpType($spcid, $spcTypeList) {
		if (isset($spcTypeList[ $spcid ])) {
			return $spcTypeList[$spcid];
		}
		return '';
	}

	/**
	 * IdPグループが利用できるサービス($resultList)以外にidpグループを入れるが、
	 * ログインユーザのMYグループや上位グループもそのサービスが利用できるのなら、
	 * サービスのグループとして追加する。
	 * $resultList[SPコネクタID][グループID]
	 * @param $ismemberofSpcIdList isMemberOf属性を利用するSPコネクタIDの一覧
	 * @param $hybrid ハイブリッド型SPCかどうか
	 * @param $spcidByhybrid ハイブリッド型SPCのID
	 */
	private function _setOtherService($groupIdList, $resultList,
			$ismemberofSpcIdList = array(), $hybrid = false, $spcidByhybrid = false) {
		$iconList = $this->SpGroup->findIconList();
		// グループIDごとのSPCID一覧を求める
		if (!$hybrid) {
			$groupIdSpcIdList = $this->getGroupIdSpcIdList($groupIdList);
		} else {
			$groupIdSpcIdList = array();
			foreach ($groupIdList as $groupid) {
				$groupIdSpcIdList[ $groupid ] = array();
				$groupIdSpcIdList[ $groupid ] = $spcidByhybrid;
			}
		}
//debug($groupIdSpcIdList);
//debug($ismemberofSpcIdList);
		// SPホスト名一覧を事前に取得
		$spHostList = $this->SpHost->getSpHostList();

		foreach ($groupIdList as $groupid) {
			if (!$hybrid) {
				if ($this->AutoGroup->isIdPGroup($groupid)) {
					continue;
				}
			}
//debug("groupid=".$groupid);
			if (!$hybrid) {
				// isMemberOf属性が不要なサービスは除外する(CiNiiなど)
				$ismemberFlg = false;
				if (!isset($groupIdSpcIdList[$groupid])) {
					continue;
				}

// debug($ismemberofSpcIdList);
				$privateServiceIdList = $this->Group->find('list', array(
					'fields' => array('id'),
					'conditions' => array(
						'id' => array_values($groupIdSpcIdList[$groupid]),
						'sp' => 2)));
				$private = false;
				$targetSpcIdList = $groupIdSpcIdList[$groupid];
				foreach ($targetSpcIdList as $spcid) {
					if (isset($ismemberofSpcIdList[$spcid])) {
						$ismemberFlg = true;
						continue;
					}
					if (in_array($spcid, $privateServiceIdList)) {
						$private = true;
						break;
					}
				}
				if (!$ismemberFlg && $private === false) {
					continue;
				} 
			}

			// IdPグループ以外のサービスを取得
			$otherServiceLinkList = $this->GroupSphost->getServiceLink(
					$groupid, $this->getLanguage(), true, $iconList, $groupIdSpcIdList, $spHostList);

			if (empty($otherServiceLinkList)) {
				continue;
			}

			// サービスの並び替えは不要
//			$otherServiceLinkList = $this->SpcSortor->updateSortForDetail(
//					$groupid, $otherServiceLinkList, $this->getAccountID(), $this->getIdentityProvider());
			foreach ($otherServiceLinkList as $spcid => $serviceLink) {
// debug($serviceLink);
// debug($hybrid);
//2015.5.29 全てのタイプを許可して、side()の最後の方でSPの除外処理を入れることにした
// 				if (!isset($resultList[$spcid])) {
// 					// ハイブリッド型サービスはresultListになくても追加する(画面に追加する)
// 					// APタイプ, AP/IdP AND APタイプ（read/write）も追加したい★
// 					if (!$hybrid) {
// 						continue;
// 					}
// 				}
//debug($groupid);
//debug($serviceLink);
				$resultList[$spcid][$groupid] = $serviceLink;
				if(isset($resultList[$spcid]['unusable'])) {
					unset($resultList[$spcid]['unusable']);
				}
			}
//debug($otherServiceLinkList);
		}
//debug($resultList);
		return $resultList;
	}

	/**
	 * グループIDごとのSPCID一覧を求める
	 */
	private function getGroupIdSpcIdList($groupIdList) {
		$addGroupList = $this->AddGroup->find('all', array(
					'fields' => array('bottom_id', 'top_id'),
					'conditions' => array('bottom_id' => $groupIdList, 'top' => 1, 'Group.active' => 0),
					'joins' => array(
						array('type' => 'INNER', 'table' => 'groups', 'alias' => 'Group', 'conditions' => array('top_id = Group.id'))
				)));
		$groupIdSpcIdList = array();
		foreach($addGroupList as $addgroup) {
			if (empty($groupIdSpcIdList[ $addgroup['AddGroup']['bottom_id'] ])) {
				$groupIdSpcIdList[ $addgroup['AddGroup']['bottom_id'] ] = array();
			}
			array_push($groupIdSpcIdList[ $addgroup['AddGroup']['bottom_id'] ],
				$addgroup['AddGroup']['top_id']);
		}
		return $groupIdSpcIdList;
	}

	/**
	 * サービス並び順をモーダルに設定します。
	 */
	private function servSortType($groupid) {
		$spcSortType = $this->SpcSort->getSortType($this->getAccountID(), $groupid);
		$this->set('spcSortType', $spcSortType);
		if (empty($spcSortType)) {
			return "";
		}
		return $spcSortType;
	}

	function side_sample() {
	
	}

	function show_unusable_change() {
		$this->autoRender = false;
		if($this->RequestHandler->isAjax()) {
			$this->Account->id = $this->getAccountID();
			$this->Account->saveField('show_unusable',
				($this->params['form']['show_unusable'] == 'true') ? 1 : 0);
		}
	}

	/**
         * get OrganizationDisplayName from Organization
	 */
	private function getOrganizationDisplayName() {
		$accountLanguage = $this->Session->read('accountLanguage');
		$displaynames = $this->Organization->find('first', array(
			'fields' => array('display_name', 'display_name_en'),
			'conditions' => array('entityid' => $this->getEntityID())));

		if ($accountLanguage == 'ja' && isset($displaynames['Organization']['display_name']) && $displaynames['Organization']['display_name'] != "") {
			$organizationDisplayName = $displaynames['Organization']['display_name'];
		} else if (isset($displaynames['Organization']['display_name_en']) && $displaynames['Organization']['display_name_en'] != "") {
			$organizationDisplayName = $displaynames['Organization']['display_name_en'];
		} else {
			$organizationDisplayName = "";
		}

		return $organizationDisplayName;
	}

}
?>
