<?php
/**
 * UUID
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2017, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class UUID {
	/*
	 * RFC4122 Appendix C - Some Name Space IDs
	 */
	/* Name string is a fully-qualified domain name */
	const nsDNS  = '6ba7b810-9dad-11d1-80b4-00c04fd430c8';

	/* Name string is a fully-qualified domain name */
	const nsURL  = '6ba7b811-9dad-11d1-80b4-00c04fd430c8';

	/* Name string is an ISO OID */
	const nsOID  = '6ba7b812-9dad-11d1-80b4-00c04fd430c8';

	/* Name string is an X.500 DN (in DER or a text output format) */
	const nsX500 = '6ba7b814-9dad-11d1-80b4-00c04fd430c8';

	/*
	 * Generate UUID version 1
	 */
	public function generate_v1() {
		return self::make(UUID_MAKE_V1);
	}

	/*
	 * Generate UUID version 3
	 */
	public function generate_v3($ns_uuid, $name) {
		return self::make_with_ns(UUID_MAKE_V3, $ns_uuid, $name);
	}

	/*
	 * Generate UUID version 4
	 */
	public function generate_v4() {
		return self::make(UUID_MAKE_V4);
	}

	/*
	 * Generate UUID version 5
	 */
	public function generate_v5($ns_uuid, $name) {
		return self::make_with_ns(UUID_MAKE_V5, $ns_uuid, $name);
	}

	/*
	 * Generate UUID for v1 and v4
	 */
	private function make($version) {
		if (!function_exists('uuid_create')) {
			return false;
		}

		$context = $uuid = null;

		uuid_create($context);
		uuid_make($context, $version);
		uuid_export($context, UUID_FMT_STR, $uuid);

		return trim($uuid);
	}

	/*
	 * Generate UUID for v3 and v5
	 */
	private function make_with_ns($version, $ns_uuid, $name) {
		if (!function_exists('uuid_create')) {
			return false;
		}

		if (!self::is_valid($ns_uuid)) {
			return false;
		}

		if (strlen($name) < 0) {
			return false;
		}

		$context = $namespace = $uuid = null;

		uuid_create($context);
		uuid_create($namespace);

		uuid_import($namespace, UUID_FMT_STR, $ns_uuid);
		uuid_make($context, $version, $namespace, $name);
		uuid_export($context, UUID_FMT_STR, $uuid);

		return trim($uuid);
	}

	/*
	 * validate UUID format
	 */
	public function is_valid($uuid) {
		$pat_uuid = "/\A[0-9a-z]{8}-[0-9a-z]{4}-[1-5][0-9a-z]{3}-[0-9a-z]{4}-[0-9a-z]{12}\z/i";
		return preg_match($pat_uuid, $uuid) === 1;
	}
}
?>
