<?php
/**
 * Account Model
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class Account extends AppModel{
	var $name = 'Account';
	var $validate = array(
        'name' => array(
    		 'maxLength' => array(
                'rule' => array('maxLength', 50),
    			'message' => 'validation.maxlength',
    			'last' => true
	),
	),
        'name_en' => array(
    		 'maxLength' => array(
                'rule' => array('maxLength', 50),
    			'message' => 'validation.maxlength',
    			'last' => true
	),
    		'alphanumeric' => array(
            	'rule' => array('custom', '/^[a-zA-Z0-9 !-\/:-@¥[-`{-~]*$/i'),
				'message' => 'validation.alphanumeric.symbols',
          		'last' => true
	),
	),
        'name_require' => array(
    		'require' => array(
	          	'rule' => array('requireName'),
	        	'message' => 'validation.enter',
	          	'last' => true
	),
	),

        'mail' => array(
        	'minLength' => array(
            	'rule' => array('minLength', 1),
        		'message' => 'validation.minlength',
        		'last' => true
	),
        	'email' => array(
            	'rule' => array('email'),
        		'message' => 'validation.mailaddress.incorrect.format',
        		'last' => true
	),
        	'maxLength' => array(
            	'rule' => array('maxLength', 100),
        		'message' => 'validation.maxlength',
    			'last' => true
	),

	),
        'university' => array(
    		 'maxLength' => array(
                'rule' => array('maxLength', 100),
    			'message' => 'validation.maxlength',
    			'last' => true
	),
	),
        'university_en' => array(
    		 'maxLength' => array(
                'rule' => array('maxLength', 100),
    			'message' => 'validation.maxlength',
    			'last' => true
	),
	),
        'introduction' => array(
    		 'maxLength' => array(
                'rule' => array('maxLength', 200),
        		'message' => 'validation.maxlength',
    			'last' => true
	),
	),
        'introduction_en' => array(
    		 'maxLength' => array(
                'rule' => array('maxLength', 200),
        		'message' => 'validation.maxlength',
    			'last' => true
	),
	),

        'agreement' => array(
			'rule' => array('requireAgreement'),
        	'message' => 'validation.selected',
	),
	);

	/**
	 * 同意を検証する。
	 */
	function requireAgreement() {
		if (empty($this->data['Account']['agreement'])) {
			return false;
		}
		$this->data['Account']['agreement'] = 'yes';
		return true;
	}

	function requireName() {
		if (!$this->data['Account']['name'] && !$this->data['Account']['name_en']) {
			return false;
		}
		return true;
	}

	function requireUniversity() {
		if (!$this->data['Account']['university'] && !$this->data['Account']['university_en']) {
			return false;
		}
		return true;
	}

	/**
	 * 仮アカウントを作成する
	 */
	public function createTemporaryAccount($groupid, $mail, $invitedAccount, $is_admin_input) {
		$account = array(
			'mail' => $mail,
			'university' => 'Inviting',
			'arrival' => 0,
			'language' => 'ja',
			'name' => 'Secret',
			'temp_groupid' => $groupid,
			'is_temp' => 1,
		);
		if (!empty($invitedAccount)) {
			foreach (array('name', 'name_en', 'language') as $key) {
				$account[$key] = $invitedAccount[$key];
			}
			if ($is_admin_input) {
				$account['is_temp'] = 3;
			} else {
				$account['is_temp'] = 2;
			}
		}

		$this->create();
		$result = $this->save($account, false);
		if (!$result) {
			return false;
		}
		$id = $this->getLastInsertID();
		return $id;
	}

	/**
	 * 仮アカウントを削除します。
	 */
	public function deleteTemporaryAccount($accountId, $groupId) {
		$account = $this->find('first', array('conditions' => array('id' => $accountId)));
		if ($account['Account']['temp_groupid'] != $groupId) {
			return false;
		}
		if ($account['Account']['is_temp'] == 0) {
			return false;
		}
		$result = $this->delete($accountId);
		return $result;
	}

	/**
	 * 自動アカウントを作成する(CG用)
	 * @param $name 名前
	 * @param $university 大学名
	 */
	public function createAutoyAccount($name = "", $mail = "", $university = "") {
		$account = array(
				'name' => $name,
				'mail' => $mail,
				'university' => $university,
				'arrival' => 0,
				'language' => 'ja'
		);

		$this->create();
		$result = $this->save($account, false);
		if (!$result) {
			return false;
		}
		$id = $this->getLastInsertID();
		return $id;
	}

	/**
	 * ゲストアカウントを作成します。(CG用)
	 */
	public function createGuestAccount($university) {
		$account = array(
				'name' => 'Guest',
				'university' => $university,
				'arrival' => 0,
				'language' => 'ja'
		);
		$this->create();
		$result = $this->save($account, false);
		if (!$result) {
			return false;
		}
		$id = $this->getLastInsertID();
		return $id;
	}

	/**
	 * アカウント名が設定されているか確認する。
	 */
	public function hasAccountName($accountId) {
		$account = $this->find('first', array('conditions' => array('id' => $accountId)));
		if (!empty($account['Account']['name'])) {
			return true;
		}
		if (!empty($account['Account']['name_en'])) {
			return true;
		}
		return false;
	}

	/**
	 * メールアドレスが設定されているか確認する。
	 */
	public function hasMailaddress($accountId) {
		$account = $this->find('first', array('conditions' => array('id' => $accountId)));
		if (!empty($account['Account']['mail'])) {
			return true;
		}
		return false;
	}
}
?>