<?php
/**
 * AutoGroup Model
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class AutoGroup extends AppModel{
    var $name = 'AutoGroup';
    var $useTable = 'auto_groups';

	/**
	 * 自動グループIDかを検証する
	 * @param $groupid
	 * @param $accountid
	 * @param $entityid
	 */
    function isAutoGroup($groupid, $accountid, $entityid, $o, $ou) {
    	if (empty($groupid)) {
    		return false;
    	}
    	$o = $this->toCaseIgnoreMatch($o);
    	$ou = $this->toCaseIgnoreMatch($ou);
		if (empty($o) && empty($ou)) {
			return false;
		}

		$count = 0;
    	if (!empty($o)) {
			$count = $this->find('count', array('conditions' => array(
				'group_id in (select id from groups where id = ? and active = 0) ' => $groupid,
				'entityid' => $entityid,
				'o' => $o)));
    	}
    	if ($count == 0 && !empty($ou)) {
			$count = $this->find('count', array('conditions' => array(
				'group_id in (select id from groups where id = ? and active = 0) ' => $groupid,
				'entityid' => $entityid,
				'ou' => $ou)));
    	}

		if ($count == 0) {
			return false;
		}
		return true;
    }

	/**
	 * oグループ、ouグループ、idpグループかを判定する
	 * @param $groupid
	 * @return o / ou / false
	 */
    function getGroupAttribute($groupid) {
    	if (empty($groupid)) {
    		return false;
    	}

		$entity = $this->find('first', array('conditions' => array('group_id' => $groupid)));
		if (empty($entity)) {
			return false;
		}
		if ($entity['AutoGroup']['o']) {
			return 'o';
		} else if ($entity['AutoGroup']['ou']) {
			return 'ou';
		} else if (!empty($entity['AutoGroup']['idp']) && $entity['AutoGroup']['idp'] === 'true') {
			return 'idp';
		}

		return false;
    }

	/**
	 * caseIgnoreMatch(LDAP)の定義に当てはまる値を返します。
	 * 英大小文字の区別なし(MySQLのSELECTでは区別しないので何もしない)
	 * 先頭、末尾の空白を無視
	 * 連続する空白は、単一の空白とみなす
	 */
	function toCaseIgnoreMatch($value) {
		if (empty($value)) {
			return $value;
		}

		$newValue = $value;
		$newValue = strtolower($newValue);
		$newValue = preg_replace('/　/', ' ', $newValue);
		$newValue = preg_replace('/\s\s+/', ' ', $newValue);
		$newValue = trim($newValue);
		return $newValue;
	}

	/**
	 * IdP機関グループを検索します。
	 * @param $entityid IdPのエンティティID
	 */
	public function findIdPGroup($entityid) {
		$entity = $this->find('first', array(
			'conditions' => array('entityid' => $entityid, 'idp' => 'true')));
		if (empty($entity)) {
			return false;
		}
		return $entity;
	}

	/**
	 * IdP機関グループ一覧を検索します。
	 * @param $entityid IdPのエンティティID
	 */
	public function findIdPGroupList() {
		$entity = $this->find('all', array(
				'conditions' => array('idp' => 'true', 'status' => 'active')));
		if (empty($entity)) {
			return false;
		}
		return $entity;
	}

	/**
	 * IdP機関グループ一覧を検索します。
	 * @param $entityid IdPのエンティティID
	 */
	public function findIdPGroupIdList() {
		$list = $this->find('list', array(
				'fields' => array('group_id', 'group_id'),
				'conditions' => array('idp' => 'true')));
		if (empty($list)) {
			return false;
		}
		return $list;
	}

	/**
	 * グループIDを元にIdPグループのエンティティIDを取得します。
	 * @param unknown $groupId
	 */
	public function findEntityIdForIdpGroup($groupId) {
		$entity = $this->find('first', array(
				'conditions' => array('group_id' => $groupId, 'idp' => 'true')));
		if (empty($entity)) {
			return false;
		}
		if (empty($entity['AutoGroup']['entityid'])) {
			return false;
		}
		return $entity['AutoGroup']['entityid'];
	}

	/**
	 * 機関グループか判定します。
	 */
	public function isIdPGroup($groupId) {
		$entity = $this->find('first', array(
				'conditions' => array('group_id' => $groupId, 'idp' => 'true')));
		if (empty($entity)) {
			return false;
		} else {
			return true;
		}
	}

	/**
	 * グループIDのリストから機関グループを除いたグループIDのリストを返す。
	 * @param $groupIdList
	 * @return $groupIdListから機関グループを除いたグループIDのリスト
	 */
	public function getGroupIdListWithOutIdPGroup($groupIdList) {
		foreach ($groupIdList as $key => $groupId) {
			if ($this->isIdPGroup($groupId)) {
				unset($groupIdList[$key]);
			}
		}
		return $groupIdList;
	}
}
?>
