<?php
/**
 * SpGroup Model
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Model', 'SpGroup');
App::import('Lib', 'uuid');

class AutoSpGroup extends SpGroup {
	var $name = 'AutoSpGroup';
	var $useTable = 'groups';

	/**
	 * 自動生成用のSPコネクタを作成します。
	 * @param $serviceName サービス名
	 * @param $entityid entityid
	 * @param $description SPの説明
	 */
	public function createAutoSpConnector($serviceName, $entityid, $description) {
		$group = array();
		if (!empty($serviceName)) {
			$group['AutoSpGroup']['name'] = $serviceName;
		} else {
			$group['AutoSpGroup']['name'] = $entityid;
		}
		$group['AutoSpGroup']['group_key'] = UUID::generate_v5(UUID::nsURL, $entityid);
		if (!$group['AutoSpGroup']['group_key']) {
			return false;
		}
		$group['AutoSpGroup']['sp'] = 1;
		$group['AutoSpGroup']['sp_type'] = 1;
		$group['AutoSpGroup']['public'] = 1; // 検索対象
		$group['AutoSpGroup']['inspectjoin'] = '1'; // 管理者の承認が必要
		$group['AutoSpGroup']['inspectquit'] = '0'; // 自由に退会
		$group['AutoSpGroup']['independence'] = '1'; // 独立させる
		$group['AutoSpGroup']['down_permission'] = '2'; // 許可する(管理者の承認を必要とします)
		$group['AutoSpGroup']['up_permission'] = '0'; // 許可しない
		$group['AutoSpGroup']['group_response'] = 1;
		$group['AutoSpGroup']['openmember'] = 0;
		$group['AutoSpGroup']['icon'] = 1;
		$group['AutoSpGroup']['spc_icon'] = 'book';
		if (!empty($description)) {
			$group['AutoSpGroup']['introduction'] = $description;
		}
		$this->create();
		$result = $this->save($group, false);
		if (!$result) {
			return false;
		}
		$groupId = $this->getLastInsertID();
		return $groupId;
	}

	/**
	 * SPコネクタの紹介文を更新します。（自動更新用）
	 */
	public function updateDescriptionGroupKeyForAuto($spcId, $entityid, $description) {
		if (empty($description)) {
			return;
		}
		$entity = $this->find('first', array('conditions' => array('id' => $spcId)));
		if (empty($entity)) {
			return;
		}

		// 紹介文が未設定の場合のみ、更新する
		if (!empty($entity['AutoSpGroup']['introduction'])) {
			return;
		}

		$entity['AutoSpGroup']['modified'] = null;
		$entity['AutoSpGroup']['introduction'] = $description;
		$this->save($entity, false);
	}
}
?>
