<?php
/**
 * Group Model
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */

App::import('Model', 'IconImage');
App::import('Model', 'AutoGroup');

define('_GROUP_IMAGE_ORIGINAL_DIRECTORY', MEDIA_TRANSFER. 'group'. DS .'original'. DS);
define('_GROUP_IMAGE_TEMPORARY_DIRECTORY', MEDIA_TRANSFER. 'group'. DS .'temp'. DS);
define('_GROUP_IMAGE_THUMBNAIL_DIRECTORY', MEDIA_TRANSFER. 'group'. DS .'thumbnails'. DS);

class Group extends AppModel{
	var $name = 'Group';
	var $actsAs = array(
		'Media.Transfer' => array(
			'transferDirectory' => _GROUP_IMAGE_ORIGINAL_DIRECTORY
		),
		'Media.Generator' => array(
			'guessExtension' => false,
			'overwrite' => true,
			'baseDirectory' => _GROUP_IMAGE_ORIGINAL_DIRECTORY,
			'filterDirectory' => _GROUP_IMAGE_THUMBNAIL_DIRECTORY,
		),
	);
	var $validate = array(
		'name' => array(
			'minlength' => array(
				'rule' => array('minLength', '1'),
				'required' => true,
				'message' => 'validation.enter',
			),
			'maxlength' => array(
				'rule' => array('maxLength', '50'),
				'message' => 'validation.maxlength',
				'last' => true
			),
			'duplication' => array(
				'rule' => array('duplicateGroupName'),
				'message' => 'validation.duplication.group',
			),
		),
		'group_key' => array(
			'minlength' => array(
				'rule' => array('minLength', '4'),
				'message' => 'validation.minlength',
				'last' => true
			),
			'maxlength' => array(
				'rule' => array('maxLength', '50'),
				'message' => 'validation.maxlength',
				'last' => true
			),
			'alphanumeric' => array(
				'rule' => array('custom', '/^[a-z\d._-]+$/i'),
				'message' => 'validation.alphanumeric',
				'last' => true
			),
			'duplication' => array(
				'rule' => array('duplicateGroupKey'),
				'message' => 'validation.duplication.group.key',
			),
		),

		'introduction' => array(
			'minlength' => array(
				'rule' => array('minLength', '1'),
				'required' => true,
				'message' => 'validation.enter',
			),
			'maxlength' => array(
				'rule' => array('maxLength', '400'),
				'message' => 'validation.maxlength',
				'last' => true
			),
		),
		'file' => array(
			'resource'   => array('rule' => 'checkResource'),
			'access'     => array('rule' => 'checkAccess'),
			'size'       => array(
				'message' => 'errors.group.icon.size',
				'rule' => array('checkSize', '10M'),
			),
			'extension'  => array(
				'message' => 'errors.group.icon.mimetype',
				'rule' => array('checkExtension', false, array(
					'jpg', 'jpeg', 'png', 'gif'
				)),
			),
			'mimeType'   => array(
				'message' => 'errors.group.icon.mimetype',
				'rule' => array('checkMimeType', false, array(
					'image/jpeg', 'image/png', 'image/gif',
				)),
			),
		),
	);

	var $tmp_image = null;

	protected function getThumbnailSizes()
	{
		if (isset($this->data[$this->name]['sp']) && $this->data[$this->name]['sp']) {
			return array(
				'64' => array('fitCrop' => array(64, 64, "topleft")),
				'408' => array('fitInside' => array(408, 96)),
			);
		} else {
			return array(
				'200' => array('fitCrop' => array(200, 200)),
				'100' => array('fitCrop' => array(100, 100)),
				 '50' => array('fitCrop' => array(50, 50)),
				 '20' => array('fitCrop' => array(20, 20)),
			);
		}
	}

	function beforeSave($options = array()) {
		Configure::write('Media.filter', array(
			'image' => $this->getThumbnailSizes(),
		));
		$this->setIconType();
		return true;
	}

	function afterSave($created) {
		parent::afterSave($created);

		$iconImage = new IconImage();

		if (isset($this->data[$this->name]['sp']) && $this->data[$this->name]['sp']) {
			$original_dir = _SP_GROUP_IMAGE_ORIGINAL_DIRECTORY;
			$thumbnail_dir = _SP_GROUP_IMAGE_THUMBNAIL_DIRECTORY;
		} else {
			$original_dir = _GROUP_IMAGE_ORIGINAL_DIRECTORY;
			$thumbnail_dir = _GROUP_IMAGE_THUMBNAIL_DIRECTORY;
		}

		if ($this->tmp_image != '') {
			$file = new File($original_dir.$this->tmp_image);
			$file->copy($original_dir.$this->id);
			$file->delete();
			foreach (array_keys($this->getThumbnailSizes()) as $thumbnail_key) {
				$dir = $thumbnail_dir.$thumbnail_key.DS;
				$file = new File($dir.$this->tmp_image);
				$file->copy($dir.$this->id);
				$file->delete();
			}
		}

		foreach (array_keys($this->getThumbnailSizes()) as $thumbnail_key) {
			$dir = $thumbnail_dir.$thumbnail_key.DS;
			if ($iconImage->updateIconImage($this->id, $thumbnail_key, $dir.$this->id)) {
				$this->log('Error: Icon update fail. group_id = '.$this->id);
			}
		}
	}


	function getData($key) {
		if (isset($this->data['Group'][$key]))
			return $this->data['Group'][$key];
		else
			return NULL;
	}


	/**
	 * グループ名の重複バリデーションを行う。
	 * @param $values
	 */
	function duplicateGroupName($values) {
		if (!$this->duplicateKey($values['name'])) {
			return false;
		}
		if (!$this->duplicateName($values['name'])) {
			return false;
		}
		if (!$this->duplicateNameEn($values['name'])) {
			return false;
		}
		return true;
	}

	/**
	 * グループKEYの重複バリデーションを行う。
	 * @param $values
	 */
	function duplicateGroupKey($values) {
		if (!$this->duplicateKey($values['group_key'])) {
			return false;
		}
		if (!$this->duplicateName($values['group_key'])) {
			return false;
		}
		return true;
	}

	function requireGroupName() {
		if (!($this->getData('name'))
			&& !($this->getData('name_en'))) {
				return false;
			}
		return true;
	}

	function requireIntroduction() {
		if (!($this->getData('introduction'))
			&& !($this->getData('introduction_en'))) {
				return false;
			}
		return true;
	}

	/**
	 * グループ名の重複バリデーションを行う。
	 * @param $values
	 */
	function duplicateGroupNameEn($values) {
		if (!$this->duplicateKey($values['name_en'])) {
			return false;
		}
		if (!$this->duplicateName($values['name_en'])) {
			return false;
		}
		if (!$this->duplicateNameEn($values['name_en'])) {
			return false;
		}
		return true;
	}

	/**
	 * グループKEYの重複バリデーションを行う。
	 * @param $values
	 */
	private function duplicateKey($value) {
		$groupid = $this->getData('id');

		// 新規作成の場合
		if (empty($groupid)) {
			$groupid = -1;
		}

		$inputName = strtoupper($value);
		$inputName = preg_replace("/[_]|[-]|[.]/", "", $inputName);
		// 削除したグループもSPと契約が残っているかもしれないので、同一名は許さない
		$count = $this->find('count', array(
			'conditions' => array(
				"REPLACE(REPLACE(REPLACE(group_key, '_', ''), '-', ''), '.', '')" => $inputName,
				'NOT' => array('id' => $groupid))));
		if ($count >= 1) {
			return false;
		}
		return true;
	}

	/**
	 * グループ名の重複バリデーションを行う。
	 * @param $values
	 */
	private function duplicateName($value) {
		if (empty($value)) {
			return true;
		}

		$groupid = $this->getData('id');
		// 新規作成の場合
		if (empty($groupid)) {
			$groupid = -1;
		}

		$count = $this->find('count', array(
			'conditions' => array(
				'name' => $value, 'NOT' => array('id' => $groupid))));
		if ($count >= 1) {
			return false;
		}
		return true;
	}

	/**
	 * グループ名の重複バリデーションを行う。
	 * @param $values
	 */
	private function duplicateNameEn($value) {
		if (empty($value)) {
			return true;
		}

		$groupid = $this->getData('id');
		// 新規作成の場合
		if (empty($groupid)) {
			$groupid = -1;
		}

		$name = $this->extract($value);
		$count = $this->find('count', array(
			'conditions' => array(
				'name_en_normalization' => $name, 'NOT' => array('id' => $groupid))));
		if ($count >= 1) {
			return false;
		}
		return true;
	}

	private function extract($value) {
		$str = '';
		for ($i = 0; $i < strlen($value); $i++) {
			$c = substr($value, $i, 1);
			if (preg_match("/[a-zA-Z\d]/", $c)) {
				$str = $str.$c;
			}
		}
		return $str;
	}

	function getGroupNameJp($group) {
		if (!empty($group['Group']['name'])) {
			return $group['Group']['name'];
		} else {
			return $group['Group']['name_en'];
		}
	}

	function getGroupNameEn($group) {
		if (!empty($group['Group']['name_en'])) {
			return $group['Group']['name_en'];
		} else {
			return $group['Group']['name'];
		}
	}

	/**
	 * グループ名を取得します。
	 * @param $group
	 */
	function getGroupName($groupid, $lang) {
		$group = $this->find('first', array('conditions' => array('id' => $groupid)));
		if ($lang === 'ja') {
			return $this->getGroupNameJp($group);
		} else {
			return $this->getGroupNameEn($group);
		}
	}

	/**
	 * グループ名一覧を検索します。
	 */
	public function findGroupNameList($groupIdList) {
		$list = $this->find('list', array('fields' => array('id', 'name'),
			'conditions' => array('id' => $groupIdList)));
		return $list;

	}

	/**
	 * グループを検索します
	 */
	function findByGroupKey($groupKey) {
		$group = $this->find('first', array(
				'conditions' => array('group_key' => $groupKey)));
		return $group;
	}

	/**
	 * グループ数を検索します
	 */
	function countByGroupKey($groupKey) {
		$count = $this->find('count', array('conditions' => array(
			'group_key' => $groupKey)));
		return $count;
	}

	/**
	 * グループを登録します
	 */
	function registerGroup($input) {
		$input += array('icon' => 0, 'file' => null);

		$group = array();
		$group['name'] = $input['name'];
		$group['group_key'] = $input['group_key'];
		$group['introduction'] = $input['introduction'];
		$group['public'] = $input['public'];
		$group['openmember'] = $input['openmember'];
		$group['inspectjoin'] = $input['inspectjoin'];
		$group['icon'] = $input['icon'];
		if (is_array($input['file'])) {
			if ($group['icon'] == 0 && isset($input['file']['error']) && $input['file']['error']) {
				$group['icon'] = 1;
			}
			$group['file'] = $input['file'];
		} elseif (!empty($input['file'])) {
			$filepath = $this->getValidTransferTemporaryPath($input['file']);
			if (!is_null($filepath)) {
				$group['file'] = $filepath;
			}
		}
		if ($group['icon'] == 0 && !isset($group['file'])) {
			$group['icon'] = 1;
		}
		if ($group['icon'] != 0) {
			$group['icon_type'] = 'png';
		}
		//inspectquit →いらない

		// デフォルト値を設定
		$group['inspectquit'] = '0'; // 自由に退会
		$group['independence'] = '1'; // 独立させる
		$group['down_permission'] = '0'; // 許可しない
		$group['up_permission'] = '0'; // 許可しない
		$group['sp'] = '0';

		$this->create();
		return $this->save($group);
	}
	private function getValidTransferTemporaryPath($basename) {
		if (!preg_match('/^[a-z0-9]+$/', $basename)) {
			return null;
		}

		$filepath = _GROUP_IMAGE_TEMPORARY_DIRECTORY.$basename;
		return file_exists($filepath) ? $filepath : null;
	}

	function transferTo($via, $from) {
		if (!isset($this->data[$this->alias]['id']) || empty($this->data[$this->alias]['id'])) {
			$this->tmp_image = sha1($this->getRandom().microtime(true));
			return $this->tmp_image;
		} else {
			return $this->data[$this->alias]['id'];
		}
	}

	private function setIconType()
	{
		$is_sp = isset($this->data[$this->name]['sp']) && $this->data[$this->name]['sp'];

		if (isset($this->data[$this->name]['file']) && !is_null($this->data[$this->name]['file'])) {
			$results = getimagesize($this->data[$this->name]['file']);
			list($image, $type) = explode('/', $results['mime']);
			if ($is_sp) {
				$this->data[$this->name]['spc_icon_type'] = $type;
				$this->data[$this->name]['spc_icon'] = null;
			} else {
				$this->data[$this->name]['icon_type'] = $type;
			}
		} else {
			// アイコンを選択した場合はicon_typeを設定し直す
			if ($is_sp) {
				if (isset($this->data[$this->name]['spc_icon'])
					&& $this->data[$this->name]['spc_icon'] != '') {
					$this->data[$this->name]['spc_icon_type'] = 'png';
				}
			} else {
				if (isset($this->data[$this->name]['icon'])
					&& $this->data[$this->name]['icon'] > 0) {
					$this->data[$this->name]['icon_type'] = 'png';
				}
			}
		}
	}

	public function transferTemporary() {
		$group = $this->Behaviors->Transfer->beforeSave($this);
		if (is_null($group['file'])) {
			return null;
		}
		$src_filepath = $group['file'];
		$file = basename($group['file']);
		$dest_filepath = _GROUP_IMAGE_TEMPORARY_DIRECTORY.$file;
		rename($src_filepath, $dest_filepath);
		return $dest_filepath;
	}

	/**
	 * グループIDリストでグループを検索する
	 */
	public function findGroupList($idList) {
		 return $this->find('list', array(
			'fields' => array('id', 'name'), 'conditions' => array('id' => $idList)));
	}

	public function findGroup($id) {
		return $this->find('first', array('conditions' => array('id' => $id)));
	}

	/**
	 * SPのTYPE一覧を取得します。
	 */
	public function findSpcTypeList() {
		return $this->find('list', array(
				'fields' => array('id', 'sp_type'), 'conditions' => array('sp' => 1)));
	}

	/**
	 * アクティブでないグループIDを取得する
	 */
	public function findNotActiveIdList() {
		return $this->find('list', array(
				'fields' => array('id', 'active'), 'conditions' => array('NOT' => array('active' => 0))));
	}

	/**
	 * グループ名を更新します。
	 */
	public function updateGroupName($groupId, $groupName) {
		$entity = $this->find('first', array('conditions' => array('id' => $groupId)));
		if (empty($entity)) {
			return;
		}

		$entity['Group']['modified'] = null;
		$entity['Group']['name'] = $groupName;
		$result = $this->save($entity, false);
	}

	function getIntroductionJp($group) {
		if (!empty($group['Group']['introduction'])) {
			return $group['Group']['introduction'];
		} else {
			return $group['Group']['introduction_en'];
		}
	}

	function getIntroductionEn($group) {
		if (!empty($group['Group']['introduction_en'])) {
			return $group['Group']['introduction_en'];
		} else {
			return $group['Group']['introduction'];
		}
	}

	/**
	 * introductionのリストを取得します。
	 * @param $group
	 */
	function getIntroductions($groupidList, $lang) {
		$groupList = $this->find('all', array(
			'fields' => array('id', 'introduction', 'introduction_en'),
			'conditions' => array('id' => $groupidList)));
		$introductionList = array();
		foreach ($groupList as $group) {
			$id = $group['Group']['id'];
			if ($lang === 'ja') {
				$introductionList[$id] = $this->getIntroductionJp($group);
			} else {
				$introductionList[$id] = $this->getIntroductionEn($group);
			}
		}
		return $introductionList;
	}

	/**
	 * 論理削除します。
	 * @param $groupid
	 */
	public function softDelete($groupid) {
		$group = $this->find('first', array('conditions' => array('id' => $groupid)));
		$group[$this->name]['active'] = 9;
		unset($group[$this->name]['modified']);
		// mAP Coreの登録データは必須項目が埋められていないため、バリデーションをスキップします。
		if ($this->save($group, $validate=false)) {
			$this->log('Group#softDelete successfully soft deleted '.$group[$this->name]['group_key'], "info");
			return true;
		} else {
			$this->log('Group#softDelete failed to delete '.$group[$this->name]['group_key'], "info");
			return false;
		}
	}

	/**
	 * グループ/SPコネクタが活動中であればtrueを返します。
	 * @param $groupid
	 */
	function isActive($groupid) {
		$group = $this->find('first', array(
			'conditions' => array(
				'id' => $groupid,
				'active' => 0)));
		if (empty($group)) {
			return false;
		}
		return true;
	}

	/**
	 * グループ/SPコネクタのアイコンを取得します。
	 * @param $groupid
         * @return アイコンを返す。
	 *         グループ: 0, 1, 2, and etc
	 *         SPコネクタ: NULL, book, calender, and etc
	 */
	function getIcon($groupid) {
		if ($groupid < 1) {
			return null;
		}

		$group = $this->find('first', array(
			'fields' => array('icon', 'spc_icon', 'sp'),
			'conditions' => array('id' => $groupid)));

		$is_sp = isset($group[$this->name]['sp']) && $group[$this->name]['sp'];
		if ($is_sp) {
			return $group[$this->name]['spc_icon'];
		} else {
			return $group[$this->name]['icon'];
		}
	}

	/**
	 * グループかを判定します。
	 *
	 */
	function isGroup($groupid) {
		$entity = $this->find('first', array('conditions' => array('id' => $groupid, 'sp' => 0)));
		if (empty($entity)) {
			return false;
		}
		return true;
	}

	/**
	 * 機関グループかを判定します。
	 *
	 */
	function isIdPGroup($groupid) {
		$autoGroup = new AutoGroup();
		return $autoGroup->isIdPGroup($groupid);
	}

	/**
	 * SPコネクタかを判定します。
	 */
	public function isSpConnector($groupid) {
		$entity = $this->find('first', array('conditions' => array('id' => $groupid, 'sp' => 1)));
		if (empty($entity)) {
			return false;
		}
		return true;
	}

	/**
	 * プライベートサービスかを判定します。
	 *
	 */
	function isPrivateService($groupid) {
		$entity = $this->find('first', array('conditions' => array('id' => $groupid, 'sp' => 2)));
		if (empty($entity)) {
			return false;
		}
		return true;
	}

	/*
	 * グループキーで一致するグループレコードを検索します。
	 *
	 * 機関グループのgroup_key + '/' で group_keyが一致するレコードを検索する。
	 * 複数レコードが返った場合は、idの昇順で返す。
	 */
	public function findGroup_bygroupkey($group_key) {

		$key = $group_key . '/';

		$groups = $this->find('list', array(
			'fields' => array('id', 'group_key'), 
			'conditions' => array(
            			'Group.group_key LIKE' =>  $key . '%'
                	),
			'order' => array('Group.id' => 'asc')
		));
		return $groups;
	}

	/**
	 * SPコネクタ/プライベートサービスのアイコンをロゴとして使用するかを返します
	 * @param $groupid
	 * @return 1: ロゴとして使用する
	 *         0: ロゴとして使用しない
	 */
	function getIconLogo($groupid) {
		$group = $this->find('first', array(
			'fields' => array('icon', 'spc_icon', 'sp', 'spc_icon_logo'),
			'conditions' => array('id' => $groupid)));

		$is_sp = isset($group[$this->name]['sp']) && $group[$this->name]['sp'];
		if ($is_sp && $group[$this->name]['spc_icon'] == NULL) {
			return $group[$this->name]['spc_icon_logo'];
		} else {
			return 0;
		}
	}

}
?>
