<?php
/**
 * IconImage Model
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */

App::import('Core', 'File');

class IconImage extends AppModel{
	var $name = 'IconImage';

	/**
	 * アイコンイメージをデータベースに保存する。
	 *
	 * @param $group_id グループID
	 * @param $size サムネイルサイズ
	 * @param $path ローカルファイルのパス
	 * @return boolean Error
	 */
	function updateIconImage($group_id, $size, $path) {
		if (Configure::read('IconImage.not_use_db') === true) {
			return false;
		}

		$tmp_file = new File($path);
		if (!$tmp_file->exists()) {
			return true;
		}

		$entry = $this->find('first', array(
			'fields' => array(
				'id', 'group_id', 'size', 'last_change'),
			'conditions' => array(
				'group_id' => $group_id,
				'size' => $size)));

		if (is_array($entry)) {
			if ($tmp_file->lastChange()
			    == $entry['IconImage']['last_change']) {
				return false;
			}
		} else {
			$entry = array('IconImage' => array(
				'group_id' => $group_id,
				'size' => (int)$size));
			$this->create();
		}

		$entry['IconImage']['last_change'] = $tmp_file->lastChange();
		$entry['IconImage']['image'] = base64_encode($tmp_file->read());
		return (!is_array($this->save($entry)));
	}

	/**
	 * アイコンイメージをデータベースから取得し、ローカルのファイルに書き出す。
	 *
	 * @param $group_id グループID
	 * @param $size サムネイルサイズ
	 * @param $path ローカルファイルのパス
	 * @return boolean Error
	 */
	function replaceIconImage($group_id, $size, $path) {
		if (Configure::read('IconImage.not_use_db') === true) {
			return false;
		}

		$last_change = $this->field('last_change', array(
			'group_id' => $group_id,
			'size' => (int) $size));
		if ($last_change === false) {
			return false;
		}

		new Folder(dirname($path), true);	// ファイルを書き出すディレクトリが無ければ作成する
		$tmp_file = new File($path);
		if ($tmp_file->exists()
		    && $last_change == $tmp_file->lastChange()) {
			return false;
		}
		$image = $this->field('image', array(
			'group_id' => $group_id,
			'size' => (int) $size));

		return $tmp_file->write(base64_decode($image), 'w', true);
	}

	/**
	 * アイコンイメージがデータベースにあればtrueを返す
	 *
	 * @param $group_id グループID
	 * @param $size サムネイルサイズ
	 * @return boolean 該当アイコンイメージがあればtrue
	 */
	function existIconImage($group_id, $size) {
		if (Configure::read('IconImage.not_use_db') === true) {
			return false;
		}

		$count = $this->find('count', array(
				'conditions' => array('group_id' => $group_id, 'size' => $size)));
		if ($count > 0) {
			return true;
		}
		return false;
	}
}
?>
