<?php
/**
 * InspectAttribute Model
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class InspectAttribute extends AppModel {
    var $name = 'InspectAttribute';

    /**
     * エンティティを生成します。
     * @param $data 入力情報
     */
	function generateEntity($attributeList, $inspectId, $fromGroupid, $toGroupid) {
		$entity = array();
		$entity['inspect_id'] = $inspectId;
		$entity['from_group_id'] = $fromGroupid;
		$entity['to_group_id'] = $toGroupid;
		$entity['ismemberof'] = $attributeList['ismemberof'];
		$entity['eptid'] = $attributeList['eptid'];
		$entity['name'] = $attributeList['name'];
		$entity['mail'] = $attributeList['mail'];
		$entity['eppn'] = $attributeList['eppn'];
		$entity['idp'] = $attributeList['idp'];
		$entity['introduction'] = $attributeList['introduction'];
		$entity['language'] = $attributeList['language'];
		$entity['status'] = 0;
		return $entity;
	}

    /**
     * 空の属性一覧を生成します。
     * @param $data 入力情報
     */
	function getEmptyAttrList() {
		$list = array();
		$list['ismemberof'] = false;
		$list['eptid'] = false;
		$list['name'] = false;
		$list['mail'] = false;
		$list['eppn'] = false;
		$list['idp'] = false;
		$list['introduction'] = false;
		$list['language'] = false;
		return $list;
	}

	/**
	 * 属性をマージする
	 * @param $entity
	 * @param $updEntity
	 */
	function marge($entity, $updEntity) {
		if (!isset($entity['ismemberof']) || !$entity['ismemberof'])
			$entity['ismemberof'] = $updEntity['ismemberof'];

		if (!isset($entity['eptid']) || !$entity['eptid'])
			$entity['eptid'] = $updEntity['eptid'];

		if (!isset($entity['name']) || !$entity['name'])
			$entity['name'] = $updEntity['name'];

		if (!isset($entity['mail']) || !$entity['mail'])
			$entity['mail'] = $updEntity['mail'];

		if (!isset($entity['eppn']) || !$entity['eppn'])
			$entity['eppn'] = $updEntity['eppn'];

		if (!isset($entity['idp']) || !$entity['idp'])
			$entity['idp'] = $updEntity['idp'];

		if (!isset($entity['introduction']) || !$entity['introduction'])
			$entity['introduction'] = $updEntity['introduction'];

		if (!isset($entity['language']) || !$entity['language'])
			$entity['language'] = $updEntity['language'];

		return $entity;
	}

	/**
	 * 指定した接続申請IDに該当する属性情報をモデル名なしで取得します。
	 * @param $inspectId 接続申請ID
	 */
	function findAttributeByInspectId($inspectId) {
		$entity = $this->find('first', array('conditions' => array('inspect_id' => $inspectId)));
		if (empty($entity)) {
			return array();
		}

		$list = array();
		$list['id'] = $entity['InspectAttribute']['id'];
		$list['ismemberof'] = $entity['InspectAttribute']['ismemberof'];
		$list['eptid'] = $entity['InspectAttribute']['eptid'];
		$list['name'] = $entity['InspectAttribute']['name'];
		$list['mail'] = $entity['InspectAttribute']['mail'];
		$list['eppn'] = $entity['InspectAttribute']['eppn'];
		$list['idp'] = $entity['InspectAttribute']['idp'];
		$list['introduction'] = $entity['InspectAttribute']['introduction'];
		$list['language'] = $entity['InspectAttribute']['language'];
		return $list;
	}

	/**
	 * 選択した属性をパスにして返します。
	 * @param unknown_type $spAttributeList
	 */
	function getSelectAttributePath($attributeList) {
		$path = '';
		if (isset($attributeList['ismemberof']) && $attributeList['ismemberof'])
			$path = $path.'/1';
		else
			$path = $path.'/0';

		if (isset($attributeList['eptid']) && $attributeList['eptid'])
			$path = $path.'/1';
		else
			$path = $path.'/0';

		if (isset($attributeList['name']) && $attributeList['name'])
			$path = $path.'/1';
		else
			$path = $path.'/0';

		if (isset($attributeList['mail']) && $attributeList['mail'])
			$path = $path.'/1';
		else
			$path = $path.'/0';

		if (isset($attributeList['eppn']) && $attributeList['eppn'])
			$path = $path.'/1';
		else
			$path = $path.'/0';

		if (isset($attributeList['idp']) && $attributeList['idp'])
			$path = $path.'/1';
		else
			$path = $path.'/0';

		if (isset($attributeList['introduction']) && $attributeList['introduction'])
			$path = $path.'/1';
		else
			$path = $path.'/0';

		if (isset($attributeList['language']) && $attributeList['language'])
			$path = $path.'/1';
		else
			$path = $path.'/0';

		return $path;
	}
}
?>
