<?php
/**
 * Invite Model
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Model','Group');
App::import('Model','Mygroup');

class Invite extends AppModel{
	var $name = 'Invite';

    function __construct() {
        parent::__construct();
 		$this->Group = new Group();
 		$this->Mygroup = new Mygroup();
    }

	function beforeSave($options = array()) {
		return $this->setRandom();
	}

	private function setRandom()
	{
		if (!(isset($this->data[$this->name]['groupid']) && !isset($this->data[$this->name]['random']))) {
			return true;
		}

		for ($i = 0; $i < 10; $i++) {
			$random = $this->getRandom(50);
			if (!$this->existsUniqueKey($this->data[$this->name]['groupid'], $random)) {
				$this->data[$this->name]['random'] = $random;
				return true;
			}
		}
		return false;
	}

	private function existsUniqueKey($groupid, $random)
	{
		$count = $this->find('count', array(
			'conditions' => array(
				'groupid' => $groupid,
				'random' => $random,
			)
		));
		return $count >= 1;
	}

    /**
     * 同じユーザーの情報を取得する
     * @param $groupid
     * @param $address メールアドレス (invitedAccountIdかどちらかが設定されている)
     * @param $invitedAccountId Account.id
     */
	public function getByGroupId_User($groupid, $address, $invitedAccountId) {

		$conditions = array('groupid' => $groupid);
		if (!empty($invitedAccountId)) {
			$conditions += array('invited_account_id' => $invitedAccountId);
		} elseif (!empty($address)) {
			$conditions += array('address' => $address, 'invited_account_id IS NULL');
		} else {
			throw new Exception('condition error');
		}

		return $this->find('all', array(
			'conditions' => $conditions
		));
	}

	/**
	 * 仮アカウントIDとグループIDで検索する
	 */
	public function findByTempAccountIDGroupID($tempAccountid, $groupid) {
		$invite = $this->find('first', array('conditions' => array(
			'temp_account_id' => $tempAccountid, 'groupid' => $groupid)));
		return $invite;
	}

	/**
	 * ステータスを無効にする
	 */
	public function updateInvalid($id) {
		$invite = $this->find('first', array('conditions' => array('id' => $id)));
		$invite['Invite']['status'] = 2;
		$result = $this->save($invite);
		return $result;
	}

	public function findById($id) {
		return $this->find('first', array('conditions' => array('id' => $id)));
	}

	/**
	 * 招待中のデータを検索する
	 */
	public function findInviting($accountid) {
		if (empty($accountid)) {
			return array();
		}

		$mygroupIdList = $this->Mygroup->find('list', array(
				'fields' => array('groupid', 'groupid'),
				'conditions' => array('account_id' => $accountid)));
		$invites = $this->find('all', array(
			'fields' => array('id', 'groupid', 'random', 'address', 'term'),
			'conditions' => array(
					'invited_account_id' => $accountid, 'status' => 0, 'term' => 'onetime',
					'NOT' => array('groupid' => $mygroupIdList)),
			'order' => array('groupid', 'id')));

		$inviteIdList = array();
		$groupIdList = array();
		foreach ($invites as $invite) {
			$groupid = $invite['Invite']['groupid'];
			$inviteIdList[$groupid] = $invite['Invite']['id'];
			array_push($groupIdList, $groupid);
		}

		$groupList = $this->Group->find('list', array('fields' => array('id', 'name'),
				'conditions' => array('id' => $groupIdList, 'active' => 0)));

		$effectiveInviteList = array();
		foreach ($invites as $invite) {
			$inviteId = $invite['Invite']['id'];
			$groupid = $invite['Invite']['groupid'];

			if (!in_array($inviteId, $inviteIdList)) {
				continue;
			}

			if (!isset($groupList[$groupid])) {
				continue;
			}
			$invite['Invite']['group_name'] = $groupList[$groupid];
			$effectiveInviteList[$inviteId] = $invite;
		}

		return $effectiveInviteList;
	}
}
?>