<?php
/**
 * Mygroup Model
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class Mygroup extends AppModel {
	var $name = 'Mygroup';

	/**
	 * グループに参加しているかどうかを判定します。
	 * 
	 * @param $accountid アカウントID
	 * @param $groupid グループID
	 */
	function isMember($accountid, $groupid) {
		$count = $this->find ( 'count', array (
				'conditions' => array (
						'account_id' => $accountid,
						'groupid' => $groupid 
				) 
		) );
		if ($count > 0) {
			return true;
		}
		return false;
	}

	/**
	 * 管理者かどうかを判定します。
	 * 
	 * @param $accountid アカウントID
	 * @param $groupid グループID
	 */
	function isAdmin($accountid, $groupid) {
		$count = $this->find ( 'count', array (
				'conditions' => array (
						'account_id' => $accountid,
						'groupid' => $groupid,
						'admin' => array (
								1,
								2 
						) 
				) 
		) );
		if ($count > 0) {
			return true;
		}
		return false;
	}

	/**
	 * 管理者のみかどうかを判定します。
	 * 
	 * @param $accountid アカウントID
	 * @param $groupid グループID
	 */
	function isAdminOnly($accountid, $groupid) {
		$count = $this->find ( 'count', array (
				'conditions' => array (
						'account_id' => $accountid,
						'groupid' => $groupid,
						'admin' => 1
				) 
		) );
		if ($count > 0) {
			return true;
		}
		return false;
	}

	/**
	 * 検索します。
	 * 
	 * @param $accountid アカウントID
	 * @param $groupid グループID
	 */
	function findByAccountIdAndGroupId($accountid, $groupid) {
		$mygroup = $this->find ( 'first', array (
				'conditions' => array (
						'account_id' => $accountid,
						'groupid' => $groupid 
				) 
		) );
		return $mygroup;
	}

	/**
	 * 削除します
	 */
	function deleteByAccountId($accountid, $groupid) {
		if (empty ( $accountid )) {
			return;
		}
		if (empty ( $groupid )) {
			return;
		}
		$this->deleteAll ( array (
				'groupid' => $groupid,
				'account_id' => $accountid 
		), false );
	}

	/**
	 * お気に入りに登録します。
	 */
	function updateFavoriteOn($accountid, $groupid, $time = null, $updator = 'user') {
		if (is_null ( $time )) {
			$time = time ();
		}
		$mygroup = $this->findByAccountIdAndGroupId ( $accountid, $groupid );
		$mygroup ['Mygroup'] ['favorite'] = date ( "Y-m-d H:i:s", $time );
		$mygroup ['Mygroup'] ['favorite_updator'] = $updator;
		$this->create ();
		$this->save ( $mygroup );
	}

	/**
	 * お気に入りを解除します。
	 */
	function updateFavoriteOff($accountid, $groupid) {
		$mygroup = $this->findByAccountIdAndGroupId ( $accountid, $groupid );
		$mygroup ['Mygroup'] ['favorite'] = NULL;
		$this->save ( $mygroup );
	}

	/**
	 * 自分のグループ(groups.sp=0)のID一覧を取得します。
	 */
	public function findMyGroupIdList($accountid, $admin = array(), $onlyActive = true) {
		if ($accountid === false) {
			return array();
		}
		$conditions = array('account_id' => $accountid);
		if (!empty($admin)) {
			$conditions['admin'] = $admin;
		}
		if ($onlyActive) {
			$conditions['Group.active'] = 0;
		}
		$idList = $this->find('list', array(
				'fields' => array('groupid', 'groupid'),
				'conditions' => $conditions,
				'joins' => array(array(
					'type' => 'INNER', 'table' => 'groups', 'alias' => 'Group',
					'conditions' => array(
						'groupid = Group.id',
						'Group.sp' => 0)))));
		return $idList;
	}

	/**
	 * 自分がメンバー(メンバー+管理者含む)であるグループ(groups.sp=0)のID一覧を取得します。
	 */
	public function findMyGroupJoinedAsMemberIdList($accountid, $onlyActive = true) {
		$admin = array(0, 2);
		$idList = $this->findMyGroupIdList($accountid, $admin, $onlyActive);
		return $idList;
	}

	/**
	 * 自分が管理者(メンバー+管理者含む)であるグループ(groups.sp=0)のID一覧を取得します。
	 */
	public function findMyGroupJoinedAsAdminIdList($accountid, $onlyActive = true) {
		$admin = array(1, 2);
		$idList = $this->findMyGroupIdList($accountid, $admin, $onlyActive);
		return $idList;
	}

	/**
	 * 自分のグループのID一覧をお気に入り順で取得します。
	 */
	public function findGroupIdSortFavarite($accountid) {
		$idList = $this->find ('list', array (
			'fields' => array (
					'Mygroup.groupid',
					'Mygroup.groupid'
			),
			'conditions' => array (
					'Mygroup.account_id' => $accountid
			),
			'order' => array (
					'favorite DESC', 'Group.name ASC'
			),
			'joins' => array(array(
					'type' => 'INNER', 'table' => 'groups', 'alias' => 'Group',
					'conditions' => array(
							'Mygroup.groupid = Group.id',
							'Mygroup.account_id' => $accountid))),
		) );

		return $idList;
	}

	/**
	 * お気に入り時刻の最大値を取得します。
	 */
	public function findMaxFavoriteByUser($accountid) {
		$entity = $this->find ( 'first', array (
				'conditions' => array (
						'account_id' => $accountid,
						'OR' => array (
								'favorite_updator' => 'user',
								'favorite_updator is null' 
						)
				),
				'order' => array (
						'favorite DESC' 
				)
		) );
		if (empty ( $entity )) {
			return '';
		}
		return $entity ['Mygroup'] ['favorite'];
	}

	public function createMygroup($accountid, $groupid) {
		$mygroup = array (
				'account_id' => $accountid,
				'groupid' => $groupid,
				'admin' => 0 
		);

		$this->create ();
		$result = $this->save ( $mygroup, false );
		if (! $result) {
			return false;
		}
		return true;
	}

	/**
	 * 管理者として作成します。
	 * @param unknown $accountid
	 * @param unknown $groupid
	 * @return boolean
	 */
	public function createMygroupAdmin($accountid, $groupid) {
		if (empty($accountid)) {
			return false;
		}
		if (empty($groupid)) {
			return false;
		}

		$mygroup = array (
				'account_id' => $accountid,
				'groupid' => $groupid,
				'admin' => 1
		);
	
		$this->create();
		$result = $this->save($mygroup, false);
		if (! $result) {
			return false;
		}
		return true;
	}

	/**
	 * システム管理者のアカウントID一覧を取得します。
	 */
	public function getSystemAdminAccountIdList() {
		$idList = $this->find('list', array('fields' => array('account_id', 'account_id'),
			'conditions' => array('groupid' => 1)));
		return $idList;
	}

	/**
	 * Mygroupにはあるがまだ、SpcSortに存在しないレコードを検索します。
	 */
	public function findSpcSortNoRecord($userid) {
		$list = $this->find('list', 
			array(
				'fields' => array('id', 'groupid'),
				'conditions' => array(
					'account_id' => $userid,
					'groupid in (select id from groups where sp=0)',
					'groupid not in (select groupid from spc_sorts where account_id=?)' => $userid,
				)
			));
		return $list;
	}

	/**
	 * Mygroupにはあるがまだ、UserSpSortに存在しないレコードを検索します。
	 */
	public function findUserSpSortNoRecord($userid) {
		$list = $this->find('list',
				array(
						'fields' => array('id', 'groupid'),
						'conditions' => array(
								'account_id' => $userid,
								'groupid in (select id from groups where sp=0)',
								'groupid not in (select groupid from user_sp_sorts where account_id=?)' => $userid,
						)
				));
		return $list;
	}
}
?>
