<?php
/**
 * SpGroup Model
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Model', 'Group');

define('_SP_GROUP_IMAGE_ORIGINAL_DIRECTORY', MEDIA_TRANSFER. 'sp_group'. DS .'original'. DS);
define('_SP_GROUP_IMAGE_THUMBNAIL_DIRECTORY', MEDIA_TRANSFER. 'sp_group'. DS .'thumbnails'. DS);

class SpGroup extends Group {
	var $name = 'SpGroup';
	var $useTable = 'groups';

	var $actsAs = array(
		'Media.Transfer' => array(
			'transferDirectory' => _SP_GROUP_IMAGE_ORIGINAL_DIRECTORY
		),
		'Media.Generator' => array(
			'guessExtension' => false,
			'overwrite' => true,
			'baseDirectory' => _SP_GROUP_IMAGE_ORIGINAL_DIRECTORY,
			'filterDirectory' => _SP_GROUP_IMAGE_THUMBNAIL_DIRECTORY,
		),
	);
	var $validate = array(
		'name' => array(
			'minlength' => array(
				'rule' => array('minLength', '1'),
				'required' => true,
				'message' => 'validation.enter',
			),
			'maxlength' => array(
				'rule' => array('maxLength', '100'),
				'message' => 'validation.maxlength',
				'last' => true
			),
			'duplication' => array(
				'rule' => array('duplicateGroupName'),
				'message' => 'validation.duplication.group',
			),
		),
		'group_key' => array(
			'minlength' => array(
				'rule' => array('minLength', '4'),
				'message' => 'validation.minlength',
				'last' => true
			),
			'maxlength' => array(
				'rule' => array('maxLength', '50'),
				'message' => 'validation.maxlength',
                                'on' => 'create',
				'last' => true
			),
			'alphanumeric' => array(
				'rule' => array('custom', '/^[a-zA-Z0-9._-]*$/'),
				'message' => 'validation.alphanumeric',
                                'on' => 'create',
				'last' => true
			),
			'duplication' => array(
				'rule' => array('duplicateGroupKey'),
				'message' => 'validation.duplication.group.key',
			),
		),
		'link_for_idp_administrator' => array(
			'maxlength' => array(
				'rule' => array('maxLength', '1024'),
				'message' => 'validation.maxlength',
				'last' => true
			),
			'website' => array(
				'rule' => array('url', true),
				'allowEmpty' => true,
				'message' => 'validation.url.incorrect.format',
			),
		),
		'introduction' => array(
			'minlength' => array(
				'rule' => array('minLength', '1'),
				'required' => true,
				'allowEmpty' => true,
				'message' => 'validation.enter',
			),
			'maxlength' => array(
				'rule' => array('maxLength', '256'),
				'message' => 'validation.maxlength',
				'last' => true
			),
		),

		'lead_url' => array(
			'rule' => array('maxLength', '200'),
			'message' => 'validation.maxlength',
		),

		'file' => array(
			'resource'   => array('rule' => 'checkResource'),
			'access'     => array('rule' => 'checkAccess'),
			'size'       => array(
				'message' => 'errors.group.icon.size',
				'rule' => array('checkSize', '10M'),
			),
			'extension'  => array(
				'message' => 'errors.group.icon.mimetype',
				'rule' => array('checkExtension', false, array(
					'jpg', 'jpeg', 'png', 'gif'
				)),
			),
			'mimeType'   => array(
				'message' => 'errors.group.icon.mimetype',
				'rule' => array('checkMimeType', false, array(
					'image/jpeg', 'image/png', 'image/gif',
				)),
			),
		),
	);

	function getData($key) {
		return $this->data['SpGroup'][$key];
	}

	/**
	 * SPコネクタを検索します。
	 */
	function findByIdList($idList) {
		$spgroups = $this->find('all', array('conditions' => array('id' => $idList)));
		return $spgroups;
	}

	/**
	 * SPコネクタのアイコン一覧を検索します。
	 */
	function findIconList() {
		$list = $this->find('list', array('fields' => array('id', 'spc_icon'),
			'conditions' => array('sp' => 1)));
		return $list;
	}

	/**
	 * 有効なSPコネクタ名を取得します。
	 */
	function findNameOrderName() {
		$result = $this->find('list', array(
			'fields' => array('id', 'name'),
			'conditions' => array('sp' => 1, 'active' => 0, 'public' => 1),
			'order' => array('name')));
		return $result;
	}

	/**
	 * IdP管理者向けリンクを取得します。
	 */
	function findLinkForIdpAdministrator() {
		$result = $this->find('list', array(
			'fields' => array('id', 'link_for_idp_administrator'),
			'conditions' => array('sp' => 1, 'active' => 0)));
		return $result;
	}

	/**
	 * 渡されたIDからアクティブかつ検索可能なSPコネクタIDに絞り込む
	 * @param $scpid array SPコネクタIDのリスト
	 * @return array アクティブかつ検索可能なSPコネクタIDのリスト
	 */
	public function findActiveIdList($spcIdList) {
		return $this->find('list', array(
				'fields' => array('id', 'id'),
				'conditions' => array('id' => $spcIdList, 'sp' => 1, 'active' => 0, 'public' => 1)));
	}

	/**
	 * SPコネクタがIdPタイプか判定します。
	 * @param $scpid SPコネクタID
	 * @return bool IdPタイプはtrueを返す。それ以外のタイプはfalseを返す。
	 */
	public function isIdPtype($spcid) {
		$entity = $this->find('first', array(
				'conditions' => array('id' => $spcid, 'sp' => 1, 'active' => 0, 'sp_type' => 1)));
		if (empty($entity)) {
			return false;
		}
		return true;
	}
}
?>
