<?php
/**
 * Account Controller Test
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
require_once('app'.DS.'tests'.DS.'cases'.DS.'controllers'.DS.'controllerTestCase.php');
App::import('Model', 'Account');
App::import('Model', 'AccountEppn');


class AccountsControllerTest extends ControllerTestCase  {
	function startCase() {
		parent::startCase();
		$this->Account =& ClassRegistry::init('Account');
		$this->AccountEppn =& ClassRegistry::init('AccountEppn');
	}

	function endCase() {
		parent::endCase();
		unset($this->Account);
		unset($this->AccountEppn);
	}

	function startTest($method) {
		parent::startTest($method);
		$this->deleteData();
	}

	function endTest($method) {
		parent::endTest($method);
		$this->deleteData();

	}

	private function deleteData() {
		$account = $this->Account->find('first', array('conditions' => array('mail' => 'testxxxx@xxxxxxx.xx.xx')));
		if (!empty($account)) {
			$this->Account->delete($account['Account']['id']);
			$accountEppn = $this->AccountEppn->find('first', array(
				'conditions' => array('account_id' => $account['Account']['id'])));
			if (empty($accountEppn)) {
				return;
			}
			$this->AccountEppn->delete($accountEppn['AccountEppn']['id']);
		}
	}

	/**
	 * アカウント画面の表示
	 */
	function testEdit_show() {
		$result = $this->testAction('/accounts/edit', array('return' => 'vars'));
		$this->assertTrue($result['cakeDebug']->data['Account']['name']);
        $this->assertTrue($result['defalutLanguage']);
		$this->assertTrue($result['organizations']);
	}

	/**
	 * アカウントの新規登録
	 */
	function testEdit_save_insert() {
		// 存在しないアカウントIDを設定
		$_SESSION['accountID'] = '9999999999';

		$data = array('Account' => array(
			'name' => 'test name',
			'mail' => 'testxxxx@xxxxxxx.xx.xx',
			'university' => 'test univ',
			'agreement' => 'yes'));
		$result = $this->testAction('/accounts/edit', array(
				'fixturize' => false,
				'data' => $data, 'method' => 'post', 'form' => array('save' => 'save')));

		$account = $this->Account->find('first', array('conditions' => array('mail' => 'testxxxx@xxxxxxx.xx.xx')));
		$this->assertTrue($account);
		$this->assertEqual('test name', $account['Account']['name']);
		$this->assertEqual('testxxxx@xxxxxxx.xx.xx', $account['Account']['mail']);
		$this->assertEqual('test univ', $account['Account']['university']);
		$accountEppn = $this->AccountEppn->find('first', array(
			'conditions' => array('account_id' => $account['Account']['id'])));
		$this->assertTrue($accountEppn);
		$this->assertEqual($account['Account']['id'], $accountEppn['AccountEppn']['account_id']);
	}


	/**
	 * アカウントの新規登録の失敗
	 */
	function testEdit_save_validation_error() {
		$data = array('Account' => array(
			'name' => 'test name',
			'mail' => 'testxxxx@xxxxxxx.xx.xx',
			'university' => 'test univ',
			'agreement' => ''));
		$result = $this->testAction('/accounts/edit',
			array('fixturize' => false, 'return' => 'result',
				'data' => $data, 'method' => 'post',	'form' => array('save' => 'save')));
		$account = $this->Account->find('first', array('conditions' => array('mail' => 'testxxxx@xxxxxxx.xx.xx')));
		$this->assertFalse($account);
	}

	/**
	 * アカウントの更新
	 */
	function testEdit_save_update() {
		$data = array('Account' => array(
			'name' => 'test name',
			'mail' => 'testxxxx@xxxxxxx.xx.xx',
			'university' => 'test univ'));
		$result = $this->testAction('/accounts/edit', array(
				'fixturize' => false,
				'data' => $data, 'method' => 'post', 'form' => array('save' => 'save')));

		$account = $this->Account->find('first', array('conditions' => array('mail' => 'testxxxx@xxxxxxx.xx.xx')));
 		$_SESSION['accountID'] = $account['Account']['id'];

		$data = array('Account' => array(
			'id' => $account['Account']['id'],
			'name' => 'test name update',
			'mail' => 'testxxxx@xxxxxxx.xx.xx',
			'university' => 'test univ update'));
		$result = $this->testAction('/accounts/edit', array(
				'fixturize' => false,
				'data' => $data, 'method' => 'post', 'form' => array('save' => 'save')));
		$account = $this->Account->find('first', array('conditions' => array('mail' => 'testxxxx@xxxxxxx.xx.xx')));
		$this->assertTrue($account);
		$this->assertEqual('test name update', $account['Account']['name']);
		$this->assertEqual('testxxxx@xxxxxxx.xx.xx', $account['Account']['mail']);
		$this->assertEqual('test univ update', $account['Account']['university']);
	}
}
?>
