<?php
/**
 * AgreementInsertShell
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
/**
 * GNMAP-176 ユーザ同意のプロセス検討対応によるデータ投入ツール
 * ・ユーザがSPコネクタに接続しているグループに入会していた場合は、グループIDのみ同意済みに設定する。
 * ・全てのSPコネクタに対してグループID属性を利用属性として設定する
 *
 * 実行例）
 * windows)
 *  $ cd C:\xampp\htdocs\map\cake\console
 *  $ cake -app C:\xampp\htdocs\map\app agreement_insert
 * linux)
 *  $ export TERM=vt100
 *  $ cd /usr/local/map/cake/console
 *  $ ./cake -app /usr/local/map/app agreement_insert
 */
class AgreementInsertShell extends Shell {

	var $uses = array('Agreement', 'AgreementIsmemberof', 'AddGroup', 'Mygroup', 'Group', 'ProvideAttribute');

	function main() {
		echo 'AgreementInsertShell start'."\n";

		// SPコネクタに接続している上位グループを取得
		$topGroupIdList = $this->AddGroup->find('list', array(
			'fields' => array('bottom_id', 'bottom_id'),
			'conditions' => array('bottom' => 0 , 'top' => 1)));

		// SPコネクタに上位グループ経由で接続している下位グループを検索
		$bottomGroupIdList = $this->AddGroup->find('list', array(
			'fields' => array('bottom_id', 'bottom_id'),
			'conditions' => array('top_id' => $topGroupIdList, 'bottom' => 0)));

		// 上位グループに参加している人
		$topMygroupList = $this->Mygroup->find('all', array('conditions' => array('groupid' => $topGroupIdList)));
		// 下位グループに参加している人
		$bottomMygroupList = $this->Mygroup->find('all', array('conditions' => array('groupid' => $bottomGroupIdList)));

		// 上位グループが対応
		foreach ($topMygroupList as $mygroup) {
			// 上位グループID
			$groupid = $mygroup['Mygroup']['groupid'];
			// MygroupのグループIDに紐付いているSPコネクタを同意済みにする
			$spcIdList = $this->AddGroup->find('list', array(
				'fields' => array('top_id', 'top_id'),
				'conditions' => array('bottom_id' => $groupid, 'top' => 1)));
			foreach ($spcIdList as $spcid) {
				// 存在しているグループのみ対象とする
				$count = $this->Group->find('count', array('conditions' => array('id' => $spcid, 'active' => 0)));
				if ($count <= 0) {
					continue;
				}

				// 既に同意していたら何もしない
				$count = $this->Agreement->find('count', array(
					'conditions' => array(
						'account_id' => $mygroup['Mygroup']['account_id'],
						'group_id' => $spcid)));
				if ($count == 0) {
					$agreement = array();
					$agreement['Agreement']['account_id'] = $mygroup['Mygroup']['account_id'];
					$agreement['Agreement']['group_id'] = $spcid;
					$agreement['Agreement']['eptid'] = 0;
					$agreement['Agreement']['name'] = 0;
					$agreement['Agreement']['mail'] = 0;
					$agreement['Agreement']['organization'] = 0;
					$agreement['Agreement']['idp'] = 0;
					$agreement['Agreement']['introduction'] = 0;
					$agreement['Agreement']['language'] = 0;
					$this->Agreement->create();
					$this->Agreement->save($agreement);
				}


				$count = $this->AgreementIsmemberof->find('count', array('conditions' => array(
					'account_id' => $mygroup['Mygroup']['account_id'],
					'spc_id' => $spcid,
					'group_id' => $groupid)));
				// 未登録なら登録
				if ($count == 0) {
					// 上のSPコネクタに紐付いている上位グループを検索
					$topIdList = $this->AddGroup->find('list', array(
						'fields' => array('bottom_id', 'bottom_id'),
						'conditions' => array('top_id' => $spcid, 'top' => 1)));
					// この人が上位グループに入っていたらismemberofを同意済みにする
					$ismemberof = array();
					$ismemberof['AgreementIsmemberof']['account_id'] = $mygroup['Mygroup']['account_id'];
					$ismemberof['AgreementIsmemberof']['spc_id'] = $spcid;
					$ismemberof['AgreementIsmemberof']['group_id'] = $groupid;
					$ismemberof['AgreementIsmemberof']['ismemberof'] = 1;
					$this->AgreementIsmemberof->create();
					$this->AgreementIsmemberof->save($ismemberof);
				}
			}
		}

		// 下位グループが対象
		foreach ($bottomMygroupList as $mygroup) {
			// 下位グループID
			$groupid = $mygroup['Mygroup']['groupid'];
			// 上位グループ
			$upIdList = $this->AddGroup->find('list', array(
				'fields' => array('top_id', 'top_id'),
				'conditions' => array('bottom_id' => $groupid, 'top' => 0)));

			// SPコネクタ
			$spcIdList = $this->AddGroup->find('list', array(
				'fields' => array('top_id', 'top_id'),
				'conditions' => array('bottom_id' => $upIdList, 'top' => 1)));

			foreach ($spcIdList as $spcid) {
				// 存在しているグループのみ対象とする
				$count = $this->Group->find('count', array('conditions' => array('id' => $spcid, 'active' => 0)));
				if ($count <= 0) {
					continue;
				}

				// 既に同意していたら何もしない
				$count = $this->Agreement->find('count', array(
					'conditions' => array(
						'account_id' => $mygroup['Mygroup']['account_id'],
						'group_id' => $spcid)));
				// 未登録なら登録
				if ($count == 0) {
					$agreement = array();
					$agreement['Agreement']['account_id'] = $mygroup['Mygroup']['account_id'];
					$agreement['Agreement']['group_id'] = $spcid;
					$agreement['Agreement']['eptid'] = 0;
					$agreement['Agreement']['name'] = 0;
					$agreement['Agreement']['mail'] = 0;
					$agreement['Agreement']['organization'] = 0;
					$agreement['Agreement']['idp'] = 0;
					$agreement['Agreement']['introduction'] = 0;
					$agreement['Agreement']['language'] = 0;
					$this->Agreement->create();
					$this->Agreement->save($agreement);
				}

				// この下位グループの上位グループを検索
				$wkUpIdList = $this->AddGroup->find('list', array(
					'fields' => array('top_id', 'top_id'),
					'conditions' => array('bottom_id' => $groupid, 'top' => 0)));
				foreach ($wkUpIdList as $topid) {
					// 上位グループがSPコネクタと接続していたらAgreementIsmemberofに登録
					$countCon = $this->AddGroup->find('count', array('conditions' => array(
						'bottom_id' => $topid, 'top_id' => $spcid)));
					if ($countCon > 0) {
						$count = $this->AgreementIsmemberof->find('count', array('conditions' => array(
							'account_id' => $mygroup['Mygroup']['account_id'],
							'spc_id' => $spcid,
							'group_id' => $topid)));
						// 未登録なら登録
						if ($count == 0) {
							// 上のSPコネクタに紐付いている上位グループを検索
							$topIdList = $this->AddGroup->find('list', array(
								'fields' => array('bottom_id', 'bottom_id'),
								'conditions' => array('top_id' => $spcid, 'top' => 1)));
							// この人が上位グループの下位グループに入っていたら
							// ismemberofを同意済みにする
							$ismemberof = array();
							$ismemberof['AgreementIsmemberof']['account_id'] = $mygroup['Mygroup']['account_id'];
							$ismemberof['AgreementIsmemberof']['spc_id'] = $spcid;
							$ismemberof['AgreementIsmemberof']['group_id'] = $topid;
							$ismemberof['AgreementIsmemberof']['ismemberof'] = 1;
							$this->AgreementIsmemberof->create();
							$this->AgreementIsmemberof->save($ismemberof);
						}
					}
				}
			}
		}


		// 全てのSPコネクタに対してismberof属性を提供するように設定する
		$spcList = $this->Group->find('list', array(
			'fields' => array('id', 'id'),
			'conditions' => array('sp' => 1, 'active' => 0)));

		foreach ($spcList as $spcid) {
			$count = $this->ProvideAttribute->find('count', array(
				'conditions' => array('group_id' => $spcid)));
			if ($count > 0) {
				continue;
			}
			$attribute = array();
			$attribute['ProvideAttribute']['group_id'] = $spcid;
			$attribute['ProvideAttribute']['ismemberof'] = 1;
			$attribute['ProvideAttribute']['eptid'] = 0;
			$attribute['ProvideAttribute']['name'] = 0;
			$attribute['ProvideAttribute']['mail'] = 0;
			$attribute['ProvideAttribute']['organization'] = 0;
			$attribute['ProvideAttribute']['idp'] = 0;
			$attribute['ProvideAttribute']['introduction'] = 0;
			$attribute['ProvideAttribute']['language'] = 0;
			$this->ProvideAttribute->create();
			$this->ProvideAttribute->save($attribute);
		}


		echo 'AgreementInsertShell end'."\n";
	}
}
?>