<?php
/**
 * kyoto-u.ac.jp Import Shell
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
/**
 * 京都大学のアカウント情報(教授、准教授、助教授、秘書等)を取込む
 *
 * 実行例）
 * windows)
 *  $ cd C:\xampp\htdocs\map\cake\console
 *  $ cake -app C:\xampp\htdocs\map\app kyoto_u_ac_jp_import C:\xampp\htdocs\map\app\tmp\csv\kyoto-u.ac.jp.sample.tsv https://authidp1.iimc.kyoto-u.ac.jp/idp/shibboleth
 * linux)
 *  $ export TERM=vt100
 *  $ cd /usr/local/map/cake/console
 *  $ ./cake -app /usr/local/map/app kyoto_u_ac_jp_import /usr/local/map/app/tmp/csv/kyoto-u.ac.jp.sample.tsv https://authidp1.iimc.kyoto-u.ac.jp/idp/shibboleth
      ./cake -app /usr/local/map/app kyoto_u_ac_jp_import ${TSV_TMPFILE} ${entityId}
 */
class KyotoUAcJpImportShell extends Shell {

	var $scope = 'kyoto-u.ac.jp';
	// デフォルトのエンティティIDを設定(パラメータで指定されていればそちらを利用する)
	var $entityid = 'https://authidp1.iimc.kyoto-u.ac.jp/idp/shibboleth';
	var $uses = array('Account', 'AccountEppn', 'Mygroup', 'Transaction');
	var $name = 'KyotoUAcJpImportShell';

	function main() {
		$this->log('KyotoUAcJpImportShell start', "batch");
		print('IMPORT TSV');
		print(PHP_EOL);
		ini_set('memory_limit', '512M');


		$this->Transaction->begin();
		try {
			if (count($this->args) == 0) {
				$this->log('KyotoUAcJpImportShell end parameter is wrong.', "batch");
				return;
			}
			$filePath = $this->args[0];
			if (isset($this->args[1]) && !empty($this->args[1])) {
				$this->entityid = $this->args[1];
			}
			if (empty($filePath)) {
				$this->log('KyotoUAcJpImportShell end file parameter is wrong.', "batch");
				return;
			}
			if (!file_exists($filePath)) {
				$this->log('KyotoUAcJpImportShell end file not exists.', "batch");
				return;
			}
			$this->prepare();
			$memberList = $this->read($filePath);
			$this->reflect($memberList);
			$this->Transaction->commit();
		} catch (Exception $e) {
			$this->log('KyotoUAcJpImportShell ERROR occurred. '.$e->getMessage(), "batch");
			$this->Transaction->rollback();
		}
		$this->log('KyotoUAcJpImportShell end', "batch");
	}

	private function prepare() {
		try {
			$this->Account->updateAll(
				array('import' => "'init'"), array('import' => 'finish')
			);
		} catch (Exception $e) {
			throw new Exception('prepare() SQL Error.');
		}
	}

	private function read($filePath) {
		$memberList = array();
		$lines = preg_split('/\r\n|\r|\n/', file_get_contents($filePath));
		foreach ($lines as $line) {
			$line = rtrim($line);
			if ($line === '') {
				continue;
			}
			$values = explode("\t", $line);
			if (count($values) != 3) {
				continue;
			}
			$memberList[] = array(
				'local_id' => $values[0],
				'eppn' => $values[1],
				'name' => $values[2],
			);
		}
		return $memberList;
	}

	private function reflect($memberList) {
		$insertCount = 0;
		$updateCount = 0;
		$i = 0;
		try {
			foreach($memberList as $member) {
				$i++;
				$this->log($i.'  '.$member['eppn'], "batch");
				print($i.':'.$member['eppn']);
				print(PHP_EOL);
				// eppnの存在チェック
				$accountEppn = $this->findAccountEppn($member['eppn']);
				if (empty($accountEppn)) {
					// 新規登録
					$this->create($member);
					$insertCount++;
				} else {
					// データ更新
					$this->update($member, $accountEppn);
					$updateCount++;
				}
			}
			sleep(10);
			// 削除
			$this->delete();

			$this->log('KyotoUAcJpImportShell INSERT COUNT='.$insertCount, "batch");
			$this->log('KyotoUAcJpImportShell UPDATE COUNT='.$updateCount, "batch");
			print('INSERT COUNT='.$insertCount);
			print(PHP_EOL);
			print('UPDATE COUNT='.$updateCount);
			print(PHP_EOL);
		} catch (Exception $es) {
			$this->log($es, "batch");
			throw new Exception('reflect()  SQL Error.');
		}
	}

	private function findAccountEppn($eppn) {
		$this->AccountEppn->cacheQueries = false;
		return $this->AccountEppn->find('first', array(
			'conditions' => array('eppn' => $eppn)));
	}

	private function create($member) {
		// Account
		$account = array();
		$account['Account']['name'] = $member['name'];
		$account['Account']['university'] = $this->scope;
		$account['Account']['university_en'] = $this->scope;
		$account['Account']['language'] = "ja";
		$account['Account']['agreement'] = "yes";
		$account['Account']['import'] = "finish";
		$this->Account->create();
		$result = $this->Account->save($account);
		if (empty($result)) {
			print("accounts table insert error");
			print(PHP_EOL);
			$message = $this->Account->lastError();
			debug($message);
			print(PHP_EOL);
			throw new Exception('SQL Error.');
		}

		// AccountEppn
		$acEppn = array();
		$accountId = $this->Account->getLastInsertID();
		$acEppn['AccountEppn']['account_id'] = $accountId;
		$acEppn['AccountEppn']['eppn'] = $member['eppn'];
		$acEppn['AccountEppn']['entityid'] = $this->entityid;
		$acEppn['AccountEppn']['local_id'] = $member['local_id'];
		$this->AccountEppn->create();
		$result = $this->AccountEppn->save($acEppn);
		if (empty($result)) {
			print("account_eppns table insert error");
			print(PHP_EOL);
			throw new Exception('SQL Error.');
		}
	}

	private function update($member, $accountEppn) {
//debug($accountEppn);
		$account = array();
		$account['Account']['id'] = $accountEppn['AccountEppn']['account_id'];
		$account['Account']['name'] = $member['name'];
		$account['Account']['import'] = "finish";
//debug($account);
		$result = $this->Account->save($account);
		if (empty($result)) {
			print("accounts table update error");
			print(PHP_EOL);
			throw new Exception('SQL Error.');
		}

		$accountEppn['AccountEppn']['local_id'] = $member['local_id'];
		$result = $this->AccountEppn->save($accountEppn);
		if (empty($result)) {
			print("account_eppns table update error");
			print(PHP_EOL);
			throw new Exception('SQL Error.');
		}
	}

	private function delete() {
		$this->Account->cacheQueries = false;
		$idList = $this->Account->find("list", array(
			"fields" => array("id"),
			"conditions" => array("import" => "init")
		));

		// account
		$this->Account->deleteAll(array('id' => $idList));

		// account_eppn
		$this->AccountEppn->deleteAll(array('account_id' => $idList));

		// mygroup
		$this->Mygroup->deleteAll(array('account_id' => $idList));

		$this->log('KyotoUAcJpImportShell DELETE COUNT='.count($idList) , "batch");
		print('DELETE COUNT='.count($idList));
		print(PHP_EOL);
	}
}
?>
