<?php
/**
 * SpConnectorCreatorShell
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Core', 'Xml');
App::import('Core', 'HttpSocket');
App::import('Shell', 'Metadata');

/**
 * sp_hostsテーブルを元にSPコネクタを生成します。
 * IdPグループに利用できるSPコネクタを接続します。
 *
 * 注意）
 * IdpGroupCreatorShellを先に実行してください。
 * sp_hostsテーブルへのデータ反映は sp_host.php で行っています。
 * 
 * 
 * ★php.iniに設定追加    httpsでJSONを取得できるようにするため
 * 下記ファイルの最終行に下記を追加する
 * Windows)
 * extension=php_openssl.dll
 * Linux)
 * extension=php_openssl.so
 *
 * 実行例）
 * windows)
 *  前提: E:\xampp\phpにPATHを通す
 *  $ cd E:\xampp\htdocs\map\cake\console
 *  $ cake -app E:\xampp\htdocs\map\app sp_connector_creator E:\xampp\htdocs\map\app\tmp\metadata\gakunin-test-metadata.xml
 *  $ cake -app E:\xampp\htdocs\map\app sp_connector_creator E:\xampp\htdocs\map\app\tmp\metadata\gakunin-metadata.xml
 *  $ cake -app E:\xampp\htdocs\map\app sp_connector_creator E:\xampp\htdocs\map\app\tmp\metadata\federation-metadata.xml
 * linux)
 *  $ export TERM=vt100
 *  $ cd /usr/local/map/cake/console
 *  $ ./cake -app /usr/local/map/app sp_connector_creator /etc/shibboleth/metadata/federation-metadata.xml
 *  $ ./cake -app /usr/local/map/app sp_connector_creator /etc/shibboleth/metadata/federation-metadata.xml
 */
class SpConnectorCreatorShell extends MetadataShell {

	var $uses = array('SpHost', 'GroupSphost', 'AutoSpGroup', 'Transaction',
		'SpAdministrator', 'Mygroup', 'AccountEppn', 'AutoGroup', 'AddGroup',
		'AddGroupsSpcLog');

	function main() {
		$this->log('SpConnectorCreatorShell start', "batch");

		// キャッシュを無効化
		$this->SpHost->cacheQueries = false;
		$this->GroupSphost->cacheQueries = false;
		$this->AutoSpGroup->cacheQueries = false;
		$this->SpAdministrator->cacheQueries = false;
		$this->Mygroup->cacheQueries = false;
		$this->AccountEppn->cacheQueries = false;
		$this->AutoGroup->cacheQueries = false;
		$this->AddGroup->cacheQueries = false;

		$metaPath = $this->args[0];
		if (empty($metaPath)) {
			$this->log('SpConnectorCreatorShell ERROR  There are no parameters. ', "batch");
			$this->log('SpConnectorCreatorShell end', "batch");
			var_dump('There are no parameters.');
			return;
		}

		try{
			$this->Transaction->begin();
			// SpHost
			$today = date("Y-m-d")."T".date("H:i:s").'Z';
			$spHostList = $this->SpHost->getEffectiveAllList($today);
			// GroupSphost
			$usedSpIdList = $this->GroupSphost->getSpIdList();

			// sp_hosts.idがgroup_sphosts.sp_idにあれば、既にSPコネクタが生成されている
			foreach($spHostList as $spHost) {
				$spid = $spHost['SpHost']['id'];
				if (isset($usedSpIdList[ $spid ])) {
					// 既にSPコネクタがある
					$this->update($spHost);
					continue;
				}

				// 新規登録
				// SPコネクタの作成
				$this->create($spHost);

				// 削除
				$this->delete($spHost);
			}

			// SPコネクタとIdPグループの自動接続
			$result = $this->connectSpConnectorToIdPGroup($metaPath);
			if (!$result) {
				throw new Exception('connectSpConnectorToIdPGroup() failure.');
			}

			$this->Transaction->commit();
		} catch (Exception $e) {
			$this->Transaction->rollback();
			var_dump($e->getMessage());
			$this->log('SpConnectorCreatorShell ERROR occurred. '.$e->getMessage(), "batch");
		}
		$this->log('SpConnectorCreatorShell end', "batch");
	}

	/**
	 * 新規でSPコネクタを作成する
	 */
	private function create($spHost) {
		// サービス名
		$serviceName = $spHost['SpHost']['service_name'];
		// SPID
		$spid = $spHost['SpHost']['id'];
		// エンティティID
		$entityid = $spHost['SpHost']['entityid'];
		// description
		$description = $spHost['SpHost']['description'];

		$this->log('SpConnectorCreatorShell create entityid='.$entityid, "batch");

		// SPコネクタの管理者の存在確認
		$spAdminEppnList = $this->SpAdministrator->getEppnList($entityid);
		if (!empty($spAdminEppnList)) {
			$accountidList = $this->AccountEppn->findAccountIdListByEppn($spAdminEppnList);
		} else {
			$this->log('SpConnectorCreatorShell Administrator of SP does not exist.  entityid=['.$entityid . ']', "batch");
			// SPコネクタの管理者がいないので、システム管理者がSPコネクタの管理者となる
			$accountidList = $this->Mygroup->getSystemAdminAccountIdList();
			if (empty($accountidList)) {
				return;
			}
		}
		if (empty($accountidList)) {
			$this->log('SpConnectorCreatorShell accountID does not exist.', "batch");
			return;
		}

		// spconnector(groups) create
		$groupid = $this->AutoSpGroup->createAutoSpConnector($serviceName, $entityid, $description);
		if (empty($groupid)) {
			throw new Exception('AutoSpGroup could not be made.');
		}

		// group_sphosts
		// SPのURLはわからないので、エンティティIDの一部を利用する
		$spUrl = "";
		$pattern = '/((?:https?|http):\/\/[-_.!~*\'()a-zA-Z0-9;?:@&=+$,%#]+)/';
		preg_match($pattern, $entityid, $matches);
		if (isset($matches[0])) {
			$spUrl = $matches[0];
		}

		// 作成
		$result = $this->GroupSphost->createAuto($groupid, $spid, $serviceName, $spUrl);
		if (empty($result)) {
			$this->log("createAutoSpConnector#create  groupid:$groupid, spid:$spid, serviceName:$serviceName, spUrl:".$spUrl, "batch");
			if (!empty($this->GroupSphost->validationErrors)) {
				$this->log("createAutoSpConnector#create  createAuto validationErrors=" . var_export($this->GroupSphost->validationErrors, true), "batch");
			}
			throw new Exception('A GroupSphost could not be made.');
		}

		// mygroups
		foreach ($accountidList as $accountid) {
			$result = $this->Mygroup->createMygroupAdmin($accountid, $groupid);
			if (empty($result)) {
				throw new Exception('A Mygroup could not be made.');
			}
		}

//		var_dump("created");
		$this->log('SpConnectorCreatorShell created', "batch");
		
	}

	/**
	 * このshellで自動作成したSPコネクタ等の情報(groups,group_sphosts,mygroups)を更新する
	 * @param unknown $spHost
	 */
	private function update($spHost) {
		$spid = $spHost['SpHost']['id'];
		$entityid = $spHost['SpHost']['entityid'];
		$spcid = $this->GroupSphost->getSpcId($spid);

		// descriptionが登録されていなかったら、更新する
		$description = $spHost['SpHost']['description'];
		if (empty($description)) {
			$description = $spHost['SpHost']['service_name'];
		}
		if (empty($description)) {
			$description = $spHost['SpHost']['service_name_en'];
		}
		if (empty($description)) {
			$description = $spHost['SpHost']['entityid'];
		}
		
		$this->AutoSpGroup->updateDescriptionGroupKeyForAuto($spcid, $entityid, $description);

		// 手動でレコードを変更することもあるので、今のところ、他の更新処理はしないことにする
		return;
	}

	/**
	 * このshellで自動作成したSPコネクタ等の情報(groups,group_sphosts,mygroups)を削除する
	 * @param unknown $spHost
	 */
	private function delete($spHost) {
		// 手動でレコードを変更することもあり、SPが削除されることはあまりないので、削除処理はしないことにする
		return;
	}

	/**
	 * SPコネクタごとに利用OKにしているIdPを調べ、このIdPグループと自動で接続する
	 * IdPグループが申請システムの情報で利用できなくなっていたら、接続を解除する
	 * 自動接続したことをレコードの持つようにして、この印がある場合のみ接続解除にする
	 * 
	 * 情報の取得URL
	 * https://office.gakunin.nii.ac.jp/ProdFed/export/discofeed/<申請書ベースID>
	 * <申請書ベースID>にはSPの「EntityDescriptor ID」を指定する
	 * 例）
	 * https://office.gakunin.nii.ac.jp/ProdFed/export/discofeed/PS0048JP
	 */
	private function connectSpConnectorToIdPGroup($metaPath) {
		$metadata = $this->metadataToXml($metaPath);
		if (!$metadata) {
			$this->log('SpConnectorCreatorShell ERROR  Metadata is empty. ', "batch");
			$this->log('SpConnectorCreatorShell end', "batch");
			return false;
		}

		$spApplicationIdList = array();
		$xmlAsArray = $metadata['xmlAsArray'];
		//SPの申請書ベースIDを取得
		foreach ($xmlAsArray['EntitiesDescriptor']['EntityDescriptor'] as $entity) {
			if (empty($entity['SPSSODescriptor'])) {
				continue;
			}
			$spApplicationIdList[ $entity['entityID'] ] = $entity['ID'];
		}
		//var_dump($spApplicationIdList);

		foreach ($spApplicationIdList as $entityId => $spApplicationId) {
//debug($entityId.' = '.$spApplicationId);
			// SPIDを取得
			$spId = $this->SpHost->getSpId($entityId);
//debug('spid='.$spId);
			if (!$spId) {
				continue;
			}
			// 紐づいているSPコネクタIDを取得
			$spcIdList = $this->GroupSphost->getSpcIdList($spId);
			if (empty($spcIdList)) {
				continue;
			}
//debug($spcIdList);
			// 有効なSPコネクタID一覧を取得
			$activeSpcIdList = $this->AutoSpGroup->findActiveIdList($spcIdList);
			foreach ($activeSpcIdList as $spcId) {
//debug('spcid='.$spcId);
				$useIdpEntityIdList = $this->requestDiscofeed($spApplicationId);
				if (empty($useIdpEntityIdList)) {
					continue;
				}
	//debug($useIdpEntityIdList);
				
				// SPコネクタとIdPグループを自動接続・自動切断する
				foreach($useIdpEntityIdList as $idpEntityId) {
					// IdPグループを検索
					$autoGroup = $this->AutoGroup->findIdPGroup($idpEntityId);
					if (empty($autoGroup)) {
						continue;
					}
					$idpGroupId = $autoGroup['AutoGroup']['group_id'];
	//debug("idpGroupId=".$idpGroupId);
					// 既に接続していたら何もしない
					$isConnect = $this->AddGroup->isDirectConnectSPConnetor($spcId, $idpGroupId);
					if ($isConnect) {
						continue;
					}
					// 手動解除された記録があれば、自動接続しない
					$count = $this->AddGroupsSpcLog->find('count', array('conditions' =>array(
						'kind' => 0, 'group_id' => $idpGroupId, 'spc_id' => $spcId, 'auto' => NULL)));
					if ($count > 0) {
						$this->log("SpConnectorCreatorShell  The connection has been manually disconnected. spcId:".$spcId. "  idpGroupId:".$idpGroupId, "batch");
						continue;
					}
					// SPコネクタとIdPグループを接続
	//debug("AddGroup#saveAuto  spcId:".$spcId. "  idpGroupId:".$idpGroupId);
					$result = $this->AddGroup->saveAuto($spcId, $idpGroupId);
					if (!$result) {
						debug("ERROR  AddGroup save error. spcId:".$spcId. "  idpGroupId:".$idpGroupId);
						$this->log("ERROR  AddGroup save error. spcId:".$spcId. "  idpGroupId:".$idpGroupId, "batch");
						return false;
					}
				}
				
				// 接続しているが、今回から利用しなくなったSPコネクタとIdPグループを切断する
				$connectedIdpGroupIdList = $this->AddGroup->findIdPGroupIdList($spcId);
	//debug($connectedIdpGroupIdList);
				foreach ($connectedIdpGroupIdList as $groupId) {
					$idpGroupEntityId = $this->AutoGroup->findEntityIdForIdpGroup($groupId);
					if (!$idpGroupEntityId) {
						continue;
					}
					if (!in_array($idpGroupEntityId, $useIdpEntityIdList)) {
						// 削除
	//debug("delete idpGroupEntityId=".$idpGroupEntityId.' spcId ='.$spcId.' groupid='.$groupId);
						$result = $this->AddGroup->deleteIdPGroup($spcId, $groupId);
						if (!$result) {
							debug("ERROR  AddGroup delete error. spcId:".$spcId. "  groupId:".$groupId);
							$this->log("ERROR  AddGroup delete error. spcId:".$spcId. "  groupId:".$groupId, "batch");
							return false;
						}
					}
				}
			}
		}
		return true;
	}

	/**
	 * SPに対して利用OKなIdPのエンティティID一覧を取得
	 */
	private function requestDiscofeed($spApplicationId) {
		$url = Configure::read('discofeed.url');
		if (empty($url)) {
			$this->log("ERROR  Failed to read discofeed.url.", "batch");
			return array();
		}
		$url = $url . $spApplicationId;

		$socket = new HttpSocket();
		$json = $socket->get($url);
		if (empty($json)) {
			$this->log("ERROR  Failed to fetch " . $url . ".", "batch");
			return array();
		}

		$useIdpList = json_decode($json, true);
		if (empty($useIdpList)) {
			$this->log("ERROR  Failed to decode data fetched from " . $url . ".", "batch");
			return array();
		}

		$useIdPEntityIdList = array();
		foreach($useIdpList as $useIdp) {
// if ($useIdp['entityID'] === 'https://idp.imc.tut.ac.jp/idp/shibboleth') {
// 	if ($spApplicationId === 'PS0007JP') {
// debug("delete");
// 		continue;
// 	}
// }
			array_push($useIdPEntityIdList, $useIdp['entityID']);
		}
		return $useIdPEntityIdList;
	}
}
?>
