<?php
/**
 * SpHostShell
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Core', 'Xml');
App::import('Shell', 'Metadata');

/**
 * メタデータを取得しSP情報を sp_hostsテーブルへ反映します。
 *
 * 実行例）
 * windows)
 *  前提: E:\xampp\phpにPATHを通す
 *  $ cd E:\xampp\htdocs\map\cake\console
 *  $ cake -app E:\xampp\htdocs\map\app sp_host E:\xampp\htdocs\map\app\tmp\metadata\gakunin-test-metadata.xml
 *  $ cake -app E:\xampp\htdocs\map\app sp_host E:\xampp\htdocs\map\app\tmp\metadata\gakunin-metadata.xml
 *  $ cake -app E:\xampp\htdocs\map\app sp_host E:\xampp\htdocs\map\app\tmp\metadata\federation-metadata.xml
 * linux)
 *  $ export TERM=vt100
 *  $ cd /usr/local/map/cake/console
 *  $ ./cake -app /usr/local/map/app sp_host /etc/shibboleth/metadata/federation-metadata.xml
 */
class SpHostShell extends MetadataShell {

	var $uses = array('SpHost');

	function main() {
		$this->log('SpHostShell start', "batch");

		try{
			$metaPath = $this->args[0];
			$metadata = $this->metadataToXml($metaPath);
			if (!$metadata) {
				$this->log('SpHostShell end', "batch");
				return;
			}

			$validUntil = $metadata['validUntil'];
			$xmlAsArray = $metadata['xmlAsArray'];
			$serviceList = array();
			$i = 0;
			foreach ($xmlAsArray['EntitiesDescriptor']['EntityDescriptor'] as $entity) {
				if (!empty($entity['SPSSODescriptor'])) {
// 					if (!isset($entity['Organization']['OrganizationDisplayName'])
// 						&& !isset($entity['SPSSODescriptor']['AttributeConsumingService']['ServiceName'])
// 						&& !isset($entity['SPSSODescriptor']['Extensions']['UIInfo']['DisplayName'])
// 						&& !isset($entity['SPSSODescriptor']['KeyDescriptor']['KeyInfo']['X509Data']['X509Certificate'])) {
// 						continue;
// 					}
// 					if (!isset($entity['SPSSODescriptor']['KeyDescriptor']['KeyInfo']['X509Data']['X509Certificate'])) {
// 						continue;
// 					}

					$entityID = $entity['entityID'];
					preg_match("/^.+:\/\/([^\/|:]+)/", $entityID, $match);

					$names = array();
					// DisplayName > ServiceName > OrganizationDisplayName
					if (isset($entity['SPSSODescriptor']['Extensions']['UIInfo']['DisplayName'])) {
						$names = $entity['SPSSODescriptor']['Extensions']['UIInfo']['DisplayName'];
					} else if (isset($entity['SPSSODescriptor']['AttributeConsumingService']['ServiceName'])) {
						$names = $entity['SPSSODescriptor']['AttributeConsumingService']['ServiceName'];
					} else if (isset($entity['Organization']['OrganizationDisplayName'])) {
						$names = $entity['Organization']['OrganizationDisplayName'];
					}

					$serviceList[$i]['ja'] = '';
					$serviceList[$i]['en'] = '';
					$serviceList[$i]['entityID'] = $entityID;
//					$serviceList[$i]['name'] = $match[1];
					$hostName = "";
					if (isset($match[1]) ) {
						$hostName = $match[1];
					} else {
						$hostName = $entityID;
					}
					$serviceList[$i]['name'] = $hostName;
					if (isset($names[0])) {
						foreach($names as $name) {
							if (isset($name['xml:lang']) && $name['xml:lang'] == 'ja') {
								$serviceList[$i]['ja'] = $name['value'];
							} else if (isset($name['xml:lang']) && $name['xml:lang'] == 'en') {
								$serviceList[$i]['en'] = $name['value'];
							}
						}
					} else {
						if (isset($names['xml:lang']) && $names['xml:lang'] == 'ja') {
							$serviceList[$i]['ja'] = $names['value'];
						} else if (isset($names['xml:lang']) && $names['xml:lang'] == 'en') {
							$serviceList[$i]['en'] = $names['value'];
						}
					}
					if ($serviceList[$i]['ja'] == '') {
						$serviceList[$i]['ja'] = $serviceList[$i]['en'];
					}

					// set X509Certificate
					if (!isset($entity['SPSSODescriptor']['KeyDescriptor'])) {
						// DO NOTHING
					} elseif (!isset($entity['SPSSODescriptor']['KeyDescriptor'][1])) {
						$serviceList[$i]['certificate'] = $this->format($entity['SPSSODescriptor']['KeyDescriptor']['KeyInfo']['X509Data']['X509Certificate']);
						$serviceList[$i]['certificate2'] = null;
					} else {
						$serviceList[$i]['certificate'] = $this->format($entity['SPSSODescriptor']['KeyDescriptor'][0]['KeyInfo']['X509Data']['X509Certificate']);
						$serviceList[$i]['certificate2'] = $this->format($entity['SPSSODescriptor']['KeyDescriptor'][1]['KeyInfo']['X509Data']['X509Certificate']);
					}

					// 説明を設定
					if (isset($entity['SPSSODescriptor']['Extensions']['UIInfo']['Description']['value'])) {
						$description = $entity['SPSSODescriptor']['Extensions']['UIInfo']['Description']['value'];
						if (!empty($description)) {
							$serviceList[$i]['description'] = trim($description);
						}
					}
					if (isset($entity['SPSSODescriptor']['AttributeConsumingService']['ServiceDescription']['value'])) {
						$serviceDescription = $entity['SPSSODescriptor']['AttributeConsumingService']['ServiceDescription']['value'];
						if (!empty($serviceDescription)) {
							if (!isset($serviceList[$i]['description'])) {
								$serviceList[$i]['description'] = trim($serviceDescription);
							}
						}
					}
					$i++;
				}
			}

			$saveNameList = array();
			foreach ($serviceList as $service) {
				$spHost = array();
				$spHost = $this->SpHost->find('first', array('fields' => array('id'),
					'conditions' => array('entityid' => $service['entityID'])));
				$spHost['SpHost']['name'] = $service['name'];
				$spHost['SpHost']['service_name'] = $service['ja'];
				$spHost['SpHost']['service_name_en'] = $service['en'];
				$spHost['SpHost']['entityid'] = $service['entityID'];
				$spHost['SpHost']['status'] = 0;
				$spHost['SpHost']['valid_util'] = $validUntil;
				if (isset($service['certificate']) || isset($service['certificate2'])) {
					$spHost['SpHost']['certificate'] = $service['certificate'];
					$spHost['SpHost']['certificate2'] = $service['certificate2'];
				}
				if (isset($service['description'])) {
					$description = substr($service['description'], 0, 256);
					$spHost['SpHost']['description'] = $description;
				}
				$this->SpHost->create();
				$this->SpHost->save($spHost);
			}
		} catch (Exception $e) {
			$this->log('SpHostShell ERROR occurred. '.$e->getMessage(), "batch");
		}
		$this->log('SpHostShell end', "batch");
	}

	private function format($cert) {
		$cert = str_replace(array(" ", "\r\n","\r","\n"), '', $cert);
		return $cert;
	}

	private function sha2($cert) {
		$cert = preg_replace("/\n/", '', $cert);
		$sha2 = hash_hmac('sha256', $cert, Configure::read("Security.salt"));
		return $sha2;
	}
}
?>
